@@ -25,9 +25,11 @@ import distro_info
 import glob
 import logging
 import os
+import pwd
 import re
 import hashlib
 import subprocess
+import pathlib
 from subprocess import PIPE, Popen
 
 from .utils import get_arch
@@ -55,6 +57,7 @@ class DistUpgradeQuirks(object):
         self._snap_list = None
         self._from_version = None
         self._to_version = None
+        self._did_change_font = False
 
     # the quirk function have the name:
     #  $Name (e.g. PostUpgrade)
@@ -109,17 +112,22 @@ class DistUpgradeQuirks(object):
     def PreCacheOpen(self):
         """ run before the apt cache is opened the first time """
         logging.debug("running Quirks.PreCacheOpen")
-        self._add_apport_ignore_tracker_extract()
+        self._add_apport_ignore_list()
 
     # individual quirks handler that run *after* the cache is opened
-    def lunarPostInitialUpdate(self):
+    def noblePostInitialUpdate(self):
         # PreCacheOpen would be better but controller.abort fails terribly
         """ run after the apt cache is opened the first time """
-        logging.debug("running Quirks.lunarPostInitialUpdate")
+        logging.debug("running Quirks.noblePostInitialUpdate")
         self._get_from_and_to_version()
         self._test_and_fail_on_i386()
         self._test_and_fail_on_aufs()
         self._test_and_fail_on_power8()
+        self._test_and_fail_on_armhf_raspi()
+        self._test_and_fail_on_tpm_fde()
+        self._set_grub_cloud_style_installation_on_cloud_installs()
+        self._test_and_fail_on_unmerged_usr()
+        self._test_and_fail_rabbitmq_server_installed()
 
         cache = self.controller.cache
         self._test_and_warn_if_ros_installed(cache)
@@ -139,8 +147,8 @@ class DistUpgradeQuirks(object):
         if self._snapstore_reachable:
             self._calculateSnapSizeRequirements()
 
-    def lunarPostUpgrade(self):
-        logging.debug("running Quirks.lunarPostUpgrade")
+    def noblePostUpgrade(self):
+        logging.debug("running Quirks.noblePostUpgrade")
         cache = self.controller.cache
         if 'snapd' not in cache:
             logging.debug("package required for Quirk not in cache")
@@ -151,6 +159,9 @@ class DistUpgradeQuirks(object):
         if 'ubuntu-desktop-raspi' in cache:
             if cache['ubuntu-desktop-raspi'].is_installed:
                 self._replace_fkms_overlay()
+        if 'ubuntu-server-raspi' in cache:
+            if cache['ubuntu-server-raspi'].is_installed:
+                self._add_kms_overlay()
         if 'linux-firmware-raspi2' in cache:
             if cache['linux-firmware-raspi2'].is_installed:
                 self._remove_uboot_on_rpi()
@@ -159,7 +170,7 @@ class DistUpgradeQuirks(object):
     def PostCleanup(self):
         " run after cleanup "
         logging.debug("running Quirks.PostCleanup")
-        self._remove_apport_ignore_tracker_extract()
+        self._remove_apport_ignore_list()
 
     # run right before the first packages get installed
     def StartUpgrade(self):
@@ -170,6 +181,8 @@ class DistUpgradeQuirks(object):
         self._killKBluetooth()
         self._pokeScreensaver()
         self._stopDocvertConverter()
+        self._set_generic_font()
+        self._disable_stub_resolver()
 
     # individual quirks handler that run *right before* the dist-upgrade
     # is calculated in the cache
@@ -177,6 +190,9 @@ class DistUpgradeQuirks(object):
         """ run right before calculating the dist-upgrade """
         logging.debug("running Quirks.PreDistUpgradeCache")
         # self._install_python_is_python2()
+        self._maybe_remove_gpg_wks_server()
+        self._install_t64_replacement_packages()
+        self._handle_ufw_breaks()
 
     # individual quirks handler that run *after* the dist-upgrade was
     # calculated in the cache
@@ -184,6 +200,7 @@ class DistUpgradeQuirks(object):
         """ run after calculating the dist-upgrade """
         logging.debug("running Quirks.PostDistUpgradeCache")
         self._install_linux_metapackage()
+        self._disable_cloud_init()
 
     # helpers
     def _get_pci_ids(self):
@@ -394,7 +411,7 @@ class DistUpgradeQuirks(object):
         with open("/proc/cpuinfo") as f:
             cpuinfo = f.read()
 
-            if re.search("^cpu\s*:\s*POWER8", cpuinfo, re.MULTILINE):
+            if re.search(r"^cpu\s*:\s*POWER8", cpuinfo, re.MULTILINE):
                 logging.error("POWER8 processor detected")
                 summary = _("Sorry, no more upgrades for this system")
                 msg = _("There will not be any further Ubuntu releases "
@@ -442,7 +459,7 @@ class DistUpgradeQuirks(object):
                 "ros-environment",
                 "ros-workspace"):
             ros_package_patterns.add(
-                re.compile("ros-[^\-]+-%s" % package_name))
+                re.compile(r"ros-[^\-]+-%s" % package_name))
 
         ros_is_installed = False
         for pkg in cache:
@@ -523,23 +540,30 @@ class DistUpgradeQuirks(object):
             self._view.getInstallProgress(self.controller.cache)
         )
 
-    def _add_apport_ignore_tracker_extract(self):
-        path = '/etc/apport/blacklist.d/upgrade-quirk-tracker-extract-3'
+    def _add_apport_ignore_list(self):
+        ignore_list = [
+            '/usr/libexec/tracker-extract-3',  # LP: #2012638
+            '/usr/sbin/update-apt-xapian-index',  # LP: #2058227
+        ]
+
+        path = '/etc/apport/blacklist.d/upgrade-quirks-ignore-list'
 
         try:
             os.makedirs(os.path.dirname(path), exist_ok=True)
             with open(path, 'w') as f:
-                f.write('/usr/libexec/tracker-extract-3\n')
-        except Exception as e:
-            logging.debug('Failed to create {}: {}'.format(path, e))
+                for bin in ignore_list:
+                    f.write(f'{bin}\n')
 
-    def _remove_apport_ignore_tracker_extract(self):
-        path = '/etc/apport/blacklist.d/upgrade-quirk-tracker-extract-3'
+        except Exception as e:
+            logging.debug(f'Failed to create {path}: {e}')
+
+    def _remove_apport_ignore_list(self):
+        path = '/etc/apport/blacklist.d/upgrade-quirks-ignore-list'
 
         try:
             os.remove(path)
         except Exception as e:
-            logging.debug('Failed to remove {}: {}'.format(path, e))
+            logging.debug(f'Failed to remove {path}: {e}')
 
     def _checkArmCPU(self):
         """
@@ -551,7 +575,7 @@ class DistUpgradeQuirks(object):
             return False
         with open("/proc/cpuinfo") as f:
             cpuinfo = f.read()
-        if re.search("^Processor\s*:\s*ARMv[45]", cpuinfo,
+        if re.search(r"^Processor\s*:\s*ARMv[45]", cpuinfo,
                      re.MULTILINE):
             return False
         return True
@@ -636,11 +660,11 @@ class DistUpgradeQuirks(object):
         connected = Popen(["snap", "debug", "connectivity"], stdout=PIPE,
                           stderr=PIPE, env=snap_env,
                           universal_newlines=True).communicate()
-        if re.search("^ \* PASS", connected[0], re.MULTILINE):
+        if re.search(r"^ \* PASS", connected[0], re.MULTILINE):
             self._snapstore_reachable = True
             return
         # can't connect
-        elif re.search("^ \*.*unreachable", connected[0], re.MULTILINE):
+        elif re.search(r"^ \*.*unreachable", connected[0], re.MULTILINE):
             logging.error("No snap store connectivity")
             old_lxd_deb_installed = False
             cache = self.controller.cache
@@ -739,10 +763,24 @@ class DistUpgradeQuirks(object):
         # _calculateSnapSizeRequirements call.
         for snap, snap_object in self._snap_list.items():
             command = snap_object['command']
-            if command == 'refresh':
+            if command == 'switch':
+                channel = snap_object['channel']
+                # TODO: This status should be updated, but the time of
+                # this change to snap switch is post-translation freeze.
                 self._view.updateStatus(_("refreshing snap %s" % snap))
-                popenargs = ["snap", command,
-                             "--channel", snap_object['channel'], snap]
+                popenargs = ["snap", command, "--channel", channel, snap]
+
+                if snap_object.get('switch_is_forced'):
+                    # We are force switching the snap tracking. Highlight this
+                    # to the user LP: #2077344.
+                    self._view.information(
+                        _(f'{snap} is now tracking {channel}'),
+                        _(
+                            'If needed, you can revert to your previous '
+                            f'revision with:\n\n$ snap revert {snap}'
+                        )
+                    )
+
             elif command == 'remove':
                 self._view.updateStatus(_("removing snap %s" % snap))
                 popenargs = ["snap", command, snap]
@@ -772,7 +810,7 @@ class DistUpgradeQuirks(object):
         pae = 0
         with open('/proc/cpuinfo') as f:
             cpuinfo = f.read()
-        if re.search("^flags\s+:.* pae ", cpuinfo, re.MULTILINE):
+        if re.search(r"^flags\s+:.* pae ", cpuinfo, re.MULTILINE):
             pae = 1
         if not pae:
             logging.error("no pae in /proc/cpuinfo")
@@ -1158,7 +1196,8 @@ class DistUpgradeQuirks(object):
                 deb = seed.get("deb", None)
                 from_chan = seed.get("from_channel", from_channel)
                 to_chan = seed.get("to_channel", to_channel)
-                seeded_snaps[snap] = (deb, from_chan, to_chan)
+                force_switch = seed.get("force_switch", False)
+                seeded_snaps[snap] = (deb, from_chan, to_chan, force_switch)
 
             for snap in d2s["unseeded"]:
                 unseed = d2s["unseeded"][snap]
@@ -1184,25 +1223,37 @@ class DistUpgradeQuirks(object):
                 if snap in seeded_snaps or snap in unseeded_snaps:
                     continue
                 else:
-                    seeded_snaps[snap] = (None, from_channel, to_channel)
+                    seeded_snaps[snap] = (
+                        None, from_channel, to_channel, False
+                    )
 
         self._view.updateStatus(_("Checking for installed snaps"))
-        for snap, (deb, from_channel, to_channel) in seeded_snaps.items():
+        for snap, props in seeded_snaps.items():
+            (deb, from_channel, to_channel, force_switch) = props
             snap_object = {}
             # check to see if the snap is already installed
             snap_info = subprocess.Popen(["snap", "info", snap],
                                          universal_newlines=True,
                                          stdout=subprocess.PIPE).communicate()
             self._view.processEvents()
-            if re.search("^installed: ", snap_info[0], re.MULTILINE):
+            if re.search(r"^installed: ", snap_info[0], re.MULTILINE):
                 logging.debug("Snap %s is installed" % snap)
-                # its not tracking the release channel so don't refresh
+
                 if not re.search(r"^tracking:.*%s" % from_channel,
                                  snap_info[0], re.MULTILINE):
                     logging.debug("Snap %s is not tracking the release channel"
                                   % snap)
-                    continue
-                snap_object['command'] = 'refresh'
+
+                    if force_switch:
+                        snap_object['switch_is_forced'] = True
+                    else:
+                        # It is not tracking the release channel, and we were
+                        # not told to force so don't switch.
+                        continue
+                else:
+                    snap_object['switch_is_forced'] = False
+
+                snap_object['command'] = 'switch'
             else:
                 # Do not replace packages not installed
                 cache = self.controller.cache
@@ -1228,7 +1279,7 @@ class DistUpgradeQuirks(object):
                                          universal_newlines=True,
                                          stdout=subprocess.PIPE).communicate()
             self._view.processEvents()
-            if re.search("^installed: ", snap_info[0], re.MULTILINE):
+            if re.search(r"^installed: ", snap_info[0], re.MULTILINE):
                 logging.debug("Snap %s is installed" % snap)
                 # its not tracking the release channel so don't remove
                 if not re.search(r"^tracking:.*%s" % from_channel,
@@ -1259,13 +1310,30 @@ class DistUpgradeQuirks(object):
                             logging.debug("Snap %s is being used by %s. "
                                           "Switching it to stable track"
                                           % (snap, plug_snap))
-                            snap_object['command'] = 'refresh'
+                            snap_object['command'] = 'switch'
                             snap_object['channel'] = 'stable'
                             break
 
                 self._snap_list[snap] = snap_object
         return self._snap_list
 
+    def _replace_pi_boot_config(self, old_config, new_config,
+                                boot_config_filename, failure_action):
+        try:
+            boot_backup_filename = boot_config_filename + '.distUpgrade'
+            with open(boot_backup_filename, 'w', encoding='utf-8') as f:
+                f.write(old_config)
+        except IOError as exc:
+            logging.error("unable to write boot config backup to %s: %s; %s",
+                          boot_backup_filename, exc, failure_action)
+            return
+        try:
+            with open(boot_config_filename, 'w', encoding='utf-8') as f:
+                f.write(new_config)
+        except IOError as exc:
+            logging.error("unable to write new boot config to %s: %s; %s",
+                          boot_config_filename, exc, failure_action)
+
     def _replace_fkms_overlay(self, boot_dir='/boot/firmware'):
         failure_action = (
             "You may need to replace the vc4-fkms-v3d overlay with "
@@ -1300,26 +1368,84 @@ class DistUpgradeQuirks(object):
             logging.warning("no fkms overlay or camera firmware line found "
                             "in %s", boot_config_filename)
             return
+        self._replace_pi_boot_config(
+            boot_config, new_config, boot_config_filename, failure_action)
 
+    def _add_kms_overlay(self, boot_dir='/boot/firmware'):
+        failure_action = (
+            "You may need to add dtoverlay=vc4-kms-v3d to an [all] section "
+            "in config.txt on your boot partition")
+        added_lines = [
+            '# added by do-release-upgrade (LP: #2065051)',
+            'dtoverlay=vc4-kms-v3d',
+            'disable_fw_kms_setup=1',
+            '',
+            '[pi3+]',
+            'dtoverlay=vc4-kms-v3d,cma-128',
+            '',
+            '[pi02]',
+            'dtoverlay=vc4-kms-v3d,cma-128',
+            '',
+            '[all]',
+        ]
         try:
-            boot_backup_filename = os.path.join(
-                boot_dir, 'config.txt.distUpgrade')
-            with open(boot_backup_filename, 'w', encoding='utf-8') as f:
-                f.write(boot_config)
-        except IOError as exc:
-            logging.error("unable to write boot config backup to %s: %s; %s",
-                          boot_backup_filename, exc, failure_action)
+            boot_config_filename = os.path.join(boot_dir, 'config.txt')
+            with open(boot_config_filename, 'r', encoding='utf-8') as f:
+                boot_config = f.read()
+        except FileNotFoundError:
+            logging.error("failed to open boot configuration in %s; %s",
+                          boot_config_filename, failure_action)
             return
-        try:
-            with open(boot_config_filename, 'w', encoding='utf-8') as f:
-                f.write(new_config)
-        except IOError as exc:
-            logging.error("unable to write new boot config to %s: %s; %s",
-                          boot_config_filename, exc, failure_action)
+
+        def find_insertion_point(lines):
+            # Returns the zero-based index of the dtoverlay=vc4-kms-v3d line in
+            # an [all] section, if one exists, or the last line of the last
+            # [all] section of the file, if one does not exist
+            in_all = True
+            last = 0
+            for index, line in enumerate(lines):
+                line = line.rstrip()
+                if in_all:
+                    last = index
+                    # startswith used to cope with any trailing dtparams
+                    if line.startswith('dtoverlay=vc4-kms-v3d'):
+                        return last
+                    elif line.startswith('[') and line.endswith(']'):
+                        in_all = line == '[all]'
+                    elif line.startswith('include '):
+                        # [sections] are included from includes verbatim, hence
+                        # (without reading the included file) we must assume
+                        # we're no longer in an [all] section
+                        in_all = False
+                else:
+                    in_all = line == '[all]'
+            return last
+
+        def add_kms_overlay(lines):
+            insert_point = find_insertion_point(lines)
+            try:
+                if lines[insert_point].startswith('dtoverlay=vc4-kms-v3d'):
+                    return lines
+            except IndexError:
+                # Empty config, apparently!
+                pass
+            lines[insert_point:insert_point] = added_lines
+            return lines
+
+        lines = [line.rstrip() for line in boot_config.splitlines()]
+        lines = add_kms_overlay(lines)
+        new_config = ''.join(line + '\n' for line in lines)
+
+        if new_config == boot_config:
+            logging.warning("no addition of KMS overlay required in %s",
+                            boot_config_filename)
+            return
+        self._replace_pi_boot_config(
+            boot_config, new_config, boot_config_filename, failure_action)
 
     def _remove_uboot_on_rpi(self, boot_dir='/boot/firmware'):
         kernel_line = 'kernel=vmlinuz'
-        initramfs_line = 'initramfs initrd.img followkernel' 
+        initramfs_line = 'initramfs initrd.img followkernel'
         failure_action = (
             "You may need to replace u_boot_* with vmlinuz, and add "
             "{initramfs_line!r} to config.txt on your boot "
@@ -1434,18 +1560,510 @@ class DistUpgradeQuirks(object):
             logging.warning("no u-boot removal performed in %s",
                             boot_config_filename)
             return
+        self._replace_pi_boot_config(
+            boot_config, new_config, boot_config_filename, failure_action)
 
-        try:
-            boot_backup_filename = boot_config_filename + '.distUpgrade'
-            with open(boot_backup_filename, 'w', encoding='utf-8') as f:
-                f.write(boot_config)
-        except IOError as exc:
-            logging.error("unable to write boot config backup to %s: %s; %s",
-                          boot_backup_filename, exc, failure_action)
+    def _set_generic_font(self):
+        """ Due to changes to the Ubuntu font we enable a generic font
+            (in practice DejaVu or Noto) during the upgrade.
+            See https://launchpad.net/bugs/2034986
+        """
+        temp_font = 'Sans'
+
+        if self._did_change_font:
+            return
+
+        if os.getenv('XDG_SESSION_TYPE', '') in ('', 'tty'):
+            # Avoid running this on server systems or when the upgrade
+            # is done over ssh.
+            return
+
+        if not {'SUDO_UID', 'PKEXEC_UID'}.intersection(os.environ):
+            logging.debug(
+                'Cannot determine how root privileges were gained, will '
+                'not change font'
+            )
             return
         try:
-            with open(boot_config_filename, 'w', encoding='utf-8') as f:
-                f.write(new_config)
-        except IOError as exc:
-            logging.error("unable to write new boot config to %s: %s; %s",
-                          boot_config_filename, exc, failure_action)
+            uid = int(os.getenv('SUDO_UID', os.getenv('PKEXEC_UID')))
+            pwuid = pwd.getpwuid(uid)
+        except ValueError:
+            logging.debug(
+                'Cannot determine non-root UID, will not change font'
+            )
+            return
+
+        schema = 'org.gnome.desktop.interface'
+
+        desktops = os.getenv('XDG_CURRENT_DESKTOP', '').split(':')
+
+        # Some flavors use other schemas for the desktop font.
+        if 'MATE' in desktops or 'UKUI' in desktops:
+            schema = 'org.mate.interface'
+        elif 'X-Cinnamon' in desktops:
+            schema = 'org.cinnamon.desktop.interface'
+
+        # Some flavors lack the systemd integration needed for a
+        # user service, so we create an autostart file instead.
+        use_autostart = bool(
+            set(['Budgie', 'LXQt', 'MATE', 'UKUI', 'X-Cinnamon', 'XFCE'])
+            & set(desktops)
+        )
+
+        r = subprocess.run(
+            ['systemd-run', '--user', '-M', f'{pwuid.pw_name}@.host',
+             '--wait', '--pipe', '-q', '--',
+             '/usr/bin/gsettings', 'get',
+             f'{schema}', 'font-name'],
+            stdout=subprocess.PIPE,
+            encoding='utf-8',
+        )
+
+        (font, _, size) = r.stdout.strip('\'\n').rpartition(' ')
+        font = font or 'Ubuntu'
+        try:
+            int(size)
+        except ValueError:
+            size = '11'
+
+        logging.debug(f'Setting generic font {temp_font} {size} during the '
+                      f'upgrade. Original font is {font} {size}.')
+
+        r = subprocess.run([
+            'systemd-run', '--user', '-M', f'{pwuid.pw_name}@.host',
+            '--wait', '--pipe', '-q', '--',
+            '/usr/bin/gsettings', 'set', f'{schema}',
+            'font-name', f'"{temp_font} {size}"'
+        ])
+        if r.returncode != 0:
+            logging.debug(f'Failed to change font to {temp_font} {size}')
+            return
+
+        self._did_change_font = True
+
+        # Touch a file to indiate that the font should be restored on the next
+        # boot.
+        need_font_restore_file = os.path.join(
+            pwuid.pw_dir, '.config/upgrade-need-font-restore'
+        )
+        os.makedirs(os.path.dirname(need_font_restore_file), exist_ok=True)
+        pathlib.Path(need_font_restore_file).touch(mode=0o666)
+        os.chown(need_font_restore_file, pwuid.pw_uid, pwuid.pw_gid)
+
+        if use_autostart:
+            autostart_file = '/etc/xdg/autostart/upgrade-restore-font.desktop'
+            os.makedirs(os.path.dirname(autostart_file), exist_ok=True)
+            flag = '$HOME/.config/upgrade-need-font-restore'
+            with open(autostart_file, 'w') as f:
+                f.write(
+                    '[Desktop Entry]\n'
+                    'Name=Restore font after upgrade\n'
+                    'Comment=Auto-generated by ubuntu-release-upgrader\n'
+                    'Type=Application\n'
+                    f'Exec=sh -c \'if [ -e "{flag}" ]; then gsettings set '
+                    f'{schema} font-name "{font} {size}";'
+                    f'rm -f "{flag}"; fi\'\n'
+                    'NoDisplay=true\n'
+                )
+            return
+
+        # If we set the font back to normal before a reboot, the font will
+        # still get all messed up. To allow normal usage whether the user
+        # reboots immediately or not, create a service that will run only if a
+        # ~/.config/upgrade-need-font-restore exists, and then remove that file
+        # in ExecStart. This has the effect of creating a one-time service on
+        # the next boot.
+        unit_file = '/usr/lib/systemd/user/upgrade-restore-font.service'
+        os.makedirs(os.path.dirname(unit_file), exist_ok=True)
+
+        with open(unit_file, 'w') as f:
+            f.write(
+                '# Auto-generated by ubuntu-release-upgrader\n'
+                '[Unit]\n'
+                'Description=Restore font after upgrade\n'
+                'After=graphical-session.target dconf.service\n'
+                'ConditionPathExists=%h/.config/upgrade-need-font-restore\n'
+                '\n'
+                '[Service]\n'
+                'Type=oneshot\n'
+                'ExecStart=/usr/bin/gsettings set '
+                f'{schema} font-name \'{font} {size}\'\n'
+                'ExecStart=/usr/bin/rm -f '
+                '%h/.config/upgrade-need-font-restore\n'
+                '\n'
+                '[Install]\n'
+                'WantedBy=graphical-session.target\n'
+            )
+
+        subprocess.run([
+            'systemctl',
+            '--user', '-M', f'{pwuid.pw_name}@.host',
+            'daemon-reload'
+        ])
+        r = subprocess.run([
+            'systemctl',
+            '--user', '-M', f'{pwuid.pw_name}@.host',
+            'enable', os.path.basename(unit_file)
+        ])
+        if r.returncode != 0:
+            logging.debug(f'Failed to enable {os.path.basename(unit_file)}. '
+                          'Font will not be restored on reboot')
+
+    def _maybe_remove_gpg_wks_server(self):
+        """
+        Prevent postfix from being unnecessarily installed, and leading to a
+        debconf prompt (LP: #2060578).
+        """
+        # We want to use attributes of the cache that are only exposed in
+        # apt_pkg.Cache, not the higher level apt.Cache. Hence we operate on
+        # apt.Cache._cache.
+        cache = self.controller.cache._cache
+
+        try:
+            if not cache['gpg-wks-server'].current_ver:
+                # Not installed, nothing to do.
+                return
+
+            provides_mta = cache['mail-transport-agent'].provides_list
+            installed_mta = [
+                ver for _, _, ver in provides_mta
+                if ver.parent_pkg.current_ver
+            ]
+        except KeyError:
+            # Something wasn't in the cache, ignore.
+            return
+
+        if not any(installed_mta):
+            logging.info(
+                'No mail-transport-agent installed, '
+                'marking gpg-wks-server for removal'
+            )
+
+            self.controller.cache['gpg-wks-server'].mark_delete(auto_fix=False)
+            apt.ProblemResolver(self.controller.cache).protect(
+                self.controller.cache['gpg-wks-server']
+            )
+
+    def _test_and_fail_on_armhf_raspi(self):
+        """
+        Test and fail if running armhf on a Raspberry Pi. Starting with Ubuntu
+        24.04, armhf is not well supported on Raspberry Pi. Hence, prevent
+        upgrades on such platforms. LP: #2060220.
+        """
+        if self.arch != 'armhf':
+            return
+
+        if (
+            'linux-image-raspi' in self.controller.cache and
+            self.controller.cache['linux-image-raspi'].is_installed
+        ):
+            logging.error('Detected Raspberry Pi kernel on armhf')
+
+            di = distro_info.UbuntuDistroInfo()
+            releases = {
+                r.series: r
+                for r in di.get_all('object')
+            }
+            from_release = releases.get(self.controller.fromDist)
+
+            if from_release is not None:
+                updates_end = from_release.eol.strftime('%B %Y')
+                updates_end_msg = _(
+                    f'Updates for Ubuntu {from_release.version} will '
+                    f'continue until {updates_end}.'
+                )
+            else:
+                # This shouldn't happen. But rather than crash the upgrader,
+                # just don't show the second part of the message.
+                updates_end_msg = ''
+
+            self._view.error(
+                _(
+                    'Sorry, cannot upgrade this system to 24.04 LTS'
+                ),
+                _(
+                    'The Raspberry Pi kernel for Ubuntu 24.04 LTS '
+                    'does not support the armhf architecture.\n\n'
+
+                ) + updates_end_msg,
+            )
+            self.controller.abort()
+
+    def _install_t64_replacement_packages(self):
+        """
+        LP: #2061175
+        """
+        for package in self.controller.cache:
+            if not package.is_installed:
+                continue
+
+            if package.candidate and package.candidate.downloadable:
+                continue
+
+            # If we get here, then it means that this package is installed
+            # and its candidate version is obsolete (i.e. not downloadable).
+            # Now, we need to find a replacement for it.
+            logging.debug(f'Searching for replacement for {package.name}')
+
+            reverse_depends_list = [
+                (dep.dep_type_untranslated, dep.parent_ver)
+                for dep in package._pkg.rev_depends_list
+            ]
+
+            provides_list = [ver for _, _, ver in package._pkg.provides_list]
+
+            for ver in provides_list:
+                if ('Replaces', ver) not in reverse_depends_list:
+                    continue
+
+                if (
+                    ('Breaks', ver) not in reverse_depends_list and
+                    ('Conflicts', ver) not in reverse_depends_list
+                ):
+                    continue
+
+                replacement = apt.Package(
+                    self.controller.cache,
+                    ver.parent_pkg
+                )
+
+                if replacement.candidate is None:
+                    continue
+
+                if replacement.candidate._cand != ver:
+                    continue
+
+                logging.debug(
+                    f'Found replacement {replacement.name} '
+                    f'for {package.name}'
+                )
+
+                # Transition the automatically installed bit
+                from_user = not package.is_auto_installed
+                replacement.mark_install(auto_fix=False, auto_inst=False,
+                                         from_user=from_user)
+                package.mark_delete(auto_fix=False)
+                apt.ProblemResolver(self.controller.cache).protect(package)
+
+                # We found a replacement. Exit the loop.
+                break
+
+            else:
+                # We did not find a replacement in the provides list.
+                logging.debug(
+                    f'Failed to find a replacement for {package.name}'
+                )
+
+    def _handle_ufw_breaks(self):
+        """
+        LP: #2061891
+        """
+
+        # This is only required for upgrades from jammy.
+        if self.controller.fromDist != 'jammy':
+            return
+
+        try:
+            ufw = self.controller.cache['ufw']
+        except KeyError:
+            return
+
+        if not ufw.is_installed:
+            return
+
+        remove_ufw = False
+        for name in ('netfilter-persistent', 'iptables-persistent'):
+            try:
+                pkg = self.controller.cache[name]
+            except KeyError:
+                continue
+
+            if pkg.is_installed:
+                logging.info(
+                    f'Keeping {name}, and removing ufw as a result.'
+                )
+
+                pkg.mark_install(auto_fix=False, auto_inst=False)
+                remove_ufw = True
+
+        if remove_ufw:
+            ufw.mark_delete(auto_fix=False)
+            apt.ProblemResolver(self.controller.cache).protect(ufw)
+
+    def _test_and_fail_on_tpm_fde(self):
+        """
+        LP: #2065229
+        """
+        try:
+            snap_list = subprocess.check_output(['snap', 'list'])
+            snaps = [s.decode().split()[0] for s in snap_list.splitlines()]
+        except FileNotFoundError:
+            # snapd not installed?
+            return
+
+        if (
+            'pc-kernel' in snaps and
+            'ubuntu-desktop-minimal' in self.controller.cache and
+            self.controller.cache['ubuntu-desktop-minimal'].is_installed
+        ):
+            logging.debug('Detected TPM FDE system')
+
+            di = distro_info.UbuntuDistroInfo()
+            version = di.version(self.controller.toDist) or 'next release'
+
+            self._view.error(
+                _(
+                    f'Sorry, cannot upgrade this system to {version}'
+                ),
+                _(
+                    'Upgrades for desktop systems running TPM FDE are not '
+                    'currently supported. '
+                    'Please see https://launchpad.net/bugs/2065229 '
+                    'for more information.'
+
+                ),
+            )
+            self.controller.abort()
+
+    def _set_grub_cloud_style_installation_on_cloud_installs(self):
+        """
+        LP: #2054103
+        """
+
+        if (
+            os.access("/etc/cloud/build.info", os.R_OK) and
+            "grub-pc" in self.controller.cache and
+            self.controller.cache["grub-pc"].is_installed
+        ):
+            logging.debug('Detected cloud image with grub-pc installed')
+
+            cmds = (
+                'echo "grub-pc grub-efi/cloud_style_installation '
+                'boolean true" | debconf-set-selections',
+                'echo "grub-pc grub-pc/cloud_style_installation '
+                'boolean true" | debconf-set-selections',
+            )
+
+            for cmd in cmds:
+                r = subprocess.run(('sh', '-c', cmd,))
+                if r.returncode != 0:
+                    logging.debug(
+                        f'Failed to run \'{cmd}\'. grub-pc package upgrades '
+                        'will continue to not update the bootloader.'
+                    )
+
+    def _disable_stub_resolver(self):
+        """
+        LP: #2043820
+        """
+
+        # This is only required for upgrades from jammy.
+        if self.controller.fromDist != 'jammy':
+            return
+
+        logging.debug(
+            'Disabling systemd-resolved stub resolver during upgrade'
+        )
+
+        os.makedirs('/run/systemd/resolved.conf.d', exist_ok=True)
+
+        with open(
+            '/run/systemd/resolved.conf.d/dist-upgrade-no-stub-resolver.conf',
+            'w'
+        ) as conf:
+            conf.write('# Temporarily disable stub resolver during upgrade\n')
+            conf.write('[Resolve]\n')
+            conf.write('DNSStubListener=no\n')
+
+        try:
+            subprocess.check_call(
+                ['systemctl', 'restart', 'systemd-resolved']
+            )
+        except subprocess.CalledProcessError as e:
+            logging.debug(f'Failed to disable stub resolver: {e}')
+
+    def _test_and_fail_on_unmerged_usr(self):
+        """
+        LP: #2058648
+        """
+        fail = False
+
+        logging.debug('Checking that system is /usr-merged')
+
+        for d in ('/bin', '/sbin', '/lib'):
+            try:
+                if os.readlink(d) != f'usr{d}':
+                    # The file is a symlink, but it's pointing to the wrong
+                    # place?
+                    fail = True
+                    break
+            except OSError:
+                # The file is not a symlink.
+                fail = True
+                break
+            except FileNotFoundError:
+                # This should not happen. Treat it like a failure just in case.
+                fail = True
+                break
+
+        if fail:
+            logging.debug('System is not /usr-merged!')
+            self._view.error(
+                _(
+                    f'Cannot upgrade system with unmerged /usr'
+                ),
+                _(
+                    'Please install the usrmerge package to fix this, '
+                    'and then try the upgrade again.'
+                ),
+            )
+            self.controller.abort()
+
+    def _disable_cloud_init(self):
+        """
+        LP: #2075968
+
+        Desktop metapackages now have Recommends: cloud-init, so it may be
+        installed for the first time on upgrades to Noble. It will run on the
+        first reboot after the upgrade by default, but we want to prevent that.
+        """
+        try:
+            pkg = self.controller.cache['cloud-init']
+
+            if not pkg.is_installed and pkg.marked_install:
+                os.makedirs('/etc/cloud', exist_ok=True)
+                with open('/etc/cloud/cloud-init.disabled', 'w') as f:
+                    f.write(
+                        'Disabled by ubuntu-release-upgrader. LP: #2075968.'
+                        '\n'
+                    )
+
+        except KeyError:
+            pass
+
+    def _test_and_fail_rabbitmq_server_installed(self):
+        """"
+        LP: #2074309
+        """
+        # This is only required for upgrades from jammy.
+        if self.controller.fromDist != 'jammy':
+            return
+
+        if (
+            'rabbitmq-server' in self.controller.cache and
+            self.controller.cache['rabbitmq-server'].is_installed
+        ):
+            self._view.error(
+                _(
+                    'Unable to upgrade to Ubuntu 24.04 LTS'
+                ),
+                _(
+                    'Currently, you have RabbitMQ server installed, which is '
+                    'not directly upgradable to the newer version. Upgrading '
+                    'may prevent the server from starting due to missing '
+                    'feature flags.\n\n'
+                    'For more information, please see '
+                    'https://bugs.launchpad.net/bugs/2074309.'
+                )
+            )
+            self.controller.abort()
