@@ -54,6 +54,7 @@ import traceback
 
 import apt
 import apt_pkg
+import distro_info
 import shlex # for osrelease
 import os
 
@@ -62,6 +63,7 @@ import pty
 from .DistUpgradeApport import run_apport, apport_crash
 
 from .DistUpgradeView import DistUpgradeView, FuzzyTimeToStr, InstallProgress, AcquireProgress
+from .DistUpgradeConfigParser import DistUpgradeConfig
 from .telemetry import get as get_telemetry
 
 import select
@@ -228,8 +230,8 @@ class DumbTerminal(QTextEdit):
         # do nothing for events like "shift"
         if not ev.text():
             return
-        # now sent the key event to the termianl as utf-8
-        os.write(self.installProgress.master_fd, ev.text().toUtf8())
+        # now send the key event to the terminal as utf-8
+        os.write(self.installProgress.master_fd, ev.text().encode('utf-8'))
 
     def onCursorPositionChanged(self):
         """ helper that ensures that the cursor is always at the end """
@@ -252,7 +254,7 @@ class KDECdromProgressAdapter(apt.progress.base.CdromProgress):
         """ update is called regularly so that the gui can be redrawn """
         if text:
           self.status.setText(text)
-        self.progressbar.setValue(step.value/float(self.totalSteps))
+        self.progressbar.setValue(int(step.value/float(self.totalSteps)))
         QApplication.processEvents()
 
     def ask_cdrom_name(self):
@@ -277,7 +279,7 @@ class KDEOpProgress(apt.progress.base.OpProgress):
       #else:
       #    self.progressbar.pulse()
       #self.progressbar.set_fraction(self.percent/100.0)
-      self.progressbar.setValue(self.percent)
+      self.progressbar.setValue(int(self.percent))
       QApplication.processEvents()
 
   def done(self):
@@ -318,7 +320,7 @@ class KDEAcquireProgressAdapter(AcquireProgress):
         # FIXME: move the status_str and progress_str into python-apt
         # (python-apt need i18n first for this)
         AcquireProgress.pulse(self, owner)
-        self.progress.setValue(self.percent)
+        self.progress.setValue(int(self.percent))
         current_item = self.current_items + 1
         if current_item > self.total_items:
             current_item = self.total_items
@@ -465,7 +467,7 @@ class KDEInstallProgressAdapter(InstallProgress):
         if self.start_time == 0.0:
           #print("setting start time to %s" % self.start_time)
           self.start_time = time.time()
-        self.progress.setValue(self.percent)
+        self.progress.setValue(int(self.percent))
         self.label_status.setText(utf8(status.strip()))
         # start showing when we gathered some data
         if percent > 1.0:
@@ -568,8 +570,10 @@ class DistUpgradeViewKDE(DistUpgradeView):
         logger.setLevel(logging.INFO)
         if not datadir or datadir == '.':
           localedir=os.path.join(os.getcwd(),"mo")
+          self.config = DistUpgradeConfig(os.getcwd())
         else:
           localedir="/usr/share/locale/ubuntu-release-upgrader"
+          self.config = DistUpgradeConfig(datadir)
 
         # FIXME: i18n must be somewhere relative do this dir
         try:
@@ -637,9 +641,13 @@ class DistUpgradeViewKDE(DistUpgradeView):
         name = _OSRelease().result["PRETTY_NAME"]
         if not name or name == "Ubuntu":
             name = "Kubuntu"
+
+        to_dist = self.config.get("Sources", "To")
+        to_version = distro_info.UbuntuDistroInfo().version(to_dist)
+
         title_string = self.window_main.label_title.text()
         title_string = title_string.replace("Ubuntu", name)
-        title_string = title_string.replace("%s", "22.04")
+        title_string = title_string.replace("%s", to_version)
         self.window_main.label_title.setText(title_string)
 
         # setup terminal text in hidden by default spot
