@@ -19,6 +19,7 @@ import sys
 import urllib.error
 import urllib.parse
 import urllib.request
+from typing import Any
 
 from apport.packaging_impl import impl as packaging
 
@@ -216,20 +217,19 @@ class CrashDatabase:
                     # we have a duplicate only identified by address sig,
                     # close it
                     master_id = addr_match
+                # our bug is a dupe of two different masters, one from
+                # symbolic, the other from addr matching (see LP#943117);
+                # make them all duplicates of each other, using the lower
+                # number as master
+                elif master_id < addr_match:
+                    self.close_duplicate(report, addr_match, master_id)
+                    self._duplicate_db_merge_id(addr_match, master_id)
                 else:
-                    # our bug is a dupe of two different masters, one from
-                    # symbolic, the other from addr matching (see LP#943117);
-                    # make them all duplicates of each other, using the lower
-                    # number as master
-                    if master_id < addr_match:
-                        self.close_duplicate(report, addr_match, master_id)
-                        self._duplicate_db_merge_id(addr_match, master_id)
-                    else:
-                        self.close_duplicate(report, master_id, addr_match)
-                        self._duplicate_db_merge_id(master_id, addr_match)
-                        master_id = addr_match
-                        # no version tracking for address signatures yet
-                        master_ver = None
+                    self.close_duplicate(report, master_id, addr_match)
+                    self._duplicate_db_merge_id(master_id, addr_match)
+                    master_id = addr_match
+                    # no version tracking for address signatures yet
+                    master_ver = None
 
         if master_id is not None and master_id != crash_id:
             if addr_sig:
@@ -569,7 +569,7 @@ class CrashDatabase:
 
         # crash got rejected
         if real_fixed_version == "invalid":
-            print(f"DEBUG: bug {crash_id} was invalidated," f" removing from database")
+            print(f"DEBUG: bug {crash_id} was invalidated, removing from database")
             self.duplicate_db_remove(crash_id)
             return
 
@@ -879,7 +879,9 @@ class CrashDatabase:
 #
 
 
-def get_crashdb(auth_file, name=None, conf=None):
+def get_crashdb(
+    auth_file: str | None, name: str | None = None, conf: str | None = None
+) -> CrashDatabase:
     """Return a CrashDatabase object for the given crash db name.
 
     This reads the configuration file 'conf'.
@@ -899,9 +901,10 @@ def get_crashdb(auth_file, name=None, conf=None):
       implementation for that crash db type). Other generally known options are
       'bug_pattern_url', 'dupdb_url', and 'problem_types'.
     """
-    if not conf:
+    if conf is None:
         conf = os.environ.get("APPORT_CRASHDB_CONF", "/etc/apport/crashdb.conf")
-    settings = {}
+    assert conf
+    settings: dict[str, Any] = {}
     with open(conf, encoding="utf-8") as f:
         # legacy, pylint: disable=exec-used
         exec(compile(f.read(), conf, "exec"), settings)
