@@ -27,8 +27,6 @@ import socket
 import stat
 import subprocess
 import time
-import typing
-from typing import Optional
 
 from apport.packaging_impl import impl as packaging
 from problem_report import ProblemReport
@@ -83,7 +81,7 @@ def get_dbus_socket(dbus_addr):
 
     # Don't support escaped values, multiple addresses, or multiple keys
     # and values
-    for search in ["%", ",", ";"]:
+    for search in ("%", ",", ";"):
         if search in dbus_addr:
             return None
 
@@ -260,7 +258,7 @@ _LOGIN_DEFS_RE = re.compile(
 )
 
 
-def _parse_login_defs(lines: io.TextIOWrapper) -> typing.Dict[str, str]:
+def _parse_login_defs(lines: io.TextIOWrapper) -> dict[str, str]:
     defs = {}
     for line in lines:
         match = _LOGIN_DEFS_RE.match(line)
@@ -271,7 +269,7 @@ def _parse_login_defs(lines: io.TextIOWrapper) -> typing.Dict[str, str]:
 
 
 @functools.cache
-def get_login_defs() -> typing.Dict[str, str]:
+def get_login_defs() -> dict[str, str]:
     """Parse /etc/login.defs and return a dictionary with its content."""
     try:
         with open("/etc/login.defs", encoding="utf-8") as login_defs_file:
@@ -575,7 +573,7 @@ def get_process_environ(proc_pid_fd: int) -> dict[str, str]:
     Raises an OSError in case the environ file could not been read.
     """
 
-    def opener(path: typing.Union[str, os.PathLike[str]], flags: int) -> int:
+    def opener(path: str | os.PathLike[str], flags: int) -> int:
         return os.open(path, flags, dir_fd=proc_pid_fd)
 
     with open(
@@ -603,11 +601,10 @@ def get_core_path(pid=None, exe=None, uid=None, timestamp=None, proc_pid_fd=None
     if pid is None:
         pid = "unknown"
         timestamp = "unknown"
-    else:
-        if timestamp is None:
-            with open(f"/proc/{pid}/stat", encoding="utf-8") as stat_file:
-                stat_contents = stat_file.read()
-            timestamp = get_starttime(stat_contents)
+    elif timestamp is None:
+        with open(f"/proc/{pid}/stat", encoding="utf-8") as stat_file:
+            stat_contents = stat_file.read()
+        timestamp = get_starttime(stat_contents)
 
     if exe is None:
         exe = get_process_path(proc_pid_fd)
@@ -697,7 +694,7 @@ def shared_libraries(path):
     return libs
 
 
-def should_skip_crash(report: ProblemReport, filename: str) -> Optional[str]:
+def should_skip_crash(report: ProblemReport, filename: str) -> str | None:
     """Check if the crash should be skipped for flood protection.
 
     In case the crash should be skipped return a string with the reason.
@@ -708,7 +705,7 @@ def should_skip_crash(report: ProblemReport, filename: str) -> Optional[str]:
     except (KeyError, ValueError):
         crash_counter = 0
     if crash_counter > 1:
-        return f"this executable already crashed" f" {crash_counter} times, ignoring"
+        return f"this executable already crashed {crash_counter} times, ignoring"
     if os.path.exists(filename) and not seen_report(filename):
         # don't clobber existing report
         return (
