@@ -12,6 +12,10 @@
 # option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
 # the full text of the license.
 
+# pylint: disable=too-many-lines
+# TODO: Address following pylint complaints
+# pylint: disable=invalid-name
+
 import base64
 import datetime
 import glob
@@ -24,9 +28,11 @@ import stat
 import subprocess
 import sys
 import tempfile
+import warnings
 
 import apport.fileutils
 from apport.packaging_impl import impl as packaging
+from problem_report import ProblemReport
 
 _invalid_key_chars_re = re.compile(r"[^0-9a-zA-Z_.-]")
 _AGENT = None
@@ -42,9 +48,7 @@ def path_to_key(path):
     return _invalid_key_chars_re.sub(".", path.replace(" ", "_"))
 
 
-def attach_file_if_exists(
-    report, path, key=None, overwrite=True, force_unicode=False
-):
+def attach_file_if_exists(report, path, key=None, overwrite=True, force_unicode=False):
     """Attach file contents if file exists.
 
     If key is not specified, the key name will be derived from the file
@@ -69,6 +73,8 @@ def attach_file_if_exists(
 
 
 def read_file(path, force_unicode=False):
+    # TODO: Split into smaller functions/methods
+    # pylint: disable=too-many-return-statements
     """Return the contents of the specified path.
 
     If the contents is valid UTF-8, or force_unicode is True, then the value
@@ -102,7 +108,7 @@ def read_file(path, force_unicode=False):
         except UnicodeDecodeError:
             return contents
     except OSError as error:
-        return "Error: " + str(error)
+        return f"Error: {str(error)}"
 
 
 def attach_file(report, path, key=None, overwrite=True, force_unicode=False):
@@ -140,7 +146,7 @@ def attach_conffiles(report, package, conffiles=None, ui=None):
         if conffiles and path not in conffiles:
             continue
 
-        key = "modified.conffile." + path_to_key(path)
+        key = f"modified.conffile.{path_to_key(path)}"
         if isinstance(contents, str) and (
             contents == "[deleted]" or contents.startswith("[inaccessible")
         ):
@@ -149,9 +155,8 @@ def attach_conffiles(report, package, conffiles=None, ui=None):
 
         if ui:
             response = ui.yesno(
-                'It seems you have modified the contents of "%s".  '
-                "Would you like to add the contents of it to your bug report?"
-                % path
+                f'It seems you have modified the contents of "{path}".  '
+                f"Would you like to add the contents of it to your bug report?"
             )
             if response:
                 report[key] = contents
@@ -161,67 +166,29 @@ def attach_conffiles(report, package, conffiles=None, ui=None):
             report[key] = "[modified]"
 
         mtime = datetime.datetime.fromtimestamp(os.stat(path).st_mtime)
-        report["mtime.conffile." + path_to_key(path)] = mtime.isoformat()
+        report[f"mtime.conffile.{path_to_key(path)}"] = mtime.isoformat()
 
 
+# pylint: disable-next=unused-argument
 def attach_upstart_overrides(report, package):
     """Attach information about any Upstart override files."""
-    try:
-        files = packaging.get_files(package)
-    except ValueError:
-        return
-
-    for file in files:
-        if os.path.exists(file) and file.startswith("/etc/init/"):
-            override = file.replace(".conf", ".override")
-            key = "upstart." + override.replace("/etc/init/", "")
-            attach_file_if_exists(report, override, key)
+    warnings.warn(
+        "apport.hookutils.attach_upstart_overrides() is obsolete."
+        " Upstart is dead. Please drop this call.",
+        PendingDeprecationWarning,
+        stacklevel=2,
+    )
 
 
+# pylint: disable-next=unused-argument
 def attach_upstart_logs(report, package):
     """Attach information about a package's session upstart logs."""
-    try:
-        files = packaging.get_files(package)
-    except ValueError:
-        return
-
-    for f in files:
-        if not os.path.exists(f):
-            continue
-        if f.startswith("/usr/share/upstart/sessions/"):
-            log = os.path.basename(f).replace(".conf", ".log")
-            key = "upstart." + log
-            try:
-                log = os.path.join(
-                    os.environ["XDG_CACHE_HOME"], "upstart", log
-                )
-            except KeyError:
-                try:
-                    log = os.path.join(
-                        os.environ["HOME"], ".cache", "upstart", log
-                    )
-                except KeyError:
-                    continue
-
-            attach_file_if_exists(report, log, key)
-
-        if f.startswith("/usr/share/applications/") and f.endswith(".desktop"):
-            desktopname = os.path.splitext(os.path.basename(f))[0]
-            key = "upstart.application." + desktopname
-            log = "application-%s.log" % desktopname
-            try:
-                log = os.path.join(
-                    os.environ["XDG_CACHE_HOME"], "upstart", log
-                )
-            except KeyError:
-                try:
-                    log = os.path.join(
-                        os.environ["HOME"], ".cache", "upstart", log
-                    )
-                except KeyError:
-                    continue
-
-            attach_file_if_exists(report, log, key)
+    warnings.warn(
+        "apport.hookutils.attach_upstart_logs() is obsolete."
+        " Upstart is dead. Please drop this call.",
+        PendingDeprecationWarning,
+        stacklevel=2,
+    )
 
 
 def attach_dmesg(report):
@@ -234,23 +201,30 @@ def attach_dmesg(report):
 
 
 def attach_dmi(report):
+    """Attach Desktop Management Interface (DMI) information to the report."""
     dmi_dir = "/sys/class/dmi/id"
     if os.path.isdir(dmi_dir):
         for f in os.listdir(dmi_dir):
-            p = os.path.realpath("%s/%s" % (dmi_dir, f))
+            if f in {"subsystem", "uevent"}:
+                continue
+            p = os.path.realpath(f"{dmi_dir}/{f}")
             st = os.stat(p)
             # ignore the root-only ones, since they have serial numbers
             if not stat.S_ISREG(st.st_mode) or (st.st_mode & 4 == 0):
                 continue
-            if f in ("subsystem", "uevent"):
-                continue
 
             try:
                 value = read_file(p)
             except OSError:
                 continue
             if value:
-                report["dmi." + f.replace("_", ".")] = value
+                report[f"dmi.{f.replace('_', '.')}"] = value
+
+    # Use the hardware information to create a machine type.
+    if "dmi.sys.vendor" in report and "dmi.product.name" in report:
+        report["MachineType"] = (
+            f"{report['dmi.sys.vendor']} {report['dmi.product.name']}"
+        )
 
 
 def attach_hardware(report):
@@ -285,9 +259,7 @@ def attach_hardware(report):
     report["Lsusb-t"] = command_output(["lsusb", "-t"])
     report["ProcModules"] = command_output(["sort", "/proc/modules"])
     report["UdevDb"] = command_output(["udevadm", "info", "--export-db"])
-    report["acpidump"] = root_command_output(
-        ["/usr/share/apport/dump_acpi_tables.py"]
-    )
+    report["acpidump"] = root_command_output(["/usr/share/apport/dump_acpi_tables.py"])
 
     # anonymize partition labels
     labels = report["UdevDb"]
@@ -301,13 +273,6 @@ def attach_hardware(report):
 
     attach_dmi(report)
 
-    # Use the hardware information to create a machine type.
-    if "dmi.sys.vendor" in report and "dmi.product.name" in report:
-        report["MachineType"] = "%s %s" % (
-            report["dmi.sys.vendor"],
-            report["dmi.product.name"],
-        )
-
     if command_available("prtconf"):
         report["Prtconf"] = command_output(["prtconf"])
 
@@ -325,13 +290,9 @@ def attach_alsa_old(report):
     for systems where alsa-info is not installed
     (i e, *buntu 12.04 and earlier)
     """
+    attach_file_if_exists(report, os.path.expanduser("~/.asoundrc"), "UserAsoundrc")
     attach_file_if_exists(
-        report, os.path.expanduser("~/.asoundrc"), "UserAsoundrc"
-    )
-    attach_file_if_exists(
-        report,
-        os.path.expanduser("~/.asoundrc.asoundconf"),
-        "UserAsoundrcAsoundconf",
+        report, os.path.expanduser("~/.asoundrc.asoundconf"), "UserAsoundrcAsoundconf"
     )
     attach_file_if_exists(report, "/etc/asound.conf")
     attach_file_if_exists(report, "/proc/asound/version", "AlsaVersion")
@@ -352,24 +313,23 @@ def attach_alsa_old(report):
                     cards.append(int(fields[0]))
 
     for card in cards:
-        key = "Card%d.Amixer.info" % card
+        key = f"Card{card}.Amixer.info"
         report[key] = command_output(["amixer", "-c", str(card), "info"])
-        key = "Card%d.Amixer.values" % card
+        key = f"Card{card}.Amixer.values"
         report[key] = command_output(["amixer", "-c", str(card)])
 
-        for codecpath in glob.glob("/proc/asound/card%d/codec*" % card):
+        for codecpath in glob.glob(f"/proc/asound/card{card}/codec*"):
             if os.path.isfile(codecpath):
                 codec = os.path.basename(codecpath)
-                key = "Card%d.Codecs.%s" % (card, path_to_key(codec))
+                key = f"Card{card}.Codecs.{path_to_key(codec)}"
                 attach_file(report, codecpath, key=key)
             elif os.path.isdir(codecpath):
                 codec = os.path.basename(codecpath)
                 for name in os.listdir(codecpath):
                     path = os.path.join(codecpath, name)
-                    key = "Card%d.Codecs.%s.%s" % (
-                        card,
-                        path_to_key(codec),
-                        path_to_key(name),
+                    key = (
+                        f"Card{card}.Codecs"
+                        f".{path_to_key(codec)}.{path_to_key(name)}"
                     )
                     attach_file(report, path, key)
 
@@ -446,7 +406,7 @@ def command_output(
             env=env,
         )
     except OSError as error:
-        return "Error: " + str(error)
+        return f"Error: {str(error)}"
 
     if sp.returncode == 0:
         res = sp.stdout.strip()
@@ -500,10 +460,10 @@ def _spawn_pkttyagent():
                 if event_type & select.EPOLLHUP:
                     os.close(r)
                     return
-    return
 
 
 def kill_pkttyagent():
+    """Kill pkttyagent (from PolicyKit) if it was started by Apport."""
     global _AGENT  # pylint: disable=global-statement
 
     if _AGENT is None:
@@ -558,13 +518,11 @@ def attach_root_command_outputs(report, command_map):
     Just like root_command_output, this passes the command through pkexec,
     unless the caller is already root.
 
-    This is preferrable to using root_command_output() multiple times, as that
+    This is preferable to using root_command_output() multiple times, as that
     will ask for the password every time.
     """
     wrapper_path = os.path.join(
-        os.path.abspath(
-            os.environ.get("APPORT_DATA_DIR", "/usr/share/apport")
-        ),
+        os.path.abspath(os.environ.get("APPORT_DATA_DIR", "/usr/share/apport")),
         "root_info_wrapper",
     )
     workdir = tempfile.mkdtemp()
@@ -578,10 +536,7 @@ def attach_root_command_outputs(report, command_map):
                 ), "command must be a string (shell command)"
                 # use "| cat" here, so that we can end commands with 2>&1
                 # (otherwise it would have the wrong redirection order)
-                script.write(
-                    "%s | cat > %s\n"
-                    % (command, os.path.join(workdir, keyname))
-                )
+                script.write(f"{command} | cat > {os.path.join(workdir, keyname)}\n")
 
         # run script
         subprocess.run(
@@ -623,17 +578,23 @@ def __filter_re_process(pattern, process):
     return ""
 
 
-def recent_syslog(pattern, path=None):
+def recent_syslog(pattern, path=None, *, journald_only_system=True):
     """Extract recent system messages which match a regex.
 
     pattern should be a "re" object. By default, messages are read from
     the systemd journal, or /var/log/syslog; but when giving "path", messages
     are read from there instead.
+    The journald_only_system parameter controls the scope of messages that are
+    extracted when reading from the systemd journal. If set to True (the
+    default), only messages from the system services are extracted. If set to
+    False, all messages that the current user can see are extracted.
     """
     if path:
         command = ["tail", "-n", "10000", path]
     elif os.path.exists("/run/systemd/system"):
-        command = ["journalctl", "--system", "--quiet", "-b", "-a"]
+        command = ["journalctl", "--quiet", "-b", "-a"]
+        if journald_only_system:
+            command.append("--system")
     elif os.access("/var/log/syslog", os.R_OK):
         command = ["tail", "-n", "10000", "/var/log/syslog"]
     else:
@@ -752,9 +713,7 @@ def attach_gsettings_schema(report, schema):
     env = os.environ.copy()
     env["XDG_CONFIG_HOME"] = "/nonexisting"
     with subprocess.Popen(
-        ["gsettings", "list-recursively", schema],
-        env=env,
-        stdout=subprocess.PIPE,
+        ["gsettings", "list-recursively", schema], env=env, stdout=subprocess.PIPE
     ) as gsettings:
         for line in gsettings.stdout:
             try:
@@ -775,12 +734,12 @@ def attach_gsettings_schema(report, schema):
                 continue  # invalid line
 
             if value != defaults.get(schema_name, {}).get(key, ""):
-                if schema_name == b"org.gnome.shell" and key in [
+                if schema_name == b"org.gnome.shell" and key in {
                     b"command-history",
                     b"favorite-apps",
-                ]:
+                }:
                     value = "redacted by apport"
-                cur_value += "%s %s %s\n" % (schema_name, key, value)
+                cur_value += f"{schema_name} {key} {value}\n"
 
     report["GsettingsChanges"] = cur_value
 
@@ -794,7 +753,7 @@ def attach_gsettings_package(report, package):
         attach_gsettings_schema(report, schema)
 
 
-def attach_journal_errors(report, time_window=10) -> None:
+def attach_journal_errors(report: ProblemReport, time_window: int = 10) -> None:
     """Attach journal warnings and errors.
 
     If the report contains a date, get the journal logs around that
@@ -821,9 +780,7 @@ def attach_network(report):
     report["IpRoute"] = command_output(["ip", "route"])
     report["IpAddr"] = command_output(["ip", "addr"])
     report["PciNetwork"] = pci_devices(PCI_NETWORK)
-    attach_file_if_exists(
-        report, "/etc/network/interfaces", key="IfupdownConfig"
-    )
+    attach_file_if_exists(report, "/etc/network/interfaces", key="IfupdownConfig")
 
     for var in ("http_proxy", "ftp_proxy", "no_proxy"):
         if var in os.environ:
@@ -858,9 +815,7 @@ def attach_wifi(report):
         iw_output = "N/A"
     report["CRDA"] = iw_output
 
-    attach_file_if_exists(
-        report, "/var/log/wpa_supplicant.log", key="WpaSupplicantLog"
-    )
+    attach_file_if_exists(report, "/var/log/wpa_supplicant.log", key="WpaSupplicantLog")
 
 
 def attach_printing(report):
@@ -877,9 +832,7 @@ def attach_printing(report):
     if ppds:
         nicknames = command_output(["fgrep", "-H", "*NickName"] + ppds)
         report["PpdFiles"] = re.sub(
-            r'/etc/cups/ppd/(.*).ppd:\*NickName: *"(.*)"',
-            r"\g<1>: \g<2>",
-            nicknames,
+            r'/etc/cups/ppd/(.*).ppd:\*NickName: *"(.*)"', r"\g<1>: \g<2>", nicknames
         )
 
     report["PrintingPackages"] = package_versions(
@@ -936,46 +889,33 @@ def attach_mac_events(report, profiles=None):
 
     if "AuditLog" not in report and os.path.exists("/var/run/auditd.pid"):
         attach_root_command_outputs(
-            report,
-            {"AuditLog": 'egrep "' + mac_regex + '" /var/log/audit/audit.log'},
+            report, {"AuditLog": f'egrep "{mac_regex}" /var/log/audit/audit.log'}
         )
 
-    attach_file_if_exists(
-        report, "/proc/version_signature", "ProcVersionSignature"
-    )
+    attach_file_if_exists(report, "/proc/version_signature", "ProcVersionSignature")
     attach_file(report, "/proc/cmdline", "ProcCmdline")
 
     for match in re.findall(
         aa_re, report.get("KernLog", "") + report.get("AuditLog", "")
     ):
         if not profiles:
-            _add_tag(report, "apparmor")
+            report.add_tags(["apparmor"])
             break
 
         try:
             if match[0] == '"':
                 profile = match[1:-1]
             else:
-                profile = bytes.fromhex(match).decode(
-                    "UTF-8", errors="replace"
-                )
+                profile = bytes.fromhex(match).decode("UTF-8", errors="replace")
         except (IndexError, ValueError):
             continue
 
         for search_profile in profiles:
-            if re.match("^" + search_profile + "$", profile):
-                _add_tag(report, "apparmor")
+            if re.match(f"^{search_profile}$", profile):
+                report.add_tags(["apparmor"])
                 break
 
 
-def _add_tag(report, tag):
-    """Add or append a tag to the report."""
-    current_tags = report.get("Tags", "")
-    if current_tags:
-        current_tags += " "
-    report["Tags"] = current_tags + tag
-
-
 def attach_related_packages(report, packages):
     """Attach version information for related packages.
 
@@ -1011,8 +951,7 @@ def package_versions(*packages):
             versions.append((package, version))
 
     package_width = max(len(version[0]) for version in versions)
-    fmt = "%%-%ds %%s" % package_width
-    return "\n".join([fmt % v for v in versions])
+    return "\n".join([f"{p:<{package_width}s} {v}" for (p, v) in versions])
 
 
 def _get_module_license(module):
@@ -1069,7 +1008,7 @@ def __drm_con_info(con):
         if f == "edid":
             val = base64.b64encode(val)
             f += "-base64"
-        info += "%s: %s\n" % (f, val.decode("UTF-8", errors="replace"))
+        info += f"{f}: {val.decode('UTF-8', errors='replace')}\n"
     return info
 
 
@@ -1085,7 +1024,7 @@ def attach_drm_info(report):
         con = os.path.join(drm_dir, f)
         if os.path.exists(os.path.join(con, "enabled")):
             # DRM can set an arbitrary string for its connector paths.
-            report["DRM." + path_to_key(f)] = __drm_con_info(con)
+            report[f"DRM.{path_to_key(f)}"] = __drm_con_info(con)
 
 
 def in_session_of_problem(report):
@@ -1121,9 +1060,7 @@ def in_session_of_problem(report):
 
     # determine session creation time
     try:
-        session_start_time = os.stat(
-            "/run/systemd/sessions/" + session_id
-        ).st_mtime
+        session_start_time = os.stat(f"/run/systemd/sessions/{session_id}").st_mtime
     except OSError:
         return None
 
@@ -1139,9 +1076,11 @@ def attach_default_grub(report, key=None):
     if os.path.exists(path):
         with open(path, "r", encoding="utf-8") as f:
             filtered = [
-                line
-                if not line.startswith("password")
-                else "### PASSWORD LINE REMOVED ###"
+                (
+                    line
+                    if not line.startswith("password")
+                    else "### PASSWORD LINE REMOVED ###"
+                )
                 for line in f.readlines()
             ]
             report[key] = "".join(filtered)
@@ -1152,9 +1091,7 @@ def attach_casper_md5check(report, location):
     result = "unknown"
     mismatches = []
     if os.path.exists(location):
-        attach_root_command_outputs(
-            report, {"CasperMD5json": "cat '%s'" % location}
-        )
+        attach_root_command_outputs(report, {"CasperMD5json": f"cat '{location}'"})
         if "CasperMD5json" in report:
             check = json.loads(report["CasperMD5json"])
             result = check["result"]
