@@ -1,5 +1,4 @@
 #!/usr/bin/python3
-# -*- coding: utf-8 -*-
 # auth - authentication key management
 #
 #  Copyright (c) 2004 Canonical
@@ -24,14 +23,6 @@
 #  USA
 """Handle GnuPG keys used to trust signed repositories."""
 
-from __future__ import print_function
-
-try:
-    from typing import Optional
-except ImportError:
-    pass
-
-
 import errno
 import os
 import os.path
@@ -43,17 +34,6 @@ import tempfile
 import apt_pkg
 from apt_pkg import gettext as _
 
-if sys.version_info.major > 2:
-    unicode = str
-
-try:
-    from typing import List, Tuple, Union
-    List  # pyflakes
-    Tuple  # pyflakes
-    Union  # pyflakes
-except ImportError:
-    pass
-
 
 class AptKeyError(Exception):
     pass
@@ -63,25 +43,22 @@ class AptKeyIDTooShortError(AptKeyError):
     """Internal class do not rely on it."""
 
 
-class TrustedKey(object):
+class TrustedKey:
 
     """Represents a trusted key."""
 
-    def __init__(self, name, keyid, date):
-        # type: (str, str, str) -> None
+    def __init__(self, name: str, keyid: str, date: str) -> None:
         self.raw_name = name
         # Allow to translated some known keys
         self.name = _(name)
         self.keyid = keyid
         self.date = date
 
-    def __str__(self):
-        # type: () -> str
-        return "%s\n%s %s" % (self.name, self.keyid, self.date)
+    def __str__(self) -> str:
+        return f"{self.name}\n{self.keyid} {self.date}"
 
 
-def _call_apt_key_script(*args, **kwargs):
-    # type: (str, Optional[str]) -> str
+def _call_apt_key_script(*args: str, **kwargs: str | None) -> str:
     """Run the apt-key script with the given arguments."""
     conf = None
     cmd = [apt_pkg.config.find_file("Dir::Bin::Apt-Key", "/usr/bin/apt-key")]
@@ -95,20 +72,20 @@ def _call_apt_key_script(*args, **kwargs):
             # configuration from the chroot to the apt-key script by using
             # a temporary APT_CONFIG file. The apt-key script uses apt-config
             # shell internally
-            conf = tempfile.NamedTemporaryFile(
-                prefix="apt-key", suffix=".conf")
+            conf = tempfile.NamedTemporaryFile(prefix="apt-key", suffix=".conf")
             conf.write(apt_pkg.config.dump().encode("UTF-8"))
             conf.flush()
             env["APT_CONFIG"] = conf.name
-        proc = subprocess.Popen(cmd, env=env, universal_newlines=True,
-                                stdin=subprocess.PIPE,
-                                stdout=subprocess.PIPE,
-                                stderr=subprocess.PIPE)
+        proc = subprocess.Popen(
+            cmd,
+            env=env,
+            universal_newlines=True,
+            stdin=subprocess.PIPE,
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE,
+        )
 
         stdin = kwargs.get("stdin", None)
-        # py2 needs this encoded, py3.3 will crash if it is
-        if sys.version_info.major < 3 and isinstance(stdin, unicode):
-            stdin = stdin.encode("utf-8")  # type: ignore
 
         output, stderr = proc.communicate(stdin)  # type: str, str
 
@@ -117,10 +94,10 @@ def _call_apt_key_script(*args, **kwargs):
                 "The apt-key script failed with return code %s:\n"
                 "%s\n"
                 "stdout: %s\n"
-                "stderr: %s" % (
-                    proc.returncode, " ".join(cmd), output, stderr))
+                "stderr: %s" % (proc.returncode, " ".join(cmd), output, stderr)
+            )
         elif stderr:
-            sys.stderr.write(stderr)    # Forward stderr
+            sys.stderr.write(stderr)  # Forward stderr
 
         return output.strip()
     finally:
@@ -128,8 +105,7 @@ def _call_apt_key_script(*args, **kwargs):
             conf.close()
 
 
-def add_key_from_file(filename):
-    # type: (str) -> None
+def add_key_from_file(filename: str) -> None:
     """Import a GnuPG key file to trust repositores signed by it.
 
     Keyword arguments:
@@ -142,8 +118,7 @@ def add_key_from_file(filename):
     _call_apt_key_script("add", filename)
 
 
-def add_key_from_keyserver(keyid, keyserver):
-    # type: (str, str) -> None
+def add_key_from_keyserver(keyid: str, keyserver: str) -> None:
     """Import a GnuPG key file to trust repositores signed by it.
 
     Keyword arguments:
@@ -159,40 +134,46 @@ def add_key_from_keyserver(keyid, keyserver):
     finally:
         # We are racing with gpg when removing sockets, so ignore
         # failure to delete non-existing files.
-        def onerror(func, path, exc_info):
-            # type: (object, str, Tuple[type, Exception, object]) -> None
-            if (isinstance(exc_info[1], OSError) and
-                exc_info[1].errno == errno.ENOENT):
+        def onerror(
+            func: object, path: str, exc_info: tuple[type, Exception, object]
+        ) -> None:
+            if isinstance(exc_info[1], OSError) and exc_info[1].errno == errno.ENOENT:
                 return
             raise
 
         shutil.rmtree(tmp_keyring_dir, onerror=onerror)
 
 
-def _add_key_from_keyserver(keyid, keyserver, tmp_keyring_dir):
-    # type: (str, str, str) -> None
+def _add_key_from_keyserver(keyid: str, keyserver: str, tmp_keyring_dir: str) -> None:
     if len(keyid.replace(" ", "").replace("0x", "")) < (160 / 4):
-        raise AptKeyIDTooShortError(
-            "Only fingerprints (v4, 160bit) are supported")
+        raise AptKeyIDTooShortError("Only fingerprints (v4, 160bit) are supported")
     # create a temp keyring dir
     tmp_secret_keyring = os.path.join(tmp_keyring_dir, "secring.gpg")
     tmp_keyring = os.path.join(tmp_keyring_dir, "pubring.gpg")
     # default options for gpg
     gpg_default_options = [
         "gpg",
-        "--no-default-keyring", "--no-options",
-        "--homedir", tmp_keyring_dir,
+        "--no-default-keyring",
+        "--no-options",
+        "--homedir",
+        tmp_keyring_dir,
     ]
     # download the key to a temp keyring first
-    res = subprocess.call(gpg_default_options + [
-        "--secret-keyring", tmp_secret_keyring,
-        "--keyring", tmp_keyring,
-        "--keyserver", keyserver,
-        "--recv", keyid,
-    ])
+    res = subprocess.call(
+        gpg_default_options
+        + [
+            "--secret-keyring",
+            tmp_secret_keyring,
+            "--keyring",
+            tmp_keyring,
+            "--keyserver",
+            keyserver,
+            "--recv",
+            keyid,
+        ]
+    )
     if res != 0:
-        raise AptKeyError("recv from '%s' failed for '%s'" % (
-            keyserver, keyid))
+        raise AptKeyError(f"recv from '{keyserver}' failed for '{keyid}'")
     # FIXME:
     # - with gnupg 1.4.18 the downloaded key is actually checked(!),
     #   i.e. gnupg will not import anything that the server sends
@@ -202,26 +183,35 @@ def _add_key_from_keyserver(keyid, keyserver, tmp_keyring_dir):
     # now export again using the long key id (to ensure that there is
     # really only this one key in our keyring) and not someone MITM us
     tmp_export_keyring = os.path.join(tmp_keyring_dir, "export-keyring.gpg")
-    res = subprocess.call(gpg_default_options + [
-        "--keyring", tmp_keyring,
-        "--output", tmp_export_keyring,
-        "--export", keyid,
-    ])
+    res = subprocess.call(
+        gpg_default_options
+        + [
+            "--keyring",
+            tmp_keyring,
+            "--output",
+            tmp_export_keyring,
+            "--export",
+            keyid,
+        ]
+    )
     if res != 0:
         raise AptKeyError("export of '%s' failed", keyid)
     # now verify the fingerprint, this is probably redundant as we
     # exported by the fingerprint in the previous command but its
     # still good paranoia
     output = subprocess.Popen(
-        gpg_default_options + [
-            "--keyring", tmp_export_keyring,
+        gpg_default_options
+        + [
+            "--keyring",
+            tmp_export_keyring,
             "--fingerprint",
             "--batch",
             "--fixed-list-mode",
             "--with-colons",
         ],
         stdout=subprocess.PIPE,
-        universal_newlines=True).communicate()[0]
+        universal_newlines=True,
+    ).communicate()[0]
     got_fingerprint = None
     for line in output.splitlines():
         if line.startswith("fpr:"):
@@ -235,25 +225,22 @@ def _add_key_from_keyserver(keyid, keyserver, tmp_keyring_dir):
         # make the error match what gnupg >= 1.4.18 will output when
         # it checks the key itself before importing it
         raise AptKeyError(
-            "recv from '%s' failed for '%s'" % (
-                keyserver, signing_key_fingerprint))
+            f"recv from '{keyserver}' failed for '{signing_key_fingerprint}'"
+        )
     # finally add it
     add_key_from_file(tmp_export_keyring)
 
 
-def add_key(content):
-    # type: (str) -> None
+def add_key(content: str) -> None:
     """Import a GnuPG key to trust repositores signed by it.
 
     Keyword arguments:
     content -- the content of the GnuPG public key
     """
-    _call_apt_key_script("adv", "--quiet", "--batch",
-                         "--import", "-", stdin=content)
+    _call_apt_key_script("adv", "--quiet", "--batch", "--import", "-", stdin=content)
 
 
-def remove_key(fingerprint):
-    # type: (str) -> None
+def remove_key(fingerprint: str) -> None:
     """Remove a GnuPG key to no longer trust repositores signed by it.
 
     Keyword arguments:
@@ -262,8 +249,7 @@ def remove_key(fingerprint):
     _call_apt_key_script("rm", fingerprint)
 
 
-def export_key(fingerprint):
-    # type: (str) -> str
+def export_key(fingerprint: str) -> str:
     """Return the GnuPG key in text format.
 
     Keyword arguments:
@@ -272,8 +258,7 @@ def export_key(fingerprint):
     return _call_apt_key_script("export", fingerprint)
 
 
-def update():
-    # type: () -> str
+def update() -> str:
     """Update the local keyring with the archive keyring and remove from
     the local keyring the archive keys which are no longer valid. The
     archive keyring is shipped in the archive-keyring package of your
@@ -282,8 +267,7 @@ def update():
     return _call_apt_key_script("update")
 
 
-def net_update():
-    # type: () -> str
+def net_update() -> str:
     """Work similar to the update command above, but get the archive
     keyring from an URI instead and validate it against a master key.
     This requires an installed wget(1) and an APT build configured to
@@ -294,15 +278,15 @@ def net_update():
     return _call_apt_key_script("net-update")
 
 
-def list_keys():
-    # type: () -> List[TrustedKey]
+def list_keys() -> list[TrustedKey]:
     """Returns a list of TrustedKey instances for each key which is
     used to trust repositories.
     """
     # The output of `apt-key list` is difficult to parse since the
     # --with-colons parameter isn't user
-    output = _call_apt_key_script("adv", "--with-colons", "--batch",
-                                  "--fixed-list-mode", "--list-keys")
+    output = _call_apt_key_script(
+        "adv", "--with-colons", "--batch", "--fixed-list-mode", "--list-keys"
+    )
     res = []
     for line in output.split("\n"):
         fields = line.split(":")
