@@ -12,23 +12,15 @@
 #
 # This file is part of cloud-init. See LICENSE file for license information.
 
-# Skip isort on this file because of the patch that comes between imports
-# isort: skip_file
-
 import argparse
 import json
 import os
 import sys
 import time
 import traceback
+import logging
+from typing import Tuple
 
-from cloudinit import patcher
-from cloudinit.config.modules import Modules
-
-patcher.patch_logging()
-
-from cloudinit.config.schema import validate_cloudconfig_schema
-from cloudinit import log as logging
 from cloudinit import netinfo
 from cloudinit import signal_handler
 from cloudinit import sources
@@ -37,17 +29,16 @@ from cloudinit import url_helper
 from cloudinit import util
 from cloudinit import version
 from cloudinit import warnings
-
 from cloudinit import reporting
-from cloudinit.reporting import events
-
-from cloudinit.settings import PER_INSTANCE, PER_ALWAYS, PER_ONCE, CLOUD_CONFIG
-
 from cloudinit import atomic_helper
-
+from cloudinit.cmd.devel import read_cfg_paths
 from cloudinit.config import cc_set_hostname
-from cloudinit import dhclient_hook
-
+from cloudinit.config.modules import Modules
+from cloudinit.config.schema import validate_cloudconfig_schema
+from cloudinit import log
+from cloudinit.reporting import events
+from cloudinit.safeyaml import load
+from cloudinit.settings import PER_INSTANCE, PER_ALWAYS, PER_ONCE, CLOUD_CONFIG
 
 # Welcome message template
 WELCOME_MSG_TPL = (
@@ -66,7 +57,7 @@ FREQ_SHORT_NAMES = {
     "once": PER_ONCE,
 }
 
-LOG = logging.getLogger()
+LOG = logging.getLogger(__name__)
 
 
 # Used for when a logger may not be active
@@ -81,10 +72,17 @@ def print_exc(msg=""):
     sys.stderr.write("\n")
 
 
+def log_ppid():
+    if util.is_Linux():
+        ppid = os.getppid()
+        LOG.info("PID [%s] started cloud-init.", ppid)
+
+
 def welcome(action, msg=None):
     if not msg:
         msg = welcome_format(action)
     util.multi_log("%s\n" % (msg), console=False, stderr=True, log=LOG)
+    log_ppid()
     return msg
 
 
@@ -142,7 +140,7 @@ def parse_cmdline_url(cmdline, names=("cloud-config-url", "url")):
     raise KeyError("No keys (%s) found in string '%s'" % (cmdline, names))
 
 
-def attempt_cmdline_url(path, network=True, cmdline=None):
+def attempt_cmdline_url(path, network=True, cmdline=None) -> Tuple[int, str]:
     """Write data from url referenced in command line to path.
 
     path: a file to write content to if downloaded.
@@ -169,7 +167,7 @@ def attempt_cmdline_url(path, network=True, cmdline=None):
     except KeyError:
         return (logging.DEBUG, "No kernel command line url found.")
 
-    path_is_local = url.startswith("file://") or url.startswith("/")
+    path_is_local = url.startswith(("file://", "/"))
 
     if path_is_local and os.path.exists(path):
         if network:
@@ -189,7 +187,7 @@ def attempt_cmdline_url(path, network=True, cmdline=None):
 
         return (level, m)
 
-    kwargs = {"url": url, "timeout": 10, "retries": 2}
+    kwargs = {"url": url, "timeout": 10, "retries": 2, "stream": True}
     if network or path_is_local:
         level = logging.WARN
         kwargs["sec_between"] = 1
@@ -201,22 +199,49 @@ def attempt_cmdline_url(path, network=True, cmdline=None):
     header = b"#cloud-config"
     try:
         resp = url_helper.read_file_or_url(**kwargs)
+        sniffed_content = b""
         if resp.ok():
-            data = resp.contents
-            if not resp.contents.startswith(header):
+            is_cloud_cfg = True
+            if isinstance(resp, url_helper.UrlResponse):
+                try:
+                    sniffed_content += next(
+                        resp.iter_content(chunk_size=len(header))
+                    )
+                except StopIteration:
+                    pass
+                if not sniffed_content.startswith(header):
+                    is_cloud_cfg = False
+            elif not resp.contents.startswith(header):
+                is_cloud_cfg = False
+            if is_cloud_cfg:
+                if cmdline_name == "url":
+                    return (
+                        log.DEPRECATED,
+                        str(
+                            util.deprecate(
+                                deprecated="The kernel command line key `url`",
+                                deprecated_version="22.3",
+                                extra_message=" Please use `cloud-config-url` "
+                                "kernel command line parameter instead",
+                                return_log=True,
+                            ),
+                        ),
+                    )
+            else:
                 if cmdline_name == "cloud-config-url":
                     level = logging.WARN
                 else:
                     level = logging.INFO
                 return (
                     level,
-                    "contents of '%s' did not start with %s" % (url, header),
+                    f"contents of '{url}' did not start with {str(header)}",
                 )
         else:
             return (
                 level,
                 "url '%s' returned code %s. Ignoring." % (url, resp.code),
             )
+        data = sniffed_content + resp.contents
 
     except url_helper.UrlError as e:
         return (level, "retrieving url '%s' failed: %s" % (url, e))
@@ -316,11 +341,11 @@ def main_init(name, args):
         LOG.debug(
             "Logging being reset, this logger may no longer be active shortly"
         )
-        logging.resetLogging()
-    logging.setupLogging(init.cfg)
+        log.reset_logging()
+    log.setup_logging(init.cfg)
     apply_reporting_cfg(init.cfg)
 
-    # Any log usage prior to setupLogging above did not have local user log
+    # Any log usage prior to setup_logging above did not have local user log
     # config applied.  We send the welcome message now, as stderr/out have
     # been redirected and log now configured.
     welcome(name, msg=w_msg)
@@ -453,8 +478,14 @@ def main_init(name, args):
         return (init.datasource, ["Consuming user data failed!"])
 
     # Validate user-data adheres to schema definition
-    if os.path.exists(init.paths.get_ipath_cur("userdata_raw")):
-        validate_cloudconfig_schema(config=init.cfg, strict=False)
+    cloud_cfg_path = init.paths.get_ipath_cur("cloud_config")
+    if os.path.exists(cloud_cfg_path) and os.stat(cloud_cfg_path).st_size != 0:
+        validate_cloudconfig_schema(
+            config=load(util.load_text_file(cloud_cfg_path)),
+            strict=False,
+            log_details=False,
+            log_deprecations=True,
+        )
     else:
         LOG.debug("Skipping user-data validation. No user-data found.")
 
@@ -472,7 +503,7 @@ def main_init(name, args):
             (outfmt, errfmt) = util.fixup_output(mods.cfg, name)
     except Exception:
         util.logexc(LOG, "Failed to re-adjust output redirection!")
-    logging.setupLogging(mods.cfg)
+    log.setup_logging(mods.cfg)
 
     # give the activated datasource a chance to adjust
     init.activate_datasource()
@@ -577,8 +608,8 @@ def main_modules(action_name, args):
         LOG.debug(
             "Logging being reset, this logger may no longer be active shortly"
         )
-        logging.resetLogging()
-    logging.setupLogging(mods.cfg)
+        log.reset_logging()
+    log.setup_logging(mods.cfg)
     apply_reporting_cfg(init.cfg)
 
     # now that logging is setup and stdout redirected, send welcome
@@ -639,8 +670,8 @@ def main_single(name, args):
         LOG.debug(
             "Logging being reset, this logger may no longer be active shortly"
         )
-        logging.resetLogging()
-    logging.setupLogging(mods.cfg)
+        log.reset_logging()
+    log.setup_logging(mods.cfg)
     apply_reporting_cfg(init.cfg)
 
     # now that logging is setup and stdout redirected, send welcome
@@ -661,7 +692,8 @@ def main_single(name, args):
 
 def status_wrapper(name, args, data_d=None, link_d=None):
     if data_d is None:
-        data_d = os.path.normpath("/var/lib/cloud/data")
+        paths = read_cfg_paths()
+        data_d = paths.get_cpath("data")
     if link_d is None:
         link_d = os.path.normpath("/run/cloud-init")
 
@@ -669,6 +701,7 @@ def status_wrapper(name, args, data_d=None, link_d=None):
     status_link = os.path.join(link_d, "status.json")
     result_path = os.path.join(data_d, "result.json")
     result_link = os.path.join(link_d, "result.json")
+    root_logger = logging.getLogger()
 
     util.ensure_dirs(
         (
@@ -692,7 +725,6 @@ def status_wrapper(name, args, data_d=None, link_d=None):
     modes = (
         "init",
         "init-local",
-        "modules-init",
         "modules-config",
         "modules-final",
     )
@@ -707,7 +739,7 @@ def status_wrapper(name, args, data_d=None, link_d=None):
             util.del_file(f)
     else:
         try:
-            status = json.loads(util.load_file(status_path))
+            status = json.loads(util.load_text_file(status_path))
         except Exception:
             pass
 
@@ -728,7 +760,11 @@ def status_wrapper(name, args, data_d=None, link_d=None):
     v1 = status["v1"]
     v1["stage"] = mode
     v1[mode]["start"] = time.time()
+    v1[mode]["recoverable_errors"] = next(
+        filter(lambda h: isinstance(h, log.LogExporter), root_logger.handlers)
+    ).export_logs()
 
+    # Write status.json prior to running init / module code
     atomic_helper.write_json(status_path, status)
     util.sym_link(
         os.path.relpath(status_path, link_d), status_link, force=True
@@ -753,6 +789,10 @@ def status_wrapper(name, args, data_d=None, link_d=None):
     v1[mode]["finished"] = time.time()
     v1["stage"] = None
 
+    # Write status.json after running init / module code
+    v1[mode]["recoverable_errors"] = next(
+        filter(lambda h: isinstance(h, log.LogExporter), root_logger.handlers)
+    ).export_logs()
     atomic_helper.write_json(status_path, status)
 
     if mode == "modules-final":
@@ -773,29 +813,27 @@ def status_wrapper(name, args, data_d=None, link_d=None):
     return len(v1[mode]["errors"])
 
 
-def _maybe_persist_instance_data(init):
+def _maybe_persist_instance_data(init: stages.Init):
     """Write instance-data.json file if absent and datasource is restored."""
-    if init.ds_restored:
-        instance_data_file = os.path.join(
-            init.paths.run_dir, sources.INSTANCE_JSON_FILE
-        )
+    if init.datasource and init.ds_restored:
+        instance_data_file = init.paths.get_runpath("instance_data")
         if not os.path.exists(instance_data_file):
-            init.datasource.persist_instance_data()
+            init.datasource.persist_instance_data(write_cache=False)
 
 
 def _maybe_set_hostname(init, stage, retry_stage):
-    """Call set-hostname if metadata, vendordata or userdata provides it.
+    """Call set_hostname if metadata, vendordata or userdata provides it.
 
     @param stage: String representing current stage in which we are running.
     @param retry_stage: String represented logs upon error setting hostname.
     """
     cloud = init.cloudify()
-    (hostname, _fqdn) = util.get_hostname_fqdn(
+    (hostname, _fqdn, _) = util.get_hostname_fqdn(
         init.cfg, cloud, metadata_only=True
     )
     if hostname:  # meta-data or user-data hostname content
         try:
-            cc_set_hostname.handle("set-hostname", init.cfg, cloud, LOG, None)
+            cc_set_hostname.handle("set_hostname", init.cfg, cloud, None)
         except cc_set_hostname.SetHostnameError as e:
             LOG.debug(
                 "Failed setting hostname in %s stage. Will"
@@ -811,6 +849,7 @@ def main_features(name, args):
 
 
 def main(sysv_args=None):
+    log.configure_root_logger()
     if not sysv_args:
         sysv_args = sys.argv
     parser = argparse.ArgumentParser(prog=sysv_args.pop(0))
@@ -823,14 +862,6 @@ def main(sysv_args=None):
         version="%(prog)s " + (version.version_string()),
         help="Show program's version number and exit.",
     )
-    parser.add_argument(
-        "--file",
-        "-f",
-        action="append",
-        dest="files",
-        help="Use additional yaml configuration files.",
-        type=argparse.FileType("rb"),
-    )
     parser.add_argument(
         "--debug",
         "-d",
@@ -864,6 +895,14 @@ def main(sysv_args=None):
         help="Start in local mode (default: %(default)s).",
         default=False,
     )
+    parser_init.add_argument(
+        "--file",
+        "-f",
+        action="append",
+        dest="files",
+        help="Use additional yaml configuration files.",
+        type=argparse.FileType("rb"),
+    )
     # This is used so that we can know which action is selected +
     # the functor to use to run this subcommand
     parser_init.set_defaults(action=("init", main_init))
@@ -880,6 +919,14 @@ def main(sysv_args=None):
         default="config",
         choices=("init", "config", "final"),
     )
+    parser_mod.add_argument(
+        "--file",
+        "-f",
+        action="append",
+        dest="files",
+        help="Use additional yaml configuration files.",
+        type=argparse.FileType("rb"),
+    )
     parser_mod.set_defaults(action=("modules", main_modules))
 
     # This subcommand allows you to run a single module
@@ -890,13 +937,13 @@ def main(sysv_args=None):
         "--name",
         "-n",
         action="store",
-        help="module name to run",
+        help="Module name to run.",
         required=True,
     )
     parser_single.add_argument(
         "--frequency",
         action="store",
-        help="Set module frequency.",
+        help="Module frequency for this run.",
         required=False,
         choices=list(FREQ_SHORT_NAMES.keys()),
     )
@@ -912,6 +959,14 @@ def main(sysv_args=None):
         metavar="argument",
         help="Any additional arguments to pass to this module.",
     )
+    parser_single.add_argument(
+        "--file",
+        "-f",
+        action="append",
+        dest="files",
+        help="Use additional yaml configuration files.",
+        type=argparse.FileType("rb"),
+    )
     parser_single.set_defaults(action=("single", main_single))
 
     parser_query = subparsers.add_parser(
@@ -919,11 +974,6 @@ def main(sysv_args=None):
         help="Query standardized instance metadata from the command line.",
     )
 
-    parser_dhclient = subparsers.add_parser(
-        dhclient_hook.NAME, help=dhclient_hook.__doc__
-    )
-    dhclient_hook.get_parser(parser_dhclient)
-
     parser_features = subparsers.add_parser(
         "features", help="List defined features."
     )
@@ -955,9 +1005,12 @@ def main(sysv_args=None):
 
     if sysv_args:
         # Only load subparsers if subcommand is specified to avoid load cost
-        subcommand = sysv_args[0]
+        subcommand = next(
+            (posarg for posarg in sysv_args if not posarg.startswith("-")),
+            None,
+        )
         if subcommand == "analyze":
-            from cloudinit.analyze.__main__ import get_parser as analyze_parser
+            from cloudinit.analyze import get_parser as analyze_parser
 
             # Construct analyze subcommand parser
             analyze_parser(parser_analyze)
@@ -1014,15 +1067,25 @@ def main(sysv_args=None):
     # Subparsers.required = True and each subparser sets action=(name, functor)
     (name, functor) = args.action
 
-    # Setup basic logging to start (until reinitialized)
-    # iff in debug mode.
-    if args.debug:
-        logging.setupBasicLogging()
+    # Setup basic logging for cloud-init:
+    # - for cloud-init stages if --debug
+    # - for all other subcommands:
+    #   - if --debug is passed, logging.DEBUG
+    #   - if --debug is not passed, logging.WARNING
+    if name not in ("init", "modules"):
+        log.setup_basic_logging(
+            logging.DEBUG if args.debug else logging.WARNING
+        )
+    elif args.debug:
+        log.setup_basic_logging()
 
     # Setup signal handlers before running
     signal_handler.attach_handlers()
 
-    if name in ("modules", "init"):
+    # Write boot stage data to write status.json and result.json
+    # Exclude modules --mode=init, since it is not a real boot stage and
+    # should not be written into status.json
+    if "init" == name or ("modules" == name and "init" != args.mode):
         functor = status_wrapper
 
     rname = None
@@ -1063,8 +1126,8 @@ def main(sysv_args=None):
             func=functor,
             args=(name, args),
         )
-        reporting.flush_events()
-        return retval
+    reporting.flush_events()
+    return retval
 
 
 if __name__ == "__main__":
