@@ -4,14 +4,13 @@
 
 """CA Certs: Add ca certificates."""
 
+import logging
 import os
-from textwrap import dedent
 
-from cloudinit import log as logging
 from cloudinit import subp, util
 from cloudinit.cloud import Cloud
 from cloudinit.config import Config
-from cloudinit.config.schema import MetaSchema, get_meta_doc
+from cloudinit.config.schema import MetaSchema
 from cloudinit.settings import PER_INSTANCE
 
 LOG = logging.getLogger(__name__)
@@ -45,6 +44,13 @@ DISTRO_OVERRIDES = {
         "ca_cert_config": None,
         "ca_cert_update_cmd": ["update-ca-certificates"],
     },
+    "photon": {
+        "ca_cert_path": "/etc/pki/tls/certs/",
+        "ca_cert_local_path": "/etc/ssl/certs/",
+        "ca_cert_filename": "cloud-init-ca-cert-{cert_index}.pem",
+        "ca_cert_config": None,
+        "ca_cert_update_cmd": ["rehash_ca_certificates.sh"],
+    },
 }
 
 for distro in (
@@ -57,21 +63,15 @@ for distro in (
 ):
     DISTRO_OVERRIDES[distro] = DISTRO_OVERRIDES["opensuse"]
 
-MODULE_DESCRIPTION = """\
-This module adds CA certificates to the system's CA store and updates any
-related files using the appropriate OS-specific utility. The default CA
-certificates can be disabled/deleted from use by the system with the
-configuration option ``remove_defaults``.
+for distro in (
+    "almalinux",
+    "cloudlinux",
+):
+    DISTRO_OVERRIDES[distro] = DISTRO_OVERRIDES["rhel"]
 
-.. note::
-    certificates must be specified using valid yaml. in order to specify a
-    multiline certificate, the yaml multiline list syntax must be used
-
-.. note::
-    Alpine Linux requires the ca-certificates package to be installed in
-    order to provide the ``update-ca-certificates`` command.
-"""
 distros = [
+    "almalinux",
+    "cloudlinux",
     "alpine",
     "debian",
     "fedora",
@@ -84,33 +84,15 @@ distros = [
     "sle-micro",
     "sles",
     "ubuntu",
+    "photon",
 ]
 
 meta: MetaSchema = {
     "id": "cc_ca_certs",
-    "name": "CA Certificates",
-    "title": "Add ca certificates",
-    "description": MODULE_DESCRIPTION,
     "distros": distros,
     "frequency": PER_INSTANCE,
-    "examples": [
-        dedent(
-            """\
-            ca_certs:
-              remove_defaults: true
-              trusted:
-                - single_line_cert
-                - |
-                  -----BEGIN CERTIFICATE-----
-                  YOUR-ORGS-TRUSTED-CA-CERT-HERE
-                  -----END CERTIFICATE-----
-            """
-        )
-    ],
     "activate_by_schema_keys": ["ca_certs", "ca-certs"],
-}
-
-__doc__ = get_meta_doc(meta)
+}  # type: ignore
 
 
 def _distro_ca_certs_configs(distro_name):
@@ -165,7 +147,7 @@ def disable_default_ca_certs(distro_name, distro_cfg):
     @param distro_name: String providing the distro class name.
     @param distro_cfg: A hash providing _distro_ca_certs_configs function.
     """
-    if distro_name == "rhel":
+    if distro_name in ["rhel", "photon"]:
         remove_default_ca_certs(distro_cfg)
     elif distro_name in ["alpine", "debian", "ubuntu"]:
         disable_system_ca_certs(distro_cfg)
@@ -174,7 +156,7 @@ def disable_default_ca_certs(distro_name, distro_cfg):
             debconf_sel = (
                 "ca-certificates ca-certificates/trust_new_crts " + "select no"
             )
-            subp.subp(("debconf-set-selections", "-"), debconf_sel)
+            subp.subp(("debconf-set-selections", "-"), data=debconf_sel)
 
 
 def disable_system_ca_certs(distro_cfg):
@@ -197,7 +179,7 @@ def disable_system_ca_certs(distro_cfg):
     added_header = False
 
     if os.stat(ca_cert_cfg_fn).st_size:
-        orig = util.load_file(ca_cert_cfg_fn)
+        orig = util.load_text_file(ca_cert_cfg_fn)
         out_lines = []
         for line in orig.splitlines():
             if line == header_comment:
@@ -285,6 +267,3 @@ def handle(name: str, cfg: Config, cloud: Cloud, args: list) -> None:
     # Update the system with the new cert configuration.
     LOG.debug("Updating certificates")
     update_ca_certs(distro_cfg)
-
-
-# vi: ts=4 expandtab
