@@ -7,12 +7,11 @@
 #
 # This file is part of cloud-init. See LICENSE file for license information.
 
-""" Mcollective: Install, configure and start mcollective"""
+"""Mcollective: Install, configure and start mcollective"""
 
 import errno
 import io
 import logging
-from textwrap import dedent
 
 # Used since this can maintain comments
 # and doesn't need a top level section
@@ -21,71 +20,20 @@ from configobj import ConfigObj
 from cloudinit import subp, util
 from cloudinit.cloud import Cloud
 from cloudinit.config import Config
-from cloudinit.config.schema import MetaSchema, get_meta_doc
+from cloudinit.config.schema import MetaSchema
 from cloudinit.settings import PER_INSTANCE
 
 PUBCERT_FILE = "/etc/mcollective/ssl/server-public.pem"
 PRICERT_FILE = "/etc/mcollective/ssl/server-private.pem"
 SERVER_CFG = "/etc/mcollective/server.cfg"
 
-MODULE_DESCRIPTION = """\
-This module installs, configures and starts mcollective. If the ``mcollective``
-key is present in config, then mcollective will be installed and started.
-
-Configuration for ``mcollective`` can be specified in the ``conf`` key under
-``mcollective``. Each config value consists of a key value pair and will be
-written to ``/etc/mcollective/server.cfg``. The ``public-cert`` and
-``private-cert`` keys, if present in conf may be used to specify the public and
-private certificates for mcollective. Their values will be written to
-``/etc/mcollective/ssl/server-public.pem`` and
-``/etc/mcollective/ssl/server-private.pem``.
-
-.. note::
-    The ec2 metadata service is readable by non-root users.
-    If security is a concern, use include-once and ssl urls.
-"""
-
-distros = ["all"]
-
 meta: MetaSchema = {
     "id": "cc_mcollective",
-    "name": "Mcollective",
-    "title": "Install, configure and start mcollective",
-    "description": MODULE_DESCRIPTION,
-    "distros": distros,
-    "examples": [
-        dedent(
-            """\
-            # Provide server private and public key and provide the following
-            # config settings in /etc/mcollective/server.cfg:
-            # loglevel: debug
-            # plugin.stomp.host: dbhost
-
-            # WARNING WARNING WARNING
-            # The ec2 metadata service is a network service, and thus is
-            # readable by non-root users on the system
-            # (ie: 'ec2metadata --user-data')
-            # If you want security for this, please use include-once + SSL urls
-            mcollective:
-              conf:
-                loglevel: debug
-                plugin.stomp.host: dbhost
-                public-cert: |
-                    -------BEGIN CERTIFICATE--------
-                    <cert data>
-                    -------END CERTIFICATE--------
-                private-cert: |
-                    -------BEGIN CERTIFICATE--------
-                    <cert data>
-                    -------END CERTIFICATE--------
-            """
-        ),
-    ],
+    "distros": ["all"],
     "frequency": PER_INSTANCE,
     "activate_by_schema_keys": ["mcollective"],
-}
+}  # type: ignore
 
-__doc__ = get_meta_doc(meta)
 LOG = logging.getLogger(__name__)
 
 
@@ -98,7 +46,7 @@ def configure(
     # Read server.cfg (if it exists) values from the
     # original file in order to be able to mix the rest up.
     try:
-        old_contents = util.load_file(server_cfg, quiet=False, decode=False)
+        old_contents = util.load_binary_file(server_cfg, quiet=False)
         mcollective_config = ConfigObj(io.BytesIO(old_contents))
     except IOError as e:
         if e.errno != errno.ENOENT:
