@@ -9,12 +9,12 @@
 import copy
 import logging
 import os
-from typing import Dict, Mapping
+from textwrap import dedent
 
 from cloudinit import subp, temp_utils, templater, type_utils, util
 from cloudinit.cloud import Cloud
 from cloudinit.config import Config
-from cloudinit.config.schema import MetaSchema
+from cloudinit.config.schema import MetaSchema, get_meta_doc
 from cloudinit.settings import PER_INSTANCE
 
 LOG = logging.getLogger(__name__)
@@ -25,8 +25,6 @@ NR_POOL_SERVERS = 4
 distros = [
     "almalinux",
     "alpine",
-    "aosc",
-    "azurelinux",
     "centos",
     "cloudlinux",
     "cos",
@@ -45,7 +43,6 @@ distros = [
     "opensuse-tumbleweed",
     "opensuse-leap",
     "photon",
-    "raspberry-pi-os",
     "rhel",
     "rocky",
     "sle_hpc",
@@ -100,7 +97,7 @@ NTP_CLIENT_CONFIG = {
 }
 
 # This is Distro-specific configuration overrides of the base config
-DISTRO_CLIENT_CONFIG: Dict[str, Dict] = {
+DISTRO_CLIENT_CONFIG = {
     "alpine": {
         "chrony": {
             "confpath": "/etc/chrony/chrony.conf",
@@ -112,21 +109,6 @@ DISTRO_CLIENT_CONFIG: Dict[str, Dict] = {
             "service_name": "ntpd",
         },
     },
-    "aosc": {
-        "systemd-timesyncd": {
-            "check_exe": "/usr/lib/systemd/systemd-timesyncd",
-            "confpath": "/etc/systemd/timesyncd.conf",
-        },
-    },
-    "azurelinux": {
-        "chrony": {
-            "service_name": "chronyd",
-        },
-        "systemd-timesyncd": {
-            "check_exe": "/usr/lib/systemd/systemd-timesyncd",
-            "confpath": "/etc/systemd/timesyncd.conf",
-        },
-    },
     "centos": {
         "ntp": {
             "service_name": "ntpd",
@@ -212,11 +194,6 @@ DISTRO_CLIENT_CONFIG: Dict[str, Dict] = {
             "confpath": "/etc/systemd/timesyncd.conf",
         },
     },
-    "raspberry-pi-os": {
-        "chrony": {
-            "confpath": "/etc/chrony/chrony.conf",
-        },
-    },
     "rhel": {
         "ntp": {
             "service_name": "ntpd",
@@ -247,9 +224,6 @@ DISTRO_CLIENT_CONFIG: Dict[str, Dict] = {
 for distro in ("opensuse-microos", "opensuse-tumbleweed", "opensuse-leap"):
     DISTRO_CLIENT_CONFIG[distro] = DISTRO_CLIENT_CONFIG["opensuse"]
 
-for distro in ("almalinux", "cloudlinux", "rocky"):
-    DISTRO_CLIENT_CONFIG[distro] = DISTRO_CLIENT_CONFIG["rhel"]
-
 for distro in ("sle_hpc", "sle-micro"):
     DISTRO_CLIENT_CONFIG[distro] = DISTRO_CLIENT_CONFIG["sles"]
 
@@ -261,10 +235,77 @@ for distro in ("sle_hpc", "sle-micro"):
 
 meta: MetaSchema = {
     "id": "cc_ntp",
+    "name": "NTP",
+    "title": "enable and configure ntp",
+    "description": dedent(
+        """\
+        Handle ntp configuration. If ntp is not installed on the system and
+        ntp configuration is specified, ntp will be installed. If there is a
+        default ntp config file in the image or one is present in the
+        distro's ntp package, it will be copied to a file with ``.dist``
+        appended to the filename before any changes are made. A list of ntp
+        pools and ntp servers can be provided under the ``ntp`` config key.
+        If no ntp ``servers`` or ``pools`` are provided, 4 pools will be used
+        in the format ``{0-3}.{distro}.pool.ntp.org``."""
+    ),
     "distros": distros,
+    "examples": [
+        dedent(
+            """\
+        # Override ntp with chrony configuration on Ubuntu
+        ntp:
+          enabled: true
+          ntp_client: chrony  # Uses cloud-init default chrony configuration
+        """
+        ),
+        dedent(
+            """\
+        # Provide a custom ntp client configuration
+        ntp:
+          enabled: true
+          ntp_client: myntpclient
+          config:
+             confpath: /etc/myntpclient/myntpclient.conf
+             check_exe: myntpclientd
+             packages:
+               - myntpclient
+             service_name: myntpclient
+             template: |
+                 ## template:jinja
+                 # My NTP Client config
+                 {% if pools -%}# pools{% endif %}
+                 {% for pool in pools -%}
+                 pool {{pool}} iburst
+                 {% endfor %}
+                 {%- if servers %}# servers
+                 {% endif %}
+                 {% for server in servers -%}
+                 server {{server}} iburst
+                 {% endfor %}
+                 {% if peers -%}# peers{% endif %}
+                 {% for peer in peers -%}
+                 peer {{peer}}
+                 {% endfor %}
+                 {% if allow -%}# allow{% endif %}
+                 {% for cidr in allow -%}
+                 allow {{cidr}}
+                 {% endfor %}
+          pools: [0.int.pool.ntp.org, 1.int.pool.ntp.org, ntp.myorg.org]
+          servers:
+            - ntp.server.local
+            - ntp.ubuntu.com
+            - 192.168.23.2
+          allow:
+            - 192.168.23.0/32
+          peers:
+            - km001
+            - km002"""
+        ),
+    ],
     "frequency": PER_INSTANCE,
     "activate_by_schema_keys": ["ntp"],
 }
+__doc__ = get_meta_doc(meta)
 
 
 REQUIRED_NTP_CONFIG_KEYS = frozenset(
@@ -286,7 +327,7 @@ def distro_ntp_client_configs(distro):
     return cfg
 
 
-def select_ntp_client(ntp_client, distro) -> Mapping:
+def select_ntp_client(ntp_client, distro):
     """Determine which ntp client is to be used, consulting the distro
        for its preference.
 
@@ -325,7 +366,7 @@ def select_ntp_client(ntp_client, distro) -> Mapping:
                 'Selected distro preferred NTP client "%s", not yet installed',
                 client,
             )
-            clientcfg = distro_cfg.get(client, {})
+            clientcfg = distro_cfg.get(client)
     else:
         LOG.debug(
             'Selected NTP client "%s" via distro system config',
@@ -433,14 +474,18 @@ def write_ntp_config_template(
     if not peers:
         peers = []
 
-    if not servers and not pools and distro_name == "cos":
+    if len(servers) == 0 and len(pools) == 0 and distro_name == "cos":
         return
-    if not servers and distro_name == "alpine" and service_name == "ntpd":
+    if (
+        len(servers) == 0
+        and distro_name == "alpine"
+        and service_name == "ntpd"
+    ):
         # Alpine's Busybox ntpd only understands "servers" configuration
         # and not "pool" configuration.
         servers = generate_server_names(distro_name)
         LOG.debug("Adding distro default ntp servers: %s", ",".join(servers))
-    elif not (servers) and not (pools):
+    elif len(servers) == 0 and len(pools) == 0:
         pools = generate_server_names(distro_name)
         LOG.debug(
             "Adding distro default ntp pool servers: %s", ",".join(pools)
@@ -568,7 +613,7 @@ def handle(name: str, cfg: Config, cloud: Cloud, args: list) -> None:
 
     template_fn = None
     if not ntp_client_config.get("template"):
-        template_name = ntp_client_config["template_name"].replace(
+        template_name = ntp_client_config.get("template_name").replace(
             "{distro}", cloud.distro.name
         )
         template_fn = cloud.get_template_filename(template_name)
@@ -614,14 +659,14 @@ def handle(name: str, cfg: Config, cloud: Cloud, args: list) -> None:
 
         try:
             cloud.distro.manage_service(
-                "enable", ntp_client_config["service_name"]
+                "enable", ntp_client_config.get("service_name")
             )
         except subp.ProcessExecutionError as e:
             LOG.exception("Failed to enable ntp service: %s", e)
             raise
     try:
         cloud.distro.manage_service(
-            "reload", ntp_client_config["service_name"]
+            "reload", ntp_client_config.get("service_name")
         )
     except subp.ProcessExecutionError as e:
         LOG.exception("Failed to reload/start ntp service: %s", e)
