@@ -6,19 +6,20 @@
 
 """Package Update Upgrade Install: update, upgrade, and install packages"""
 
+import logging
 import os
 import time
 from textwrap import dedent
 
-from cloudinit import log as logging
 from cloudinit import subp, util
 from cloudinit.cloud import Cloud
 from cloudinit.config import Config
 from cloudinit.config.schema import MetaSchema, get_meta_doc
 from cloudinit.distros import ALL_DISTROS
+from cloudinit.log import flush_loggers
 from cloudinit.settings import PER_INSTANCE
 
-REBOOT_FILE = "/var/run/reboot-required"
+REBOOT_FILES = ("/var/run/reboot-required", "/run/reboot-needed")
 REBOOT_CMD = ["/sbin/reboot"]
 
 MODULE_DESCRIPTION = """\
@@ -43,6 +44,12 @@ meta: MetaSchema = {
               - pwgen
               - pastebinit
               - [libpython3.8, 3.8.10-0ubuntu1~20.04.2]
+              - snap:
+                - certbot
+                - [juju, --edge]
+                - [lxd, --channel=5.15/stable]
+              - apt:
+                - mg
             package_update: true
             package_upgrade: true
             package_reboot_if_required: true
@@ -95,7 +102,7 @@ def handle(name: str, cfg: Config, cloud: Cloud, args: list) -> None:
     pkglist = util.get_cfg_option_list(cfg, "packages", [])
 
     errors = []
-    if update or len(pkglist) or upgrade:
+    if update or upgrade:
         try:
             cloud.distro.update_package_sources()
         except Exception as e:
@@ -113,21 +120,26 @@ def handle(name: str, cfg: Config, cloud: Cloud, args: list) -> None:
         try:
             cloud.distro.install_packages(pkglist)
         except Exception as e:
-            util.logexc(LOG, "Failed to install packages: %s", pkglist)
+            util.logexc(
+                LOG, "Failure when attempting to install packages: %s", pkglist
+            )
             errors.append(e)
 
     # TODO(smoser): handle this less violently
     # kernel and openssl (possibly some other packages)
     # write a file /var/run/reboot-required after upgrading.
     # if that file exists and configured, then just stop right now and reboot
-    reboot_fn_exists = os.path.isfile(REBOOT_FILE)
+    for reboot_marker in REBOOT_FILES:
+        reboot_fn_exists = os.path.isfile(reboot_marker)
+        if reboot_fn_exists:
+            break
     if (upgrade or pkglist) and reboot_if_required and reboot_fn_exists:
         try:
             LOG.warning(
-                "Rebooting after upgrade or install per %s", REBOOT_FILE
+                "Rebooting after upgrade or install per %s", reboot_marker
             )
             # Flush the above warning + anything else out...
-            logging.flushLoggers(LOG)
+            flush_loggers(LOG)
             _fire_reboot()
         except Exception as e:
             util.logexc(LOG, "Requested reboot did not happen!")
@@ -138,6 +150,3 @@ def handle(name: str, cfg: Config, cloud: Cloud, args: list) -> None:
             "%s failed with exceptions, re-raising the last one", len(errors)
         )
         raise errors[-1]
-
-
-# vi: ts=4 expandtab
