@@ -8,17 +8,16 @@
 
 """Phone Home: Post data to url"""
 
-from textwrap import dedent
+import logging
 
 from cloudinit import templater, url_helper, util
-from cloudinit.config.schema import MetaSchema, get_meta_doc
+from cloudinit.cloud import Cloud
+from cloudinit.config import Config
+from cloudinit.config.schema import MetaSchema
 from cloudinit.distros import ALL_DISTROS
 from cloudinit.settings import PER_INSTANCE
 
-frequency = PER_INSTANCE
-
 POST_LIST_ALL = [
-    "pub_key_dsa",
     "pub_key_rsa",
     "pub_key_ecdsa",
     "pub_key_ed25519",
@@ -27,71 +26,14 @@ POST_LIST_ALL = [
     "fqdn",
 ]
 
-MODULE_DESCRIPTION = """\
-This module can be used to post data to a remote host after boot is complete.
-If the post url contains the string ``$INSTANCE_ID`` it will be replaced with
-the id of the current instance. Either all data can be posted or a list of
-keys to post. Available keys are:
-
-    - ``pub_key_dsa``
-    - ``pub_key_rsa``
-    - ``pub_key_ecdsa``
-    - ``pub_key_ed25519``
-    - ``instance_id``
-    - ``hostname``
-    - ``fdqn``
-
-Data is sent as ``x-www-form-urlencoded`` arguments.
-
-**Example HTTP POST**:
-
-.. code-block:: http
-
-    POST / HTTP/1.1
-    Content-Length: 1337
-    User-Agent: Cloud-Init/21.4
-    Accept-Encoding: gzip, deflate
-    Accept: */*
-    Content-Type: application/x-www-form-urlencoded
-
-    pub_key_dsa=dsa_contents&pub_key_rsa=rsa_contents&pub_key_ecdsa=ecdsa_contents&pub_key_ed25519=ed25519_contents&instance_id=i-87018aed&hostname=myhost&fqdn=myhost.internal
-"""
-
 meta: MetaSchema = {
     "id": "cc_phone_home",
-    "name": "Phone Home",
-    "title": "Post data to url",
-    "description": MODULE_DESCRIPTION,
     "distros": [ALL_DISTROS],
     "frequency": PER_INSTANCE,
-    "examples": [
-        dedent(
-            """\
-            phone_home:
-                url: http://example.com/$INSTANCE_ID/
-                post: all
-            """
-        ),
-        dedent(
-            """\
-            phone_home:
-                url: http://example.com/$INSTANCE_ID/
-                post:
-                    - pub_key_dsa
-                    - pub_key_rsa
-                    - pub_key_ecdsa
-                    - pub_key_ed25519
-                    - instance_id
-                    - hostname
-                    - fqdn
-                tries: 5
-            """
-        ),
-    ],
-}
-
-__doc__ = get_meta_doc(meta)
+    "activate_by_schema_keys": ["phone_home"],
+}  # type: ignore
 
+LOG = logging.getLogger(__name__)
 # phone_home:
 #  url: http://my.foo.bar/$INSTANCE/
 #  post: all
@@ -99,17 +41,17 @@ __doc__ = get_meta_doc(meta)
 #
 # phone_home:
 #  url: http://my.foo.bar/$INSTANCE_ID/
-#  post: [ pub_key_dsa, pub_key_rsa, pub_key_ecdsa, instance_id, hostname,
+#  post: [ pub_key_rsa, pub_key_ecdsa, instance_id, hostname,
 #          fqdn ]
 #
 
 
-def handle(name, cfg, cloud, log, args):
+def handle(name: str, cfg: Config, cloud: Cloud, args: list) -> None:
     if len(args) != 0:
         ph_cfg = util.read_conf(args[0])
     else:
         if "phone_home" not in cfg:
-            log.debug(
+            LOG.debug(
                 "Skipping module named %s, "
                 "no 'phone_home' configuration found",
                 name,
@@ -118,7 +60,7 @@ def handle(name, cfg, cloud, log, args):
         ph_cfg = cfg["phone_home"]
 
     if "url" not in ph_cfg:
-        log.warning(
+        LOG.warning(
             "Skipping module named %s, "
             "no 'url' found in 'phone_home' configuration",
             name,
@@ -133,7 +75,7 @@ def handle(name, cfg, cloud, log, args):
     except (ValueError, TypeError):
         tries = 10
         util.logexc(
-            log,
+            LOG,
             "Configuration entry 'tries' is not an integer, using %s instead",
             tries,
         )
@@ -143,23 +85,22 @@ def handle(name, cfg, cloud, log, args):
 
     all_keys = {
         "instance_id": cloud.get_instance_id(),
-        "hostname": cloud.get_hostname(),
-        "fqdn": cloud.get_hostname(fqdn=True),
+        "hostname": cloud.get_hostname().hostname,
+        "fqdn": cloud.get_hostname(fqdn=True).hostname,
     }
 
     pubkeys = {
-        "pub_key_dsa": "/etc/ssh/ssh_host_dsa_key.pub",
         "pub_key_rsa": "/etc/ssh/ssh_host_rsa_key.pub",
         "pub_key_ecdsa": "/etc/ssh/ssh_host_ecdsa_key.pub",
         "pub_key_ed25519": "/etc/ssh/ssh_host_ed25519_key.pub",
     }
 
-    for (n, path) in pubkeys.items():
+    for n, path in pubkeys.items():
         try:
-            all_keys[n] = util.load_file(path)
+            all_keys[n] = util.load_text_file(path)
         except Exception:
             util.logexc(
-                log, "%s: failed to open, can not phone home that data!", path
+                LOG, "%s: failed to open, can not phone home that data!", path
             )
 
     submit_keys = {}
@@ -168,7 +109,7 @@ def handle(name, cfg, cloud, log, args):
             submit_keys[k] = all_keys[k]
         else:
             submit_keys[k] = None
-            log.warning(
+            LOG.warning(
                 "Requested key %s from 'post'"
                 " configuration list not available",
                 k,
@@ -176,7 +117,7 @@ def handle(name, cfg, cloud, log, args):
 
     # Get them read to be posted
     real_submit_keys = {}
-    for (k, v) in submit_keys.items():
+    for k, v in submit_keys.items():
         if v is None:
             real_submit_keys[k] = "N/A"
         else:
@@ -197,8 +138,5 @@ def handle(name, cfg, cloud, log, args):
         )
     except Exception:
         util.logexc(
-            log, "Failed to post phone home data to %s in %s tries", url, tries
+            LOG, "Failed to post phone home data to %s in %s tries", url, tries
         )
-
-
-# vi: ts=4 expandtab
