@@ -9,12 +9,12 @@
 import base64
 import logging
 import os
-from textwrap import dedent
+from typing import Optional
 
-from cloudinit import util
+from cloudinit import url_helper, util
 from cloudinit.cloud import Cloud
 from cloudinit.config import Config
-from cloudinit.config.schema import MetaSchema, get_meta_doc
+from cloudinit.config.schema import MetaSchema
 from cloudinit.settings import PER_INSTANCE
 
 DEFAULT_PERMS = 0o644
@@ -25,97 +25,10 @@ LOG = logging.getLogger(__name__)
 
 meta: MetaSchema = {
     "id": "cc_write_files",
-    "name": "Write Files",
-    "title": "write arbitrary files",
-    "description": dedent(
-        """\
-        Write out arbitrary content to files, optionally setting permissions.
-        Parent folders in the path are created if absent.
-        Content can be specified in plain text or binary. Data encoded with
-        either base64 or binary gzip data can be specified and will be decoded
-        before being written. For empty file creation, content can be omitted.
-
-    .. note::
-        If multiline data is provided, care should be taken to ensure that it
-        follows yaml formatting standards. To specify binary data, use the yaml
-        option ``!!binary``
-
-    .. note::
-        Do not write files under /tmp during boot because of a race with
-        systemd-tmpfiles-clean that can cause temp files to get cleaned during
-        the early boot process. Use /run/somedir instead to avoid race
-        LP:1707222.
-
-    .. warning::
-       Existing files will be overridden."""
-    ),
     "distros": ["all"],
-    "examples": [
-        dedent(
-            """\
-        # Write out base64 encoded content to /etc/sysconfig/selinux
-        write_files:
-        - encoding: b64
-          content: CiMgVGhpcyBmaWxlIGNvbnRyb2xzIHRoZSBzdGF0ZSBvZiBTRUxpbnV4...
-          owner: root:root
-          path: /etc/sysconfig/selinux
-          permissions: '0644'
-        """
-        ),
-        dedent(
-            """\
-        # Appending content to an existing file
-        write_files:
-        - content: |
-            15 * * * * root ship_logs
-          path: /etc/crontab
-          append: true
-        """
-        ),
-        dedent(
-            """\
-        # Provide gziped binary content
-        write_files:
-        - encoding: gzip
-          content: !!binary |
-              H4sIAIDb/U8C/1NW1E/KzNMvzuBKTc7IV8hIzcnJVyjPL8pJ4QIA6N+MVxsAAAA=
-          path: /usr/bin/hello
-          permissions: '0755'
-        """
-        ),
-        dedent(
-            """\
-        # Create an empty file on the system
-        write_files:
-        - path: /root/CLOUD_INIT_WAS_HERE
-        """
-        ),
-        dedent(
-            """\
-        # Defer writing the file until after the package (Nginx) is
-        # installed and its user is created alongside
-        write_files:
-        - path: /etc/nginx/conf.d/example.com.conf
-          content: |
-            server {
-                server_name example.com;
-                listen 80;
-                root /var/www;
-                location / {
-                    try_files $uri $uri/ $uri.html =404;
-                }
-            }
-          owner: 'nginx:nginx'
-          permissions: '0640'
-          defer: true
-        """
-        ),
-    ],
     "frequency": PER_INSTANCE,
     "activate_by_schema_keys": ["write_files"],
-}
-
-__doc__ = get_meta_doc(meta)
+}  # type: ignore
 
 
 def handle(name: str, cfg: Config, cloud: Cloud, args: list) -> None:
@@ -132,7 +45,8 @@ def handle(name: str, cfg: Config, cloud: Cloud, args: list) -> None:
             name,
         )
         return
-    write_files(name, filtered_files, cloud.distro.default_owner)
+    ssl_details = util.fetch_ssl_details(cloud.paths)
+    write_files(name, filtered_files, cloud.distro.default_owner, ssl_details)
 
 
 def canonicalize_extraction(encoding_type):
@@ -146,7 +60,7 @@ def canonicalize_extraction(encoding_type):
     # Yaml already encodes binary data as base64 if it is given to the
     # yaml file as binary, so those will be automatically decoded for you.
     # But the above b64 is just for people that are more 'comfortable'
-    # specifing it manually (which might be a possibility)
+    # specifying it manually (which might be a possibility)
     if encoding_type in ["b64", "base64"]:
         return ["application/base64"]
     if encoding_type == TEXT_PLAIN_ENC:
@@ -160,11 +74,11 @@ def canonicalize_extraction(encoding_type):
     return [TEXT_PLAIN_ENC]
 
 
-def write_files(name, files, owner: str):
+def write_files(name, files, owner: str, ssl_details: Optional[dict] = None):
     if not files:
         return
 
-    for (i, f_info) in enumerate(files):
+    for i, f_info in enumerate(files):
         path = f_info.get("path")
         if not path:
             LOG.warning(
@@ -174,8 +88,23 @@ def write_files(name, files, owner: str):
             )
             continue
         path = os.path.abspath(path)
-        extractions = canonicalize_extraction(f_info.get("encoding"))
-        contents = extract_contents(f_info.get("content", ""), extractions)
+        # Read content from provided URL, if any, or decode from inline
+        contents = read_url_or_decode(
+            f_info.get("source", None),
+            ssl_details,
+            f_info.get("content", None),
+            f_info.get("encoding", None),
+        )
+        if contents is None:
+            LOG.warning(
+                "No content could be loaded for entry %s in module %s;"
+                " skipping",
+                i + 1,
+                name,
+            )
+            continue
+        # Only create the file if content exists. This will not happen, for
+        # example, if the URL fails and no inline content was provided
         (u, g) = util.extract_usergroup(f_info.get("owner", owner))
         perms = decode_perms(f_info.get("permissions"), DEFAULT_PERMS)
         omode = "ab" if util.get_cfg_option_bool(f_info, "append") else "wb"
@@ -206,6 +135,43 @@ def decode_perms(perm, default):
         return default
 
 
+def read_url_or_decode(source, ssl_details, content, encoding):
+    url = None if source is None else source.get("uri", None)
+    use_url = bool(url)
+    # Special case: empty URL and content. Write a blank file
+    if content is None and not use_url:
+        return ""
+    # Fetch file content from source URL, if provided
+    result = None
+    if use_url:
+        try:
+            # NOTE: These retry parameters are arbitrarily chosen defaults.
+            # They have no significance, and may be changed if appropriate
+            result = url_helper.read_file_or_url(
+                url,
+                headers=source.get("headers", None),
+                retries=3,
+                sec_between=3,
+                ssl_details=ssl_details,
+            ).contents
+        except Exception:
+            util.logexc(
+                LOG,
+                'Failed to retrieve contents from source "%s"; falling back to'
+                ' data from "contents" key',
+                url,
+            )
+            use_url = False
+    # If inline content is provided, and URL is not provided or is
+    # inaccessible, parse the former
+    if content is not None and not use_url:
+        # NOTE: This is not simply an "else"! Notice that `use_url` can change
+        # in the previous "if" block
+        extractions = canonicalize_extraction(encoding)
+        result = extract_contents(content, extractions)
+    return result
+
+
 def extract_contents(contents, extraction_types):
     result = contents
     for t in extraction_types:
