@@ -22,7 +22,7 @@ class PackageManager(ABC):
         """Return if package manager is installed on system."""
 
     @abstractmethod
-    def update_package_sources(self):
+    def update_package_sources(self, *, force=False):
         ...
 
     @abstractmethod
