@@ -7,46 +7,42 @@
 # Author: Joshua Harlow <harlowja@yahoo-inc.com>
 #
 # This file is part of cloud-init. See LICENSE file for license information.
+import logging
+import os
 
-from cloudinit import distros
-from cloudinit import helpers
-from cloudinit import log as logging
-from cloudinit import subp
-from cloudinit import util
-
-from cloudinit.distros import rhel_util
+from cloudinit import distros, helpers, subp, util
+from cloudinit.distros import PackageList, rhel_util
+from cloudinit.distros.parsers.hostname import HostnameConf
 from cloudinit.settings import PER_INSTANCE
 
 LOG = logging.getLogger(__name__)
 
 
-def _make_sysconfig_bool(val):
-    if val:
-        return 'yes'
-    else:
-        return 'no'
-
-
 class Distro(distros.Distro):
     # See: https://access.redhat.com/documentation/en-US/Red_Hat_Enterprise_Linux/7/html/Networking_Guide/sec-Network_Configuration_Using_sysconfig_Files.html # noqa
     clock_conf_fn = "/etc/sysconfig/clock"
-    locale_conf_fn = '/etc/sysconfig/i18n'
-    systemd_locale_conf_fn = '/etc/locale.conf'
+    locale_conf_fn = "/etc/sysconfig/i18n"
+    systemd_locale_conf_fn = "/etc/locale.conf"
     network_conf_fn = "/etc/sysconfig/network"
     hostname_conf_fn = "/etc/sysconfig/network"
     systemd_hostname_conf_fn = "/etc/hostname"
-    network_script_tpl = '/etc/sysconfig/network-scripts/ifcfg-%s'
-    resolve_conf_fn = "/etc/resolv.conf"
+    network_script_tpl = "/etc/sysconfig/network-scripts/ifcfg-%s"
     tz_local_fn = "/etc/localtime"
     usr_lib_exec = "/usr/libexec"
+    # RHEL and derivatives use NetworkManager DHCP client by default.
+    # But if NM is configured with using dhclient ("dhcp=dhclient" statement)
+    # then the following location is used:
+    # /var/lib/NetworkManager/dhclient-<uuid>-<network_interface>.lease
+    dhclient_lease_directory = "/var/lib/NetworkManager"
+    dhclient_lease_file_regex = r"dhclient-[\w-]+\.lease"
     renderer_configs = {
-        'sysconfig': {
-            'control': 'etc/sysconfig/network',
-            'iface_templates': '%(base)s/network-scripts/ifcfg-%(name)s',
-            'route_templates': {
-                'ipv4': '%(base)s/network-scripts/route-%(name)s',
-                'ipv6': '%(base)s/network-scripts/route6-%(name)s'
-            }
+        "sysconfig": {
+            "control": "etc/sysconfig/network",
+            "iface_templates": "%(base)s/network-scripts/ifcfg-%(name)s",
+            "route_templates": {
+                "ipv4": "%(base)s/network-scripts/route-%(name)s",
+                "ipv6": "%(base)s/network-scripts/route6-%(name)s",
+            },
         }
     }
 
@@ -57,43 +53,88 @@ class Distro(distros.Distro):
     def __init__(self, name, cfg, paths):
         distros.Distro.__init__(self, name, cfg, paths)
         # This will be used to restrict certain
-        # calls from repeatly happening (when they
+        # calls from repeatedly happening (when they
         # should only happen say once per instance...)
         self._runner = helpers.Runners(paths)
-        self.osfamily = 'redhat'
-        cfg['ssh_svcname'] = 'sshd'
+        self.osfamily = "redhat"
+        self.default_locale = "en_US.UTF-8"
+        self.system_locale = None
+        cfg["ssh_svcname"] = "sshd"
 
-    def install_packages(self, pkglist):
-        self.package_command('install', pkgs=pkglist)
+    def install_packages(self, pkglist: PackageList):
+        self.package_command("install", pkgs=pkglist)
 
-    def _write_network_config(self, netconfig):
-        return self._supported_write_network_config(netconfig)
+    def get_locale(self):
+        """Return the default locale if set, else use system locale"""
+
+        # read system locale value
+        if not self.system_locale:
+            self.system_locale = self._read_system_locale()
+
+        # Return system_locale setting if valid, else use default locale
+        return (
+            self.system_locale if self.system_locale else self.default_locale
+        )
 
     def apply_locale(self, locale, out_fn=None):
         if self.uses_systemd():
             if not out_fn:
                 out_fn = self.systemd_locale_conf_fn
-            out_fn = self.systemd_locale_conf_fn
         else:
             if not out_fn:
                 out_fn = self.locale_conf_fn
         locale_cfg = {
-            'LANG': locale,
+            "LANG": locale,
         }
         rhel_util.update_sysconfig_file(out_fn, locale_cfg)
 
-    def _write_hostname(self, hostname, out_fn):
+    def _read_system_locale(self, keyname="LANG"):
+        """Read system default locale setting, if present"""
+        if self.uses_systemd():
+            locale_fn = self.systemd_locale_conf_fn
+        else:
+            locale_fn = self.locale_conf_fn
+
+        if not locale_fn:
+            raise ValueError("Invalid path: %s" % locale_fn)
+
+        if os.path.exists(locale_fn):
+            (_exists, contents) = rhel_util.read_sysconfig_file(locale_fn)
+            if keyname in contents:
+                return contents[keyname]
+            else:
+                return None
+
+    def _write_hostname(self, hostname, filename):
         # systemd will never update previous-hostname for us, so
         # we need to do it ourselves
-        if self.uses_systemd() and out_fn.endswith('/previous-hostname'):
-            util.write_file(out_fn, hostname)
+        if self.uses_systemd() and filename.endswith("/previous-hostname"):
+            conf = HostnameConf("")
+            conf.set_hostname(hostname)
+            util.write_file(filename, str(conf), 0o644)
         elif self.uses_systemd():
-            subp.subp(['hostnamectl', 'set-hostname', str(hostname)])
+            create_hostname_file = util.get_cfg_option_bool(
+                self._cfg, "create_hostname_file", True
+            )
+            if create_hostname_file:
+                subp.subp(["hostnamectl", "set-hostname", str(hostname)])
+            else:
+                subp.subp(
+                    [
+                        "hostnamectl",
+                        "set-hostname",
+                        "--transient",
+                        str(hostname),
+                    ]
+                )
+                LOG.info(
+                    "create_hostname_file is False; hostname set transiently"
+                )
         else:
             host_cfg = {
-                'HOSTNAME': hostname,
+                "HOSTNAME": hostname,
             }
-            rhel_util.update_sysconfig_file(out_fn, host_cfg)
+            rhel_util.update_sysconfig_file(filename, host_cfg)
 
     def _read_system_hostname(self):
         if self.uses_systemd():
@@ -103,27 +144,22 @@ class Distro(distros.Distro):
         return (host_fn, self._read_hostname(host_fn))
 
     def _read_hostname(self, filename, default=None):
-        if self.uses_systemd() and filename.endswith('/previous-hostname'):
-            return util.load_file(filename).strip()
+        if self.uses_systemd() and filename.endswith("/previous-hostname"):
+            return util.load_text_file(filename).strip()
         elif self.uses_systemd():
-            (out, _err) = subp.subp(['hostname'])
+            (out, _err) = subp.subp(["hostname"])
+            out = out.strip()
             if len(out):
                 return out
             else:
                 return default
         else:
             (_exists, contents) = rhel_util.read_sysconfig_file(filename)
-            if 'HOSTNAME' in contents:
-                return contents['HOSTNAME']
+            if "HOSTNAME" in contents:
+                return contents["HOSTNAME"]
             else:
                 return default
 
-    def _bring_up_interfaces(self, device_names):
-        if device_names and 'all' in device_names:
-            raise RuntimeError(('Distro %s can not translate '
-                                'the device name "all"') % (self.name))
-        return distros.Distro._bring_up_interfaces(self, device_names)
-
     def set_timezone(self, tz):
         tz_file = self._find_tz_file(tz)
         if self.uses_systemd():
@@ -134,7 +170,7 @@ class Distro(distros.Distro):
         else:
             # Adjust the sysconfig clock zone setting
             clock_cfg = {
-                'ZONE': str(tz),
+                "ZONE": str(tz),
             }
             rhel_util.update_sysconfig_file(self.clock_conf_fn, clock_cfg)
             # This ensures that the correct tz will be used for the system
@@ -144,18 +180,18 @@ class Distro(distros.Distro):
         if pkgs is None:
             pkgs = []
 
-        if subp.which('dnf'):
-            LOG.debug('Using DNF for package management')
-            cmd = ['dnf']
+        if subp.which("dnf"):
+            LOG.debug("Using DNF for package management")
+            cmd = ["dnf"]
         else:
-            LOG.debug('Using YUM for package management')
+            LOG.debug("Using YUM for package management")
             # the '-t' argument makes yum tolerant of errors on the command
             # line with regard to packages.
             #
             # For example: if you request to install foo, bar and baz and baz
             # is installed; yum won't error out complaining that baz is already
             # installed.
-            cmd = ['yum', '-t']
+            cmd = ["yum", "-t"]
         # Determines whether or not yum prompts for confirmation
         # of critical actions. We don't want to prompt...
         cmd.append("-y")
@@ -167,14 +203,16 @@ class Distro(distros.Distro):
 
         cmd.append(command)
 
-        pkglist = util.expand_package_list('%s-%s', pkgs)
+        pkglist = util.expand_package_list("%s-%s", pkgs)
         cmd.extend(pkglist)
 
         # Allow the output of this to flow outwards (ie not be captured)
         subp.subp(cmd, capture=False)
 
     def update_package_sources(self):
-        self._runner.run("update-sources", self.package_command,
-                         ["makecache"], freq=PER_INSTANCE)
-
-# vi: ts=4 expandtab
+        self._runner.run(
+            "update-sources",
+            self.package_command,
+            ["makecache"],
+            freq=PER_INSTANCE,
+        )
