@@ -15,7 +15,7 @@ from configparser import NoOptionError, NoSectionError, RawConfigParser
 from io import StringIO
 from time import time
 
-from cloudinit import persistence, type_utils, util
+from cloudinit import persistence, settings, type_utils, util
 from cloudinit.settings import CFG_ENV_NAME, PER_ALWAYS, PER_INSTANCE, PER_ONCE
 
 LOG = logging.getLogger(__name__)
@@ -104,23 +104,7 @@ class FileSemaphores:
         sem_file = self._get_path(cname, freq)
         # This isn't really a good atomic check
         # but it suffices for where and when cloudinit runs
-        if os.path.exists(sem_file):
-            return True
-
-        # this case could happen if the migrator module hadn't run yet
-        # but the item had run before we did canon_sem_name.
-        if cname != name and os.path.exists(self._get_path(name, freq)):
-            LOG.warning(
-                "%s has run without canonicalized name [%s].\n"
-                "likely the migrator has not yet run. "
-                "It will run next boot.\n"
-                "run manually with: cloud-init single --name=migrator",
-                name,
-                cname,
-            )
-            return True
-
-        return False
+        return os.path.exists(sem_file)
 
     def _get_path(self, name, freq):
         sem_path = self.sem_path
@@ -323,7 +307,10 @@ class Paths(persistence.CloudInitPickleMixin):
         self.cfgs = path_cfgs
         # Populate all the initial paths
         self.cloud_dir: str = path_cfgs.get("cloud_dir", "/var/lib/cloud")
-        self.run_dir: str = path_cfgs.get("run_dir", "/run/cloud-init")
+        self.docs_dir: str = path_cfgs.get(
+            "docs_dir", "/usr/share/doc/cloud-init/"
+        )
+        self.run_dir: str = path_cfgs.get("run_dir", settings.DEFAULT_RUN_DIR)
         self.instance_link: str = os.path.join(self.cloud_dir, "instance")
         self.boot_finished: str = os.path.join(
             self.instance_link, "boot-finished"
@@ -363,6 +350,7 @@ class Paths(persistence.CloudInitPickleMixin):
             "vendor_cloud_config": "vendor-cloud-config.txt",
             "vendor_scripts": "scripts/vendor",
             "warnings": "warnings",
+            "hotplug.enabled": "hotplug.enabled",
         }
         # Set when a datasource becomes active
         self.datasource = ds
@@ -381,13 +369,15 @@ class Paths(persistence.CloudInitPickleMixin):
         if "instance_data" not in self.lookups:
             self.lookups["instance_data"] = "instance-data.json"
         if "instance_data_sensitive" not in self.lookups:
-            self.lookups[
-                "instance_data_sensitive"
-            ] = "instance-data-sensitive.json"
+            self.lookups["instance_data_sensitive"] = (
+                "instance-data-sensitive.json"
+            )
         if "combined_cloud_config" not in self.lookups:
-            self.lookups[
-                "combined_cloud_config"
-            ] = "combined-cloud-config.json"
+            self.lookups["combined_cloud_config"] = (
+                "combined-cloud-config.json"
+            )
+        if "hotplug.enabled" not in self.lookups:
+            self.lookups["hotplug.enabled"] = "hotplug.enabled"
 
     # get_ipath_cur: get the current instance path for an item
     def get_ipath_cur(self, name=None):
