@@ -4,15 +4,16 @@
 """
 import contextlib
 import logging
-from typing import Any, Dict, List, Optional
+from functools import partial
+from typing import Any, Callable, Dict, List, Optional
 
 import cloudinit.net as net
-from cloudinit import subp
 from cloudinit.net.dhcp import (
-    IscDhclient,
+    Dhcpcd,
     NoDHCPLeaseError,
     maybe_perform_dhcp_discovery,
 )
+from cloudinit.subp import ProcessExecutionError
 
 LOG = logging.getLogger(__name__)
 
@@ -30,6 +31,7 @@ class EphemeralIPv4Network:
 
     def __init__(
         self,
+        distro,
         interface,
         ip,
         prefix_or_mask,
@@ -71,7 +73,9 @@ class EphemeralIPv4Network:
         self.router = router
         self.static_routes = static_routes
         # List of commands to run to cleanup state.
-        self.cleanup_cmds: List[str] = []
+        self.cleanup_cmds: List[Callable] = []
+        self.distro = distro
+        self.cidr = f"{self.ip}/{self.prefix}"
 
     def __enter__(self):
         """Perform ephemeral network setup if interface is not connected."""
@@ -103,46 +107,28 @@ class EphemeralIPv4Network:
                 self._bringup_static_routes()
             elif self.router:
                 self._bringup_router()
-        except subp.ProcessExecutionError:
-            LOG.error(
-                "Error bringing up EphemeralIPv4Network. "
-                "Datasource setup cannot continue"
-            )
+        except ProcessExecutionError:
             self.__exit__(None, None, None)
             raise
 
     def __exit__(self, excp_type, excp_value, excp_traceback):
         """Teardown anything we set up."""
         for cmd in self.cleanup_cmds:
-            subp.subp(cmd, capture=True)
+            cmd()
 
     def _bringup_device(self):
-        """Perform the ip comands to fully setup the device."""
-        cidr = "{0}/{1}".format(self.ip, self.prefix)
+        """Perform the ip commands to fully setup the device."""
         LOG.debug(
             "Attempting setup of ephemeral network on %s with %s brd %s",
             self.interface,
-            cidr,
+            self.cidr,
             self.broadcast,
         )
         try:
-            subp.subp(
-                [
-                    "ip",
-                    "-family",
-                    "inet",
-                    "addr",
-                    "add",
-                    cidr,
-                    "broadcast",
-                    self.broadcast,
-                    "dev",
-                    self.interface,
-                ],
-                capture=True,
-                update_env={"LANG": "C"},
+            self.distro.net_ops.add_addr(
+                self.interface, self.cidr, self.broadcast
             )
-        except subp.ProcessExecutionError as e:
+        except ProcessExecutionError as e:
             if "File exists" not in str(e.stderr):
                 raise
             LOG.debug(
@@ -152,52 +138,24 @@ class EphemeralIPv4Network:
             )
         else:
             # Address creation success, bring up device and queue cleanup
-            subp.subp(
-                [
-                    "ip",
-                    "-family",
-                    "inet",
-                    "link",
-                    "set",
-                    "dev",
+            self.distro.net_ops.link_up(self.interface, family="inet")
+            self.cleanup_cmds.append(
+                partial(
+                    self.distro.net_ops.link_down,
                     self.interface,
-                    "up",
-                ],
-                capture=True,
+                    family="inet",
+                )
             )
             self.cleanup_cmds.append(
-                [
-                    "ip",
-                    "-family",
-                    "inet",
-                    "link",
-                    "set",
-                    "dev",
-                    self.interface,
-                    "down",
-                ]
-            )
-            self.cleanup_cmds.append(
-                [
-                    "ip",
-                    "-family",
-                    "inet",
-                    "addr",
-                    "del",
-                    cidr,
-                    "dev",
-                    self.interface,
-                ]
+                partial(
+                    self.distro.net_ops.del_addr, self.interface, self.cidr
+                )
             )
 
     def _bringup_static_routes(self):
         # static_routes = [("169.254.169.254/32", "130.56.248.255"),
         #                  ("0.0.0.0/0", "130.56.240.1")]
         for net_address, gateway in self.static_routes:
-            via_arg = []
-            if gateway != "0.0.0.0":
-                via_arg = ["via", gateway]
-
             # Use "append" rather than "add" since the DHCP server may provide
             # rfc3442 classless static routes with multiple routes to the same
             # subnet via different routers or local interface addresses.
@@ -205,23 +163,23 @@ class EphemeralIPv4Network:
             # In this scenario, `ip r add` fails.
             #
             # RHBZ: #2003231
-            subp.subp(
-                ["ip", "-4", "route", "append", net_address]
-                + via_arg
-                + ["dev", self.interface],
-                capture=True,
+            self.distro.net_ops.append_route(
+                self.interface, net_address, gateway
             )
             self.cleanup_cmds.insert(
                 0,
-                ["ip", "-4", "route", "del", net_address]
-                + via_arg
-                + ["dev", self.interface],
+                partial(
+                    self.distro.net_ops.del_route,
+                    self.interface,
+                    net_address,
+                    gateway=gateway,
+                ),
             )
 
     def _bringup_router(self):
         """Perform the ip commands to fully setup the router if needed."""
         # Check if a default route exists and exit if it does
-        out, _ = subp.subp(["ip", "route", "show", "0.0.0.0/0"], capture=True)
+        out = self.distro.net_ops.get_default_route()
         if "default" in out:
             LOG.debug(
                 "Skip ephemeral route setup. %s already has default route: %s",
@@ -229,50 +187,24 @@ class EphemeralIPv4Network:
                 out.strip(),
             )
             return
-        subp.subp(
-            [
-                "ip",
-                "-4",
-                "route",
-                "add",
-                self.router,
-                "dev",
-                self.interface,
-                "src",
-                self.ip,
-            ],
-            capture=True,
+        self.distro.net_ops.add_route(
+            self.interface, self.router, source_address=self.ip
         )
         self.cleanup_cmds.insert(
             0,
-            [
-                "ip",
-                "-4",
-                "route",
-                "del",
-                self.router,
-                "dev",
+            partial(
+                self.distro.net_ops.del_route,
                 self.interface,
-                "src",
-                self.ip,
-            ],
+                self.router,
+                source_address=self.ip,
+            ),
         )
-        subp.subp(
-            [
-                "ip",
-                "-4",
-                "route",
-                "add",
-                "default",
-                "via",
-                self.router,
-                "dev",
-                self.interface,
-            ],
-            capture=True,
+        self.distro.net_ops.add_route(
+            self.interface, "default", gateway=self.router
         )
         self.cleanup_cmds.insert(
-            0, ["ip", "-4", "route", "del", "default", "dev", self.interface]
+            0,
+            partial(self.distro.net_ops.del_route, self.interface, "default"),
         )
 
 
@@ -283,7 +215,7 @@ class EphemeralIPv6Network:
     sufficient for link-local communication.
     """
 
-    def __init__(self, interface):
+    def __init__(self, distro, interface):
         """Setup context manager and validate call signature.
 
         @param interface: Name of the network interface to bring up.
@@ -294,6 +226,7 @@ class EphemeralIPv6Network:
             raise ValueError("Cannot init network on {0}".format(interface))
 
         self.interface = interface
+        self.distro = distro
 
     def __enter__(self):
         """linux kernel does autoconfiguration even when autoconf=0
@@ -301,10 +234,7 @@ class EphemeralIPv6Network:
         https://www.kernel.org/doc/html/latest/networking/ipv6.html
         """
         if net.read_sys_net(self.interface, "operstate") != "up":
-            subp.subp(
-                ["ip", "link", "set", "dev", self.interface, "up"],
-                capture=False,
-            )
+            self.distro.net_ops.link_up(self.interface)
 
     def __exit__(self, *_args):
         """No need to set the link to down state"""
@@ -344,11 +274,9 @@ class EphemeralDHCPv4:
 
     def clean_network(self):
         """Exit _ephipv4 context to teardown of ip configuration performed."""
-        if self.lease:
-            self.lease = None
-        if not self._ephipv4:
-            return
-        self._ephipv4.__exit__(None, None, None)
+        self.lease = None
+        if self._ephipv4:
+            self._ephipv4.__exit__(None, None, None)
 
     def obtain_lease(self):
         """Perform dhcp discovery in a sandboxed environment if possible.
@@ -361,12 +289,11 @@ class EphemeralDHCPv4:
         """
         if self.lease:
             return self.lease
-        leases = maybe_perform_dhcp_discovery(
+        self.lease = maybe_perform_dhcp_discovery(
             self.distro, self.iface, self.dhcp_log_func
         )
-        if not leases:
+        if not self.lease:
             raise NoDHCPLeaseError()
-        self.lease = leases[-1]
         LOG.debug(
             "Received dhcp lease on %s for %s/%s",
             self.lease["interface"],
@@ -381,6 +308,7 @@ class EphemeralDHCPv4:
             "static_routes": [
                 "rfc3442-classless-static-routes",
                 "classless-static-routes",
+                "static_routes",
             ],
             "router": "routers",
         }
@@ -390,12 +318,17 @@ class EphemeralDHCPv4:
                 kwargs["prefix_or_mask"], kwargs["ip"]
             )
         if kwargs["static_routes"]:
-            kwargs["static_routes"] = IscDhclient.parse_static_routes(
+            kwargs[
+                "static_routes"
+            ] = self.distro.dhcp_client.parse_static_routes(
                 kwargs["static_routes"]
             )
         if self.connectivity_url_data:
             kwargs["connectivity_url_data"] = self.connectivity_url_data
-        ephipv4 = EphemeralIPv4Network(**kwargs)
+        if isinstance(self.distro.dhcp_client, Dhcpcd):
+            ephipv4 = DhcpcdEphemeralIPv4Network(self.distro, **kwargs)
+        else:
+            ephipv4 = EphemeralIPv4Network(self.distro, **kwargs)
         ephipv4.__enter__()
         self._ephipv4 = ephipv4
         return self.lease
@@ -419,8 +352,36 @@ class EphemeralDHCPv4:
                 result[internal_mapping] = self.lease.get(different_names)
 
 
+class DhcpcdEphemeralIPv4Network(EphemeralIPv4Network):
+    """dhcpcd sets up its own ephemeral network and routes"""
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+        # clean up after dhcpcd
+        self.cleanup_cmds.append(
+            partial(
+                self.distro.net_ops.link_down,
+                self.interface,
+                family="inet",
+            )
+        )
+        self.cleanup_cmds.append(
+            partial(self.distro.net_ops.del_addr, self.interface, self.cidr)
+        )
+
+    def __enter__(self):
+        return
+
+
 class EphemeralIPNetwork:
-    """Marries together IPv4 and IPv6 ephemeral context managers"""
+    """Combined ephemeral context manager for IPv4 and IPv6
+
+    Either ipv4 or ipv6 ephemeral network may fail to initialize, but if either
+    succeeds, then this context manager will not raise exception. This allows
+    either ipv4 or ipv6 ephemeral network to succeed, but requires that error
+    handling for networks unavailable be done within the context.
+    """
 
     def __init__(
         self,
@@ -437,22 +398,47 @@ class EphemeralIPNetwork:
         self.distro = distro
 
     def __enter__(self):
-        # ipv6 dualstack might succeed when dhcp4 fails
-        # therefore catch exception unless only v4 is used
-        try:
-            if self.ipv4:
+        if not (self.ipv4 or self.ipv6):
+            # no ephemeral network requested, but this object still needs to
+            # function as a context manager
+            return self
+        exceptions = []
+        ephemeral_obtained = False
+        if self.ipv4:
+            try:
                 self.stack.enter_context(
-                    EphemeralDHCPv4(self.distro, self.interface)
+                    EphemeralDHCPv4(
+                        self.distro,
+                        self.interface,
+                    )
                 )
-            if self.ipv6:
-                self.stack.enter_context(EphemeralIPv6Network(self.interface))
-        # v6 link local might be usable
-        # caller may want to log network state
-        except NoDHCPLeaseError as e:
-            if self.ipv6:
-                self.state_msg = "using link-local ipv6"
-            else:
-                raise e
+                ephemeral_obtained = True
+            except (ProcessExecutionError, NoDHCPLeaseError) as e:
+                LOG.info("Failed to bring up %s for ipv4.", self)
+                exceptions.append(e)
+
+        if self.ipv6:
+            try:
+                self.stack.enter_context(
+                    EphemeralIPv6Network(
+                        self.distro,
+                        self.interface,
+                    )
+                )
+                ephemeral_obtained = True
+                if exceptions or not self.ipv4:
+                    self.state_msg = "using link-local ipv6"
+            except ProcessExecutionError as e:
+                LOG.info("Failed to bring up %s for ipv6.", self)
+                exceptions.append(e)
+        if not ephemeral_obtained:
+            # Ephemeral network setup failed in linkup for both ipv4 and
+            # ipv6. Raise only the first exception found.
+            LOG.error(
+                "Failed to bring up EphemeralIPNetwork. "
+                "Datasource setup cannot continue"
+            )
+            raise exceptions[0]
         return self
 
     def __exit__(self, *_args):
