@@ -58,10 +58,10 @@ def read_user_data_callback(mount_dir):
 
     # First try deltacloud_user_data_file. On failure try user_data_file.
     try:
-        user_data = util.load_file(deltacloud_user_data_file).strip()
+        user_data = util.load_text_file(deltacloud_user_data_file).strip()
     except IOError:
         try:
-            user_data = util.load_file(user_data_file).strip()
+            user_data = util.load_text_file(user_data_file).strip()
         except IOError:
             util.logexc(LOG, "Failed accessing user data file.")
             return None
@@ -100,7 +100,9 @@ class DataSourceAltCloud(sources.DataSource):
         """
         if os.path.exists(CLOUD_INFO_FILE):
             try:
-                cloud_type = util.load_file(CLOUD_INFO_FILE).strip().upper()
+                cloud_type = (
+                    util.load_text_file(CLOUD_INFO_FILE).strip().upper()
+                )
             except IOError:
                 util.logexc(
                     LOG,
