@@ -112,6 +112,7 @@ class SocketHTTPConnection(HTTPConnection):
     def __init__(self, socket_path):
         super().__init__("localhost")
         self.socket_path = socket_path
+        self.sock = None
 
     def connect(self):
         self.sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
@@ -131,6 +132,13 @@ class LXDSocketAdapter(HTTPAdapter):
     def get_connection(self, url, proxies=None):
         return SocketConnectionPool(LXD_SOCKET_PATH)
 
+    # Fix for requests 2.32.2+:
+    # https://github.com/psf/requests/pull/6710
+    def get_connection_with_tls_context(
+        self, request, verify, proxies=None, cert=None
+    ):
+        return self.get_connection(request.url, proxies)
+
 
 def _raw_instance_data_to_dict(metadata_type: str, metadata_value) -> dict:
     """Convert raw instance data from str, bytes, YAML to dict
@@ -166,7 +174,7 @@ class DataSourceLXD(sources.DataSource):
     dsname = "LXD"
 
     _network_config: Union[Dict, str] = sources.UNSET
-    _crawled_metadata: Union[Dict, str] = sources.UNSET
+    _crawled_metadata: Optional[Union[Dict, str]] = sources.UNSET
 
     sensitive_metadata_keys: Tuple[
         str, ...
@@ -205,10 +213,6 @@ class DataSourceLXD(sources.DataSource):
             user_metadata = _raw_instance_data_to_dict(
                 "user.meta-data", user_metadata
             )
-        if not isinstance(self.metadata, dict):
-            self.metadata = util.mergemanydict(
-                [util.load_yaml(self.metadata), user_metadata]
-            )
         if "user-data" in self._crawled_metadata:
             self.userdata_raw = self._crawled_metadata["user-data"]
         if "network-config" in self._crawled_metadata:
@@ -283,7 +287,7 @@ def _get_json_response(
             "Skipping %s on [HTTP:%d]:%s",
             url,
             url_response.status_code,
-            url_response.text,
+            url_response.content.decode("utf-8"),
         )
         return {}
     try:
@@ -292,7 +296,7 @@ def _get_json_response(
         raise sources.InvalidMetaDataException(
             "Unable to process LXD config at {url}."
             " Expected JSON but found: {resp}".format(
-                url=url, resp=url_response.text
+                url=url, resp=url_response.content.decode("utf-8")
             )
         ) from exc
 
@@ -320,7 +324,7 @@ def _do_request(
             "Invalid HTTP response [{code}] from {route}: {resp}".format(
                 code=response.status_code,
                 route=url,
-                resp=response.text,
+                resp=response.content.decode("utf-8"),
             )
         )
     return response
@@ -331,7 +335,7 @@ class MetaDataKeys(Flag):
     CONFIG = auto()
     DEVICES = auto()
     META_DATA = auto()
-    ALL = CONFIG | DEVICES | META_DATA
+    ALL = CONFIG | DEVICES | META_DATA  # pylint: disable=E1131
 
 
 class _MetaDataReader:
@@ -363,12 +367,13 @@ class _MetaDataReader:
             config_route_response = _do_request(
                 session, config_route_url, do_raise=False
             )
+            response_text = config_route_response.content.decode("utf-8")
             if not config_route_response.ok:
                 LOG.debug(
                     "Skipping %s on [HTTP:%d]:%s",
                     config_route_url,
                     config_route_response.status_code,
-                    config_route_response.text,
+                    response_text,
                 )
                 continue
 
@@ -376,16 +381,14 @@ class _MetaDataReader:
             # Leave raw data values/format unchanged to represent it in
             # instance-data.json for cloud-init query or jinja template
             # use.
-            config["config"][cfg_key] = config_route_response.text
+            config["config"][cfg_key] = response_text
             # Promote common CONFIG_KEY_ALIASES to top-level keys.
             if cfg_key in CONFIG_KEY_ALIASES:
                 # Due to sort of config_routes, promote cloud-init.*
                 # aliases before user.*. This allows user.* keys to act as
                 # fallback config on old LXD, with new cloud-init images.
                 if CONFIG_KEY_ALIASES[cfg_key] not in config:
-                    config[
-                        CONFIG_KEY_ALIASES[cfg_key]
-                    ] = config_route_response.text
+                    config[CONFIG_KEY_ALIASES[cfg_key]] = response_text
                 else:
                     LOG.warning(
                         "Ignoring LXD config %s in favor of %s value.",
@@ -403,7 +406,9 @@ class _MetaDataReader:
                 md_route = url_helper.combine_url(
                     self._version_url, "meta-data"
                 )
-                md["meta-data"] = _do_request(session, md_route).text
+                md["meta-data"] = _do_request(
+                    session, md_route
+                ).content.decode("utf-8")
             if MetaDataKeys.CONFIG in metadata_keys:
                 md.update(self._process_config(session))
             if MetaDataKeys.DEVICES in metadata_keys:
