@@ -9,11 +9,11 @@
 # This file is part of cloud-init. See LICENSE file for license information.
 
 import errno
+import logging
 import os
+from functools import partial
 
-from cloudinit import dmi
-from cloudinit import log as logging
-from cloudinit import sources, util
+from cloudinit import dmi, lifecycle, sources, util
 from cloudinit.net import eni
 
 LOG = logging.getLogger(__name__)
@@ -32,10 +32,21 @@ class DataSourceNoCloud(sources.DataSource):
         ]
         self.seed_dir = None
         self.supported_seed_starts = ("/", "file://")
+        self._network_config = None
+        self._network_eni = None
 
     def __str__(self):
-        root = sources.DataSource.__str__(self)
-        return "%s [seed=%s][dsmode=%s]" % (root, self.seed, self.dsmode)
+        """append seed and dsmode info when they contain non-default values"""
+        return (
+            super().__str__()
+            + " "
+            + (f"[seed={self.seed}]" if self.seed else "")
+            + (
+                f"[dsmode={self.dsmode}]"
+                if self.dsmode != sources.DSMODE_NETWORK
+                else ""
+            )
+        )
 
     def _get_devices(self, label):
         fslist = util.find_devs_with("TYPE=vfat")
@@ -65,7 +76,7 @@ class DataSourceNoCloud(sources.DataSource):
 
         try:
             # Parse the system serial label from dmi. If not empty, try parsing
-            # like the commandline
+            # like the command line
             md = {}
             serial = dmi.read_dmi_data("system-serial-number")
             if serial and load_cmdline_data(md, serial):
@@ -119,6 +130,12 @@ class DataSourceNoCloud(sources.DataSource):
 
         label = self.ds_cfg.get("fs_label", "cidata")
         if label is not None:
+            if label.lower() != "cidata":
+                lifecycle.deprecate(
+                    deprecated="Custom fs_label keys",
+                    deprecated_version="24.3",
+                    extra_message="This key isn't supported by ds-identify.",
+                )
             for dev in self._get_devices(label):
                 try:
                     LOG.debug("Attempting to use data from %s", dev)
@@ -165,7 +182,7 @@ class DataSourceNoCloud(sources.DataSource):
                     seedfound = proto
                     break
             if not seedfound:
-                LOG.debug("Seed from %s not supported by %s", seedfrom, self)
+                self._log_unusable_seedfrom(seedfrom)
                 return False
             # check and replace instances of known dmi.<dmi_keys> such as
             # chassis-serial-number or baseboard-product-name
@@ -173,7 +190,7 @@ class DataSourceNoCloud(sources.DataSource):
 
             # This could throw errors, but the user told us to do it
             # so if errors are raised, let them raise
-            (md_seed, ud, vd) = util.read_seeded(seedfrom, timeout=None)
+            md_seed, ud, vd, _ = util.read_seeded(seedfrom, timeout=None)
             LOG.debug("Using seeded cache data from %s", seedfrom)
 
             # Values in the command line override those from the seed
@@ -209,13 +226,20 @@ class DataSourceNoCloud(sources.DataSource):
 
     @property
     def platform_type(self):
-        # Handle upgrade path of pickled ds
-        if not hasattr(self, "_platform_type"):
-            self._platform_type = None
         if not self._platform_type:
             self._platform_type = "lxd" if util.is_lxd() else "nocloud"
         return self._platform_type
 
+    def _log_unusable_seedfrom(self, seedfrom: str):
+        """Stage-specific level and message."""
+        LOG.info(
+            "%s only uses seeds starting with %s - will try to use %s "
+            "in the network stage.",
+            self,
+            self.supported_seed_starts,
+            seedfrom,
+        )
+
     def _get_cloud_name(self):
         """Return unknown when 'cloud-name' key is absent from metadata."""
         return sources.METADATA_UNKNOWN
@@ -247,6 +271,13 @@ class DataSourceNoCloud(sources.DataSource):
     def network_config(self):
         if self._network_config is None:
             if self._network_eni is not None:
+                lifecycle.deprecate(
+                    deprecated="Eni network configuration in NoCloud",
+                    deprecated_version="24.3",
+                    extra_message=(
+                        "You can use network v1 or network v2 instead"
+                    ),
+                )
                 self._network_config = eni.convert_eni_data(self._network_eni)
         return self._network_config
 
@@ -289,7 +320,9 @@ def load_cmdline_data(fill, cmdline=None):
 
         seedfrom = fill.get("seedfrom")
         if seedfrom:
-            if seedfrom.startswith(("http://", "https://")):
+            if seedfrom.startswith(
+                ("http://", "https://", "ftp://", "ftps://")
+            ):
                 fill["dsmode"] = sources.DSMODE_NETWORK
             elif seedfrom.startswith(("file://", "/")):
                 fill["dsmode"] = sources.DSMODE_LOCAL
@@ -366,15 +399,68 @@ def _merge_new_seed(cur, seeded):
 class DataSourceNoCloudNet(DataSourceNoCloud):
     def __init__(self, sys_cfg, distro, paths):
         DataSourceNoCloud.__init__(self, sys_cfg, distro, paths)
-        self.supported_seed_starts = ("http://", "https://")
+        self.supported_seed_starts = (
+            "http://",
+            "https://",
+            "ftp://",
+            "ftps://",
+        )
+
+    def _log_unusable_seedfrom(self, seedfrom: str):
+        """Stage-specific level and message."""
+        LOG.warning(
+            "%s only uses seeds starting with %s - %s is not valid.",
+            self,
+            self.supported_seed_starts,
+            seedfrom,
+        )
 
     def ds_detect(self):
-        """NoCloud requires "nocloud-net" as the way to specify
-        seeding from an http(s) address. This diverges from all other
-        datasources in that it does a kernel commandline match on something
-        other than the datasource dsname for only DEP_NETWORK.
+        """Check dmi and kernel command line for dsname
+
+        NoCloud historically used "nocloud-net" as its dsname
+        for network timeframe (DEP_NETWORK), which supports http(s) urls.
+        For backwards compatiblity, check for that dsname.
         """
-        return "nocloud-net" == sources.parse_cmdline()
+        log_deprecated = partial(
+            lifecycle.deprecate,
+            deprecated="The 'nocloud-net' datasource name",
+            deprecated_version="24.1",
+            extra_message=(
+                "Use 'nocloud' instead, which uses the seedfrom protocol"
+                "scheme (http// or file://) to decide how to run."
+            ),
+        )
+
+        if "nocloud-net" == sources.parse_cmdline():
+            log_deprecated()
+            return True
+
+        serial = sources.parse_cmdline_or_dmi(
+            dmi.read_dmi_data("system-serial-number") or ""
+        ).lower()
+
+        if serial in (self.dsname.lower(), "nocloud-net"):
+            LOG.debug(
+                "Machine is configured by dmi serial number to run on "
+                "single datasource %s.",
+                self,
+            )
+            if serial == "nocloud-net":
+                log_deprecated()
+            return True
+        elif (
+            self.sys_cfg.get("datasource", {})
+            .get("NoCloud", {})
+            .get("seedfrom")
+        ):
+            LOG.debug(
+                "Machine is configured by system configuration to run on "
+                "single datasource %s.",
+                self,
+            )
+            return True
+        return False
 
 
 # Used to match classes to dependencies
@@ -387,3 +473,15 @@ datasources = [
 # Return a list of data sources that match this set of dependencies
 def get_datasource_list(depends):
     return sources.list_from_depends(depends, datasources)
+
+
+if __name__ == "__main__":
+    from sys import argv
+
+    logging.basicConfig(level=logging.DEBUG)
+    seedfrom = argv[1]
+    md_seed, ud, vd, network = util.read_seeded(seedfrom)
+    print(f"seeded: {md_seed}")
+    print(f"ud: {ud}")
+    print(f"vd: {vd}")
+    print(f"network: {network}")
