@@ -14,6 +14,9 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with launchpadlib. If not, see <http://www.gnu.org/licenses/>.
 
-import pkg_resources
-__version__ = pkg_resources.resource_string(
-    "launchpadlib", "version.txt").strip()
+try:
+    import importlib.metadata as importlib_metadata
+except ImportError:
+    import importlib_metadata
+
+__version__ = importlib_metadata.version("launchpadlib")
