@@ -16,8 +16,9 @@
 # License along with lazr.restfulclient.  If not, see
 # <http://www.gnu.org/licenses/>.
 
-import pkg_resources
-__version__ = pkg_resources.resource_string(
-    "lazr.restfulclient", "version.txt").strip()
-if not isinstance(__version__, str):
-    __version__ = __version__.decode('ASCII')
+try:
+    import importlib.metadata as importlib_metadata
+except ImportError:
+    import importlib_metadata
+
+__version__ = importlib_metadata.version("lazr.restfulclient")
