@@ -2,8 +2,25 @@ from typing import List
 
 from uaclient.api.api import APIEndpoint
 from uaclient.api.data_types import AdditionalInfo
-from uaclient.api.u.pro.security.fix import FixPlanUSNResult, fix_plan_usn
 from uaclient.api.u.pro.security.fix._common import get_expected_overall_status
+
+# Some of these imports are intentionally not used in this module.
+# The rationale is that we want users to import such Data Objects
+# directly from the associated endpoints and not through the _common module
+from uaclient.api.u.pro.security.fix._common.plan.v1 import (  # noqa: F401
+    AptUpgradeData,
+    AttachData,
+    EnableData,
+    FixPlanError,
+    FixPlanResult,
+    FixPlanStep,
+    FixPlanUSNResult,
+    FixPlanWarning,
+    NoOpData,
+    PackageCannotBeInstalledData,
+    SecurityIssueNotFixedData,
+    fix_plan_usn,
+)
 from uaclient.config import UAConfig
 from uaclient.data_types import DataObject, Field, StringDataValue, data_list
 
