@@ -8,10 +8,13 @@ from typing import List, Optional
 import apt_pkg
 
 from uaclient import defaults, messages, system, util
-from uaclient.api.u.pro.status.is_attached.v1 import _is_attached
+from uaclient.api.u.pro.status.is_attached.v1 import (
+    ContractExpiryStatus,
+    _is_attached,
+)
+from uaclient.apt import ensure_apt_pkg_init, get_pkg_version, version_compare
 from uaclient.clouds.identity import get_cloud_type
 from uaclient.config import UAConfig
-from uaclient.contract import ContractExpiryStatus, get_contract_expiry_status
 from uaclient.data_types import (
     BoolDataValue,
     DataObject,
@@ -20,7 +23,7 @@ from uaclient.data_types import (
     StringDataValue,
     data_list,
 )
-from uaclient.files import state_files
+from uaclient.files import notices, state_files
 
 LOG = logging.getLogger(util.replace_top_level_logger_name(__name__))
 
@@ -30,6 +33,10 @@ class AptNewsMessageSelectors(DataObject):
         Field("codenames", data_list(StringDataValue), required=False),
         Field("clouds", data_list(StringDataValue), required=False),
         Field("pro", BoolDataValue, required=False),
+        Field("architectures", data_list(StringDataValue), required=False),
+        Field(
+            "packages", data_list(data_list(StringDataValue)), required=False
+        ),
     ]
 
     def __init__(
@@ -37,11 +44,15 @@ class AptNewsMessageSelectors(DataObject):
         *,
         codenames: Optional[List[str]] = None,
         clouds: Optional[List[str]] = None,
-        pro: Optional[bool] = None
+        pro: Optional[bool] = None,
+        architectures: Optional[List[str]] = None,
+        packages: Optional[List[List[str]]] = None
     ):
         self.codenames = codenames
         self.clouds = clouds
         self.pro = pro
+        self.architectures = architectures
+        self.packages = packages
 
 
 class AptNewsMessage(DataObject):
@@ -66,6 +77,30 @@ class AptNewsMessage(DataObject):
         self.lines = lines
 
 
+def _does_package_selector_apply(package_selector):
+    try:
+        package_name, version_operator, package_version = package_selector
+    except ValueError:
+        LOG.warning("Invalid package selector: %r", package_selector)
+        return False
+    installed_package_version = get_pkg_version(package_name)
+    if installed_package_version is None:
+        return False
+    version_comparison = version_compare(
+        installed_package_version, package_version
+    )
+    return any(
+        [
+            (
+                version_comparison == 0
+                and version_operator in ["==", "<=", ">="]
+            ),
+            (version_comparison < 0 and version_operator in ["<", "<="]),
+            (version_comparison > 0 and version_operator in [">", ">="]),
+        ]
+    )
+
+
 def do_selectors_apply(
     cfg: UAConfig, selectors: Optional[AptNewsMessageSelectors]
 ) -> bool:
@@ -87,6 +122,19 @@ def do_selectors_apply(
         if selectors.pro != _is_attached(cfg).is_attached:
             return False
 
+    if selectors.architectures is not None:
+        if system.get_dpkg_arch() not in selectors.architectures:
+            return False
+
+    if selectors.packages is not None:
+        if not any(
+            [
+                _does_package_selector_apply(package_selector)
+                for package_selector in selectors.packages
+            ]
+        ):
+            return False
+
     return True
 
 
@@ -149,11 +197,12 @@ def select_message(
     return None
 
 
+@ensure_apt_pkg_init
 def fetch_aptnews_json(cfg: UAConfig):
     os.makedirs(defaults.UAC_RUN_PATH, exist_ok=True)
     acq = apt_pkg.Acquire()
     apt_news_file = apt_pkg.AcquireFile(
-        acq, cfg.apt_news_url, destdir=defaults.UAC_RUN_PATH
+        acq, cfg.apt_news_url, hash="", destdir=defaults.UAC_RUN_PATH
     )
     acq.run()
     apt_news_contents = system.load_file(apt_news_file.destfile)
@@ -176,14 +225,22 @@ def local_apt_news(cfg: UAConfig) -> Optional[str]:
     """
     :return: str if local news, None otherwise
     """
-    expiry_status, remaining_days = get_contract_expiry_status(cfg)
+    is_attached_info = _is_attached(cfg)
+    expiry_status = is_attached_info.contract_status
+    remaining_days = is_attached_info.contract_remaining_days
 
-    if expiry_status == ContractExpiryStatus.ACTIVE_EXPIRED_SOON:
-        return messages.CONTRACT_EXPIRES_SOON_APT_NEWS.format(
+    if expiry_status == ContractExpiryStatus.EXPIRED.value:
+        notices.add(notices.Notice.CONTRACT_EXPIRED)
+        return messages.CONTRACT_EXPIRED
+
+    notices.remove(notices.Notice.CONTRACT_EXPIRED)
+
+    if expiry_status == ContractExpiryStatus.ACTIVE_EXPIRED_SOON.value:
+        return messages.CONTRACT_EXPIRES_SOON.pluralize(remaining_days).format(
             remaining_days=remaining_days
         )
 
-    if expiry_status == ContractExpiryStatus.EXPIRED_GRACE_PERIOD:
+    if expiry_status == ContractExpiryStatus.EXPIRED_GRACE_PERIOD.value:
         grace_period_remaining = (
             defaults.CONTRACT_EXPIRY_GRACE_PERIOD_DAYS + remaining_days
         )
@@ -192,13 +249,12 @@ def local_apt_news(cfg: UAConfig) -> Optional[str]:
             exp_dt_str = "Unknown"
         else:
             exp_dt_str = exp_dt.strftime("%d %b %Y")
-        return messages.CONTRACT_EXPIRED_GRACE_PERIOD_APT_NEWS.format(
+        return messages.CONTRACT_EXPIRED_GRACE_PERIOD.pluralize(
+            remaining_days
+        ).format(
             expired_date=exp_dt_str, remaining_days=grace_period_remaining
         )
 
-    if expiry_status == ContractExpiryStatus.EXPIRED:
-        return messages.CONTRACT_EXPIRED_APT_NEWS
-
     return None
 
 
@@ -214,7 +270,6 @@ def update_apt_news(cfg: UAConfig):
     try:
         news = local_apt_news(cfg)
         if not news:
-            apt_pkg.init()
             news = fetch_and_process_apt_news(cfg)
         if news:
             state_files.apt_news_raw_file.write(news)
