@@ -57,9 +57,7 @@ from uaclient.api.u.pro.status.is_attached.v1 import (
     ContractExpiryStatus,
     _is_attached,
 )
-from uaclient.cli.commands import ProArgument, ProArgumentGroup, ProCommand
-from uaclient.cli.detach import action_detach
-from uaclient.cli.parser import HelpCategory
+from uaclient.cli.constants import NAME, USAGE_TMPL
 from uaclient.clouds.identity import (
     CLOUD_TYPE_TO_TITLE,
     PRO_CLOUD_URLS,
@@ -142,6 +140,31 @@ class FixContext:
             )
 
 
+def add_parser(subparsers):
+    parser_fix = subparsers.add_parser("fix", help=messages.CLI_ROOT_FIX)
+    parser_fix.set_defaults(action=action_fix)
+    fix_parser(parser_fix)
+
+
+def fix_parser(parser):
+    """Build or extend an arg parser for fix subcommand."""
+    parser.usage = USAGE_TMPL.format(
+        name=NAME, command="fix <CVE-yyyy-nnnn+>|<USN-nnnn-d+>"
+    )
+    parser.prog = "fix"
+    parser.description = messages.CLI_FIX_DESC
+    parser._optionals.title = messages.CLI_FLAGS
+    parser.add_argument("security_issue", help=messages.CLI_FIX_ISSUE)
+    parser.add_argument(
+        "--dry-run", action="store_true", help=messages.CLI_FIX_DRY_RUN
+    )
+    parser.add_argument(
+        "--no-related", action="store_true", help=messages.CLI_FIX_NO_RELATED
+    )
+
+    return parser
+
+
 def print_cve_header(cve: FixPlanResult):
     lines = [
         "{issue}: {description}".format(
@@ -449,6 +472,8 @@ def _prompt_for_new_token(cfg: UAConfig) -> bool:
     """
     import argparse
 
+    from uaclient import cli
+
     _inform_ubuntu_pro_existence_if_applicable()
     print(messages.SECURITY_UPDATE_NOT_INSTALLED_EXPIRED)
     choice = util.prompt_choices(
@@ -459,7 +484,9 @@ def _prompt_for_new_token(cfg: UAConfig) -> bool:
         print(messages.PROMPT_EXPIRED_ENTER_TOKEN)
         token = input("> ")
         print(colorize_commands([["pro", "detach"]]))
-        action_detach(argparse.Namespace(assume_yes=True, format="cli"), cfg)
+        cli.action_detach(
+            argparse.Namespace(assume_yes=True, format="cli"), cfg
+        )
         return _run_ua_attach(cfg, token)
 
     return False
@@ -909,29 +936,3 @@ def action_fix(args, *, cfg, **kwargs):
         )
 
     return status.exit_code
-
-
-fix_command = ProCommand(
-    "fix",
-    help=messages.CLI_ROOT_FIX,
-    description=messages.CLI_FIX_DESC,
-    action=action_fix,
-    help_category=HelpCategory.SECURITY,
-    argument_groups=[
-        ProArgumentGroup(
-            arguments=[
-                ProArgument("security_issue", help=messages.CLI_FIX_ISSUE),
-                ProArgument(
-                    "--dry-run",
-                    help=messages.CLI_FIX_DRY_RUN,
-                    action="store_true",
-                ),
-                ProArgument(
-                    "--no-related",
-                    help=messages.CLI_FIX_NO_RELATED,
-                    action="store_true",
-                ),
-            ]
-        )
-    ],
-)
