@@ -2,6 +2,7 @@ import logging
 from typing import Any, Dict, Optional, Tuple
 
 from uaclient import (
+    api,
     event_logger,
     exceptions,
     http,
@@ -11,7 +12,7 @@ from uaclient import (
     system,
     util,
 )
-from uaclient.entitlements.base import IncompatibleService, UAEntitlement
+from uaclient.entitlements.base import EntitlementWithMessage, UAEntitlement
 from uaclient.entitlements.entitlement_status import ApplicationStatus
 from uaclient.types import StaticAffordance
 
@@ -27,10 +28,11 @@ LOG = logging.getLogger(util.replace_top_level_logger_name(__name__))
 
 
 class LivepatchEntitlement(UAEntitlement):
-    help_doc_url = "https://ubuntu.com/security/livepatch"
+    help_doc_url = messages.urls.LIVEPATCH_HOME_PAGE
     name = "livepatch"
-    title = "Livepatch"
-    description = "Canonical Livepatch service"
+    title = messages.LIVEPATCH_TITLE
+    description = messages.LIVEPATCH_DESCRIPTION
+    help_text = messages.LIVEPATCH_HELP_TEXT
     affordance_check_kernel_min_version = False
     affordance_check_kernel_flavor = False
     # we do want to check series because livepatch errors on non-lts releases
@@ -40,15 +42,15 @@ class LivepatchEntitlement(UAEntitlement):
     affordance_check_arch = True
 
     @property
-    def incompatible_services(self) -> Tuple[IncompatibleService, ...]:
+    def incompatible_services(self) -> Tuple[EntitlementWithMessage, ...]:
         from uaclient.entitlements.fips import FIPSEntitlement
         from uaclient.entitlements.realtime import RealtimeKernelEntitlement
 
         return (
-            IncompatibleService(
+            EntitlementWithMessage(
                 FIPSEntitlement, messages.LIVEPATCH_INVALIDATES_FIPS
             ),
-            IncompatibleService(
+            EntitlementWithMessage(
                 RealtimeKernelEntitlement,
                 messages.REALTIME_LIVEPATCH_INCOMPATIBLE,
             ),
@@ -59,7 +61,7 @@ class LivepatchEntitlement(UAEntitlement):
         # Use a lambda so we can mock system.is_container in tests
         from uaclient.entitlements.fips import FIPSEntitlement
 
-        fips_ent = FIPSEntitlement(self.cfg)
+        fips_ent = FIPSEntitlement(cfg=self.cfg)
 
         is_fips_enabled = bool(
             fips_ent.application_status()[0] == ApplicationStatus.ENABLED
@@ -80,16 +82,52 @@ class LivepatchEntitlement(UAEntitlement):
             ),
         )
 
-    def _perform_enable(self, silent: bool = False) -> bool:
+    def enable_steps(self) -> int:
+        return 2
+
+    def disable_steps(self) -> int:
+        return 1
+
+    def _perform_enable(self, progress: api.ProgressWrapper) -> bool:
         """Enable specific entitlement.
 
         @return: True on success, False otherwise.
         """
+        progress.progress(messages.INSTALLING_LIVEPATCH)
+
         if not snap.is_snapd_installed():
-            event.info("Installing snapd")
+            progress.emit(
+                "info", messages.INSTALLING_PACKAGES.format(packages="snapd")
+            )
             snap.install_snapd()
 
-        snap.run_snapd_wait_cmd()
+        if not snap.is_snapd_installed_as_a_snap():
+            progress.emit(
+                "info",
+                messages.INSTALLING_PACKAGES.format(packages="snapd snap"),
+            )
+            try:
+                snap.install_snap("snapd")
+            except exceptions.ProcessExecutionError as e:
+                LOG.warning("Failed to install snapd as a snap", exc_info=e)
+                progress.emit(
+                    "info",
+                    messages.EXECUTING_COMMAND_FAILED.format(
+                        command="snap install snapd"
+                    ),
+                )
+
+        snap.run_snapd_wait_cmd(progress)
+
+        try:
+            snap.refresh_snap("snapd")
+        except exceptions.ProcessExecutionError as e:
+            LOG.warning("Failed to refresh snapd snap", exc_info=e)
+            event.info(
+                messages.EXECUTING_COMMAND_FAILED.format(
+                    command="snap refresh snapd"
+                )
+            )
 
         http_proxy = http.validate_proxy(
             "http", self.cfg.http_proxy, http.PROXY_VALIDATION_SNAP_HTTP_URL
@@ -103,7 +141,12 @@ class LivepatchEntitlement(UAEntitlement):
             retry_sleeps=snap.SNAP_INSTALL_RETRIES,
         )
         if not livepatch.is_livepatch_installed():
-            event.info("Installing canonical-livepatch snap")
+            progress.emit(
+                "info",
+                messages.INSTALLING_PACKAGES.format(
+                    packages="canonical-livepatch snap"
+                ),
+            )
             try:
                 snap.install_snap("canonical-livepatch")
             except exceptions.ProcessExecutionError as e:
@@ -112,11 +155,14 @@ class LivepatchEntitlement(UAEntitlement):
         livepatch.configure_livepatch_proxy(http_proxy, https_proxy)
 
         return self.setup_livepatch_config(
-            process_directives=True, process_token=True
+            progress, process_directives=True, process_token=True
         )
 
     def setup_livepatch_config(
-        self, process_directives: bool = True, process_token: bool = True
+        self,
+        progress: api.ProgressWrapper,
+        process_directives: bool = True,
+        process_token: bool = True,
     ) -> bool:
         """Processs configuration setup for livepatch directives.
 
@@ -125,16 +171,20 @@ class LivepatchEntitlement(UAEntitlement):
         :param process_token: Boolean set True when token should be
             processsed.
         """
-        entitlement_cfg = self.cfg.machine_token_file.entitlements.get(
-            self.name
-        )
+        progress.progress(messages.SETTING_UP_LIVEPATCH)
+
+        entitlement_cfg = self.machine_token_file.entitlements().get(self.name)
         if process_directives:
             try:
                 process_config_directives(entitlement_cfg)
             except exceptions.ProcessExecutionError as e:
-                msg = "Unable to configure Livepatch: " + str(e)
-                event.info(msg)
-                LOG.error(msg)
+                LOG.error(str(e), exc_info=e)
+                progress.emit(
+                    "info",
+                    messages.LIVEPATCH_UNABLE_TO_CONFIGURE.format(
+                        error_msg=str(e)
+                    ),
+                )
                 return False
         if process_token:
             livepatch_token = entitlement_cfg.get("resourceToken")
@@ -144,17 +194,17 @@ class LivepatchEntitlement(UAEntitlement):
                     " %s credentials",
                     self.title,
                 )
-                livepatch_token = self.cfg.machine_token["machineToken"]
+                livepatch_token = self.machine_token_file.machine_token[
+                    "machineToken"
+                ]
             application_status, _details = self.application_status()
             if application_status != ApplicationStatus.DISABLED:
-                LOG.info(
-                    "Disabling %s prior to re-attach with new token",
-                    self.title,
-                )
+                LOG.info("Disabling livepatch before re-enabling")
+                progress.emit("info", messages.LIVEPATCH_DISABLE_REATTACH)
                 try:
                     system.subp([livepatch.LIVEPATCH_CMD, "disable"])
                 except exceptions.ProcessExecutionError as e:
-                    LOG.error(str(e))
+                    LOG.error(str(e), exc_info=e)
                     return False
             try:
                 system.subp(
@@ -162,26 +212,29 @@ class LivepatchEntitlement(UAEntitlement):
                     capture=True,
                 )
             except exceptions.ProcessExecutionError as e:
-                msg = "Unable to enable Livepatch: "
+                msg = messages.LIVEPATCH_UNABLE_TO_ENABLE
                 for error_message, print_message in ERROR_MSG_MAP.items():
                     if error_message in str(e):
                         msg += print_message
                         break
-                if msg == "Unable to enable Livepatch: ":
+                if msg == messages.LIVEPATCH_UNABLE_TO_ENABLE:
                     msg += str(e)
-                event.info(msg)
+                progress.emit("info", msg)
                 return False
-            event.info("Canonical livepatch enabled.")
         return True
 
-    def _perform_disable(self, silent=False):
+    def _perform_disable(self, progress: api.ProgressWrapper):
         """Disable specific entitlement
 
         @return: True on success, False otherwise.
         """
         if not livepatch.is_livepatch_installed():
             return True
-        system.subp([livepatch.LIVEPATCH_CMD, "disable"], capture=True)
+        cmd = [livepatch.LIVEPATCH_CMD, "disable"]
+        progress.progress(
+            messages.EXECUTING_COMMAND.format(command=" ".join(cmd))
+        )
+        system.subp(cmd, capture=True)
         return True
 
     def application_status(
@@ -192,7 +245,17 @@ class LivepatchEntitlement(UAEntitlement):
         if not livepatch.is_livepatch_installed():
             return (ApplicationStatus.DISABLED, messages.LIVEPATCH_NOT_ENABLED)
 
-        if livepatch.status() is None:
+        try:
+            livepatch_status = livepatch.status()
+        except exceptions.ProcessExecutionError as e:
+            return (
+                ApplicationStatus.WARNING,
+                messages.LIVEPATCH_CLIENT_FAILURE_WARNING.format(
+                    livepatch_error=e.stderr,
+                ),
+            )
+
+        if livepatch_status is None:
             # TODO(May want to parse INACTIVE/failure assessment)
             return (
                 ApplicationStatus.DISABLED,
@@ -227,6 +290,7 @@ class LivepatchEntitlement(UAEntitlement):
                 True,
                 messages.LIVEPATCH_KERNEL_UPGRADE_REQUIRED,
             )
+
         # if on_supported_kernel returns UNKNOWN we default to no warning
         # because there would be no way for a user to resolve the warning
         return False, None
@@ -263,11 +327,11 @@ class LivepatchEntitlement(UAEntitlement):
 
         delta_entitlement = deltas.get("entitlement", {})
         process_enable_default = delta_entitlement.get("obligations", {}).get(
-            "enabledByDefault", False
+            "enableByDefault", False
         )
 
         if process_enable_default:
-            enable_success, _ = self.enable()
+            enable_success, _ = self.enable(api.ProgressWrapper())
             return enable_success
 
         application_status, _ = self.application_status()
@@ -280,8 +344,17 @@ class LivepatchEntitlement(UAEntitlement):
         )
         process_token = bool(deltas.get("resourceToken", False))
         if any([process_directives, process_token]):
-            LOG.info("Updating '%s' on changed directives.", self.name)
+            LOG.info(
+                "New livepatch directives or token. running "
+                "setup_livepatch_config"
+            )
+            event.info(
+                messages.SERVICE_UPDATING_CHANGED_DIRECTIVES.format(
+                    service=self.name
+                )
+            )
             return self.setup_livepatch_config(
+                progress=api.ProgressWrapper(),
                 process_directives=process_directives,
                 process_token=process_token,
             )
