@@ -6,6 +6,7 @@ from os.path import exists
 from typing import Any, Dict, List, Optional, Tuple, Union
 
 from uaclient import (
+    api,
     apt,
     contract,
     event_logger,
@@ -16,18 +17,27 @@ from uaclient import (
     util,
 )
 from uaclient.entitlements import base
-from uaclient.entitlements.entitlement_status import ApplicationStatus
+from uaclient.entitlements.entitlement_status import (
+    ApplicationStatus,
+    CanDisableFailure,
+    CanDisableFailureReason,
+)
+from uaclient.files.state_files import status_cache_file
 
 event = event_logger.get_event_logger()
 LOG = logging.getLogger(util.replace_top_level_logger_name(__name__))
 
+# See 'What does a specific Ubuntu kernel version number mean?' in
+# https://wiki.ubuntu.com/Kernel/FAQ
+RE_KERNEL_PKG = r"^linux-image-([\d]+[.-][\d]+[.-][\d]+-[\d]+-[A-Za-z0-9_-]+)$"
+
 
 class RepoEntitlement(base.UAEntitlement):
-    repo_list_file_tmpl = "/etc/apt/sources.list.d/ubuntu-{name}.list"
+    repo_file_tmpl = "/etc/apt/sources.list.d/ubuntu-{name}.{extension}"
     repo_pref_file_tmpl = "/etc/apt/preferences.d/ubuntu-{name}"
     repo_url_tmpl = "{}/ubuntu"
 
-    # The repo Origin value for setting pinning
+    # The repo Origin value defined in apt metadata
     origin = None  # type: Optional[str]
 
     # GH: #1084 call apt in noninteractive mode
@@ -37,11 +47,26 @@ class RepoEntitlement(base.UAEntitlement):
     # the service is enabled or not
     check_packages_are_installed = False
 
+    # RepoEntitlements can be purged, unless specifically stated
+    supports_purge = True
+
     # Optional repo pin priority in subclass
     @property
     def repo_pin_priority(self) -> Union[int, str, None]:
         return None
 
+    @property
+    def repo_file(self) -> str:
+        extension = "sources"
+        series = system.get_release_info().series
+        if series in apt.SERIES_NOT_USING_DEB822:
+            extension = "list"
+        return self.repo_file_tmpl.format(name=self.name, extension=extension)
+
+    @property
+    def repo_policy_check_tmpl(self) -> str:
+        return self.repo_url_tmpl + " {}"
+
     @property
     def packages(self) -> List[str]:
         """debs to install on enablement"""
@@ -59,6 +84,22 @@ class RepoEntitlement(base.UAEntitlement):
 
         return packages
 
+    @property
+    def apt_url(self) -> Optional[str]:
+        return (
+            self.entitlement_cfg.get("entitlement", {})
+            .get("directives", {})
+            .get("aptURL")
+        )
+
+    @property
+    def apt_suites(self) -> Optional[str]:
+        return (
+            self.entitlement_cfg.get("entitlement", {})
+            .get("directives", {})
+            .get("suites")
+        )
+
     def _check_for_reboot(self) -> bool:
         """Check if system needs to be rebooted."""
         reboot_required = system.should_reboot(
@@ -72,34 +113,250 @@ class RepoEntitlement(base.UAEntitlement):
     def repo_key_file(self) -> str:
         pass
 
-    def _perform_enable(self, silent: bool = False) -> bool:
+    def can_disable(
+        self, ignore_dependent_services: bool = False
+    ) -> Tuple[bool, Optional[CanDisableFailure]]:
+        result, reason = super().can_disable(
+            ignore_dependent_services=ignore_dependent_services
+        )
+        if result is False:
+            return result, reason
+
+        if not self.origin and self.purge:
+            return (
+                False,
+                CanDisableFailure(
+                    CanDisableFailureReason.NO_PURGE_WITHOUT_ORIGIN,
+                    messages.REPO_PURGE_FAIL_NO_ORIGIN.format(
+                        entitlement_name=self.title, title=self.title
+                    ),
+                ),
+            )
+
+        return result, reason
+
+    def enable_steps(self) -> int:
+        will_install = self.packages is not None and len(self.packages) > 0
+        if self.access_only or not will_install:
+            # 1. Configure APT
+            # 2. Update APT lists
+            return 2
+        else:
+            # 3. Install packages
+            return 3
+
+    def _perform_enable(self, progress: api.ProgressWrapper) -> bool:
         """Enable specific entitlement.
 
         @return: True on success, False otherwise.
-        @raises: UserFacingError on failure to install suggested packages
+        @raises: UbuntuProError on failure to install suggested packages
         """
-        self.setup_apt_config(silent=silent)
+        progress.progress(
+            messages.CONFIGURING_APT_ACCESS.format(service=self.title)
+        )
+        self.setup_apt_config(progress)
 
         if self.supports_access_only and self.access_only:
-            packages_str = (
-                ": " + " ".join(self.packages)
-                if len(self.packages) > 0
-                else ""
-            )
-            event.info("Skipping installing packages{}".format(packages_str))
-            event.info(messages.ACCESS_ENABLED_TMPL.format(title=self.title))
+            if len(self.packages) > 0:
+                progress.emit(
+                    "info",
+                    messages.SKIPPING_INSTALLING_PACKAGES.format(
+                        packages=" ".join(self.packages)
+                    ),
+                )
         else:
-            self.install_packages()
-            event.info(messages.ENABLED_TMPL.format(title=self.title))
-            self._check_for_reboot_msg(operation="install")
+            self.install_packages(progress)
         return True
 
-    def _perform_disable(self, silent=False):
+    def disable_steps(self) -> int:
+        if not self.purge:
+            # 1. Unconfigure APT
+            # 2. Update package lists
+            return 2
+        else:
+            # 3. Purge
+            return 3
+
+    def _perform_disable(self, progress: api.ProgressWrapper):
+        if self.purge and self.origin:
+            progress.emit("info", messages.PURGE_EXPERIMENTAL)
+            progress.emit("info", "")
+
+            repo_origin_packages = apt.get_installed_packages_by_origin(
+                self.origin
+            )
+
+            if not self.purge_kernel_check(repo_origin_packages, progress):
+                return False
+
+            packages_to_reinstall = []
+            packages_to_remove = []
+            for package in repo_origin_packages:
+                alternatives = apt.get_remote_versions_for_package(
+                    package, exclude_origin=self.origin
+                )
+                if alternatives:
+                    # We can call max(List[Version]) but mypy doesn't know.
+                    # Or doesn't like it.
+                    packages_to_reinstall.append(
+                        (package, max(alternatives))  # type: ignore
+                    )
+                else:
+                    packages_to_remove.append(package)
+
+            if not self.prompt_for_purge(
+                packages_to_remove, packages_to_reinstall, progress
+            ):
+                return False
+
         if hasattr(self, "remove_packages"):
             self.remove_packages()
-        self.remove_apt_config(silent=silent)
+        self.remove_apt_config(progress)
+
+        if self.purge and self.origin:
+            progress.progress(
+                messages.PURGING_PACKAGES.format(title=self.title)
+            )
+            self.execute_reinstall(packages_to_reinstall)
+            self.execute_removal(packages_to_remove)
         return True
 
+    def purge_kernel_check(self, package_list, progress: api.ProgressWrapper):
+        """
+        Checks if the purge operation involves a kernel.
+
+        When package called 'linux-image-*' is in the package list, warn the
+        user that a kernel is being removed. Then, show the user what the
+        current kernel is.
+
+        If the current kernel is to be removed, and there are no other valid
+        Ubuntu Kernels installed in the system, return False to abort the
+        operation.
+
+        If there is another Ubuntu kernel - besides the one installed - then
+        prompt the user for confirmation before proceeding.
+        """
+        linux_image_versions = []
+        for package in package_list:
+            m = re.search(RE_KERNEL_PKG, package.name)
+            if m:
+                linux_image_versions.append(m.group(1))
+        if linux_image_versions:
+            # A kernel needs to be removed to purge
+            # API will fail here, but we want CLI to allow it to continue
+            # after a prompt
+            if not progress.is_interactive():
+                raise exceptions.NonInteractiveKernelPurgeDisallowed()
+
+            progress.emit(
+                "info",
+                messages.PURGE_KERNEL_REMOVAL.format(service=self.title),
+            )
+            progress.emit("info", " ".join(linux_image_versions))
+
+            current_kernel = system.get_kernel_info().uname_release
+            progress.emit(
+                "info",
+                messages.PURGE_CURRENT_KERNEL.format(
+                    kernel_version=current_kernel
+                ),
+            )
+
+            installed_kernels = system.get_installed_ubuntu_kernels()
+            # Any installed Ubuntu Kernel not being touched in this operation
+            alternative_kernels = [
+                version
+                for version in installed_kernels
+                if version not in linux_image_versions
+            ]
+
+            if not alternative_kernels:
+                progress.emit("info", messages.PURGE_NO_ALTERNATIVE_KERNEL)
+                return False
+
+            progress.emit(
+                "message_operation",
+                [
+                    (
+                        util.prompt_for_confirmation,
+                        {"msg": messages.PURGE_KERNEL_CONFIRMATION},
+                    )
+                ],
+            )
+
+        return True
+
+    def prompt_for_purge(
+        self,
+        packages_to_remove,
+        packages_to_reinstall,
+        progress: api.ProgressWrapper,
+    ):
+        prompt = False
+        if packages_to_remove:
+            progress.emit("info", messages.WARN_PACKAGES_REMOVAL)
+            progress.emit(
+                "info",
+                util.create_package_list_str(
+                    [package.name for package in packages_to_remove]
+                ),
+            )
+            prompt = True
+
+        if packages_to_reinstall:
+            progress.emit("info", messages.WARN_PACKAGES_REINSTALL)
+            progress.emit(
+                "info",
+                util.create_package_list_str(
+                    [package.name for (package, _) in packages_to_reinstall]
+                ),
+            )
+            prompt = True
+
+        if prompt:
+            progress.emit(
+                "message_operation",
+                [
+                    (
+                        util.prompt_for_confirmation,
+                        {"msg": messages.PROCEED_YES_NO},
+                    )
+                ],
+            )
+        return True
+
+    def execute_removal(self, packages_to_remove):
+        # We need to check again if the package is installed, because there is
+        # an intermediate step between listing the packages and acting on them.
+        # Some reinstalls may also uninstall dependencies.
+        # Packages may be removed between those operations.
+        installed_packages = apt.get_installed_packages_names()
+        to_remove = [
+            package.name
+            for package in packages_to_remove
+            if package.name in installed_packages
+        ]
+        if to_remove:
+            apt.purge_packages(
+                to_remove,
+                messages.UNINSTALLING_PACKAGES_FAILED.format(
+                    packages=to_remove
+                ),
+            )
+
+    def execute_reinstall(self, packages_to_reinstall):
+        # We need to check again if the package is installed, because there is
+        # an intermediate step between listing the packages and acting on them.
+        # Packages may be removed between those operations.
+        installed_packages = apt.get_installed_packages_names()
+        to_reinstall = [
+            "{}={}".format(package.name, version.ver_str)
+            for (package, version) in packages_to_reinstall
+            if package.name in installed_packages
+        ]
+        if to_reinstall:
+            apt.reinstall_packages(to_reinstall)
+
     def application_status(
         self,
     ) -> Tuple[ApplicationStatus, Optional[messages.NamedMessage]]:
@@ -118,16 +375,25 @@ class RepoEntitlement(base.UAEntitlement):
                 ApplicationStatus.DISABLED,
                 messages.NO_APT_URL_FOR_SERVICE.format(title=self.title),
             )
-        policy = apt.get_apt_cache_policy(
-            error_msg=messages.APT_POLICY_FAILED.msg
-        )
-        match = re.search(self.repo_url_tmpl.format(repo_url), policy)
-        if match:
-            current_status = (
-                ApplicationStatus.ENABLED,
-                messages.SERVICE_IS_ACTIVE.format(title=self.title),
+        repo_suites = directives.get("suites")
+        if not repo_suites:
+            return (
+                ApplicationStatus.DISABLED,
+                messages.NO_SUITES_FOR_SERVICE.format(title=self.title),
             )
 
+        policy = apt.get_apt_cache_policy(error_msg=messages.APT_POLICY_FAILED)
+        for suite in repo_suites:
+            service_match = re.search(
+                self.repo_policy_check_tmpl.format(repo_url, suite), policy
+            )
+            if service_match:
+                current_status = (
+                    ApplicationStatus.ENABLED,
+                    messages.SERVICE_IS_ACTIVE.format(title=self.title),
+                )
+                break
+
         if self.check_packages_are_installed:
             for package in self.packages:
                 if not apt.is_installed(package):
@@ -148,7 +414,7 @@ class RepoEntitlement(base.UAEntitlement):
         :return: False if apt url is already found on the source file.
                  True otherwise.
         """
-        apt_file = self.repo_list_file_tmpl.format(name=self.name)
+        apt_file = self.repo_file
         # If the apt file is commented out, we will assume that we need
         # to regenerate the apt file, regardless of the apt url delta
         if all(
@@ -191,7 +457,7 @@ class RepoEntitlement(base.UAEntitlement):
         delta_directives = delta_entitlement.get("directives", {})
         delta_apt_url = delta_directives.get("aptURL")
         delta_packages = delta_directives.get("additionalPackages")
-        status_cache = self.cfg.read_cache("status-cache")
+        status_cache = status_cache_file.read()
 
         if delta_directives and status_cache:
             application_status = self._check_application_status_on_cache()
@@ -203,42 +469,47 @@ class RepoEntitlement(base.UAEntitlement):
 
         if not self._check_apt_url_is_applied(delta_apt_url):
             LOG.info(
-                "Updating '%s' apt sources list on changed directives.",
+                "New aptURL, updating %s apt sources list to %s",
                 self.name,
+                delta_apt_url,
+            )
+            event.info(
+                messages.REPO_UPDATING_APT_SOURCES.format(service=self.name)
             )
 
             orig_entitlement = orig_access.get("entitlement", {})
             old_url = orig_entitlement.get("directives", {}).get("aptURL")
             if old_url:
                 # Remove original aptURL and auth and rewrite
-                repo_filename = self.repo_list_file_tmpl.format(name=self.name)
-                apt.remove_auth_apt_repo(repo_filename, old_url)
+                apt.remove_auth_apt_repo(self.repo_file, old_url)
 
-            self.remove_apt_config()
-            self.setup_apt_config()
+            self.remove_apt_config(api.ProgressWrapper())
+            self.setup_apt_config(api.ProgressWrapper())
 
         if delta_packages:
-            LOG.info(
-                "Installing packages on changed directives: {}".format(
-                    ", ".join(delta_packages)
+            LOG.info("New additionalPackages, installing %r", delta_packages)
+            event.info(
+                messages.REPO_REFRESH_INSTALLING_PACKAGES.format(
+                    packages=", ".join(delta_packages)
                 )
             )
-            self.install_packages(package_list=delta_packages)
+            self.install_packages(
+                api.ProgressWrapper(), package_list=delta_packages
+            )
 
         return True
 
     def install_packages(
         self,
+        progress: api.ProgressWrapper,
         package_list: Optional[List[str]] = None,
         cleanup_on_failure: bool = True,
-        verbose: bool = True,
     ) -> None:
         """Install contract recommended packages for the entitlement.
 
         :param package_list: Optional package list to use instead of
             self.packages.
         :param cleanup_on_failure: Cleanup apt files if apt install fails.
-        :param verbose: If true, print messages to stdout
         """
 
         if not package_list:
@@ -247,12 +518,18 @@ class RepoEntitlement(base.UAEntitlement):
         if not package_list:
             return
 
-        msg_ops = self.messaging.get("pre_install", [])
-        if not util.handle_message_operations(msg_ops):
-            return
+        progress.emit("message_operation", self.messaging.get("pre_install"))
 
-        if verbose:
-            event.info("Installing {title} packages".format(title=self.title))
+        try:
+            self._update_sources_list(progress)
+        except exceptions.UbuntuProError:
+            if cleanup_on_failure:
+                self.remove_apt_config(api.ProgressWrapper())
+            raise
+
+        progress.progress(
+            messages.INSTALLING_SERVICE_PACKAGES.format(title=self.title)
+        )
 
         if self.apt_noninteractive:
             override_env_vars = {"DEBIAN_FRONTEND": "noninteractive"}
@@ -266,23 +543,26 @@ class RepoEntitlement(base.UAEntitlement):
             apt_options = []
 
         try:
-            msg = messages.ENABLED_FAILED.format(title=self.title)
             apt.run_apt_install_command(
                 packages=package_list,
                 apt_options=apt_options,
-                error_msg=msg.msg,
                 override_env_vars=override_env_vars,
             )
-        except exceptions.UserFacingError:
+        except exceptions.UbuntuProError:
             if cleanup_on_failure:
-                self.remove_apt_config()
+                LOG.info(
+                    "Apt install failed, removing apt config for {}".format(
+                        self.name
+                    )
+                )
+                self.remove_apt_config(api.ProgressWrapper())
             raise
 
-    def setup_apt_config(self, silent: bool = False) -> None:
+    def setup_apt_config(self, progress: api.ProgressWrapper) -> None:
         """Setup apt config based on the resourceToken and directives.
         Also sets up apt proxy if necessary.
 
-        :raise UserFacingError: on failure to setup any aspect of this apt
+        :raise UbuntuProError: on failure to setup any aspect of this apt
            configuration
         """
         http_proxy = None  # type: Optional[str]
@@ -316,13 +596,15 @@ class RepoEntitlement(base.UAEntitlement):
         apt.setup_apt_proxy(
             http_proxy=http_proxy, https_proxy=https_proxy, proxy_scope=scope
         )
-        repo_filename = self.repo_list_file_tmpl.format(name=self.name)
+        repo_filename = self.repo_file
         resource_cfg = self.entitlement_cfg
         directives = resource_cfg["entitlement"].get("directives", {})
         obligations = resource_cfg["entitlement"].get("obligations", {})
         token = resource_cfg.get("resourceToken")
         if not token:
-            machine_token = self.cfg.machine_token["machineToken"]
+            machine_token = self.machine_token_file.machine_token[
+                "machineToken"
+            ]
             if not obligations.get("enableByDefault"):
                 # services that are not enableByDefault need to obtain specific
                 # resource access for tokens. We want to refresh this every
@@ -342,28 +624,18 @@ class RepoEntitlement(base.UAEntitlement):
                 )
         aptKey = directives.get("aptKey")
         if not aptKey:
-            raise exceptions.UserFacingError(
-                "Ubuntu Pro server provided no aptKey directive for"
-                " {}.".format(self.name)
-            )
+            raise exceptions.RepoNoAptKey(entitlement_name=self.name)
         repo_url = directives.get("aptURL")
         if not repo_url:
-            raise exceptions.MissingAptURLDirective(self.name)
+            raise exceptions.MissingAptURLDirective(entitlement_name=self.name)
         repo_suites = directives.get("suites")
         if not repo_suites:
-            raise exceptions.UserFacingError(
-                "Empty {} apt suites directive from {}".format(
-                    self.name, self.cfg.contract_url
-                )
-            )
+            raise exceptions.RepoNoSuites(entitlement_name=self.name)
         if self.repo_pin_priority:
             if not self.origin:
-                raise exceptions.UserFacingError(
-                    "Cannot setup apt pin. Empty apt repo origin value '{}'.\n"
-                    "{}".format(
-                        self.origin,
-                        messages.ENABLED_FAILED.format(title=self.title).msg,
-                    )
+                raise exceptions.RepoPinFailNoOrigin(
+                    entitlement_name=self.name,
+                    title=self.title,
                 )
             repo_pref_file = self.repo_pref_file_tmpl.format(name=self.name)
             apt.add_ppa_pinning(
@@ -380,16 +652,16 @@ class RepoEntitlement(base.UAEntitlement):
             prerequisite_pkgs.append("ca-certificates")
 
         if prerequisite_pkgs:
-            if not silent:
-                event.info(
-                    "Installing prerequisites: {}".format(
-                        ", ".join(prerequisite_pkgs)
-                    )
-                )
+            progress.emit(
+                "info",
+                messages.INSTALLING_PACKAGES.format(
+                    packages=", ".join(prerequisite_pkgs)
+                ),
+            )
             try:
                 apt.run_apt_install_command(packages=prerequisite_pkgs)
-            except exceptions.UserFacingError:
-                self.remove_apt_config()
+            except exceptions.UbuntuProError:
+                self.remove_apt_config(api.ProgressWrapper())
                 raise
         apt.add_auth_apt_repo(
             repo_filename,
@@ -402,16 +674,17 @@ class RepoEntitlement(base.UAEntitlement):
         # probably wants access to the repo that was just enabled.
         # Side-effect is that apt policy will now report the repo as accessible
         # which allows pro status to report correct info
-        if not silent:
-            event.info(messages.APT_UPDATING_LISTS)
+        progress.progress(messages.APT_UPDATING_LIST.format(name=self.title))
         try:
-            apt.run_apt_update_command()
-        except exceptions.UserFacingError:
-            self.remove_apt_config(run_apt_update=False)
+            apt.update_sources_list(repo_filename)
+        except exceptions.UbuntuProError:
+            self.remove_apt_config(api.ProgressWrapper(), run_apt_update=False)
             raise
 
     def remove_apt_config(
-        self, run_apt_update: bool = True, silent: bool = False
+        self,
+        progress: api.ProgressWrapper,
+        run_apt_update: bool = True,
     ):
         """Remove any repository apt configuration files.
 
@@ -419,15 +692,18 @@ class RepoEntitlement(base.UAEntitlement):
             command after removing the apt files.
         """
         series = system.get_release_info().series
-        repo_filename = self.repo_list_file_tmpl.format(name=self.name)
-        entitlement = self.cfg.machine_token_file.entitlements[self.name].get(
+        repo_filename = self.repo_file
+        entitlement = self.machine_token_file.entitlements()[self.name].get(
             "entitlement", {}
         )
         access_directives = entitlement.get("directives", {})
         repo_url = access_directives.get("aptURL")
         if not repo_url:
-            raise exceptions.MissingAptURLDirective(self.name)
+            raise exceptions.MissingAptURLDirective(entitlement_name=self.name)
 
+        progress.progress(
+            messages.REMOVING_APT_CONFIGURATION.format(title=self.title)
+        )
         apt.remove_auth_apt_repo(repo_filename, repo_url, self.repo_key_file)
         apt.remove_apt_list_files(repo_url, series)
 
@@ -436,6 +712,5 @@ class RepoEntitlement(base.UAEntitlement):
             system.ensure_file_absent(repo_pref_file)
 
         if run_apt_update:
-            if not silent:
-                event.info(messages.APT_UPDATING_LISTS)
+            progress.progress(messages.APT_UPDATING_LISTS)
             apt.run_apt_update_command()
