@@ -19,20 +19,6 @@ class DelayProLicensePolling(IsProLicensePresentError):
     pass
 
 
-class UrlError(IOError):
-    def __init__(
-        self,
-        cause: Exception,
-        url: str,
-    ):
-        if getattr(cause, "reason", None):
-            cause_error = str(getattr(cause, "reason"))
-        else:
-            cause_error = str(cause)
-        super().__init__(cause_error)
-        self.url = url
-
-
 class ProcessExecutionError(IOError):
     def __init__(
         self,
@@ -186,6 +172,10 @@ class ErrorInstallingLivepatch(UbuntuProError):
 ###############################################################################
 
 
+class InvalidUrl(UbuntuProError):
+    _formatted_msg = messages.E_INVALID_URL
+
+
 class ProxyNotWorkingError(UbuntuProError):
     _formatted_msg = messages.E_NOT_SETTING_PROXY_NOT_WORKING
 
@@ -206,10 +196,6 @@ class ProxyAuthenticationFailed(UbuntuProError):
     _msg = messages.E_PROXY_AUTH_FAIL
 
 
-class ConnectivityError(UbuntuProError):
-    _msg = messages.E_CONNECTIVITY_ERROR
-
-
 class ExternalAPIError(UbuntuProError):
     _formatted_msg = messages.E_EXTERNAL_API_ERROR
     code = None  # type: int
@@ -236,11 +222,40 @@ class PycurlCACertificatesError(UbuntuProError):
         self.url = url
 
 
+class ConnectivityError(UbuntuProError, IOError):
+    _formatted_msg = messages.E_CONNECTIVITY_ERROR
+
+    def __init__(
+        self,
+        cause: Exception,
+        url: str,
+    ):
+        if getattr(cause, "reason", None):
+            cause_error = str(getattr(cause, "reason"))
+        else:
+            cause_error = str(cause)
+        IOError.__init__(self, cause_error)
+        UbuntuProError.__init__(self, cause_error=cause_error, url=url)
+
+        # Even though we already set those variable through UbuntuProError
+        # we need to set them again to avoid mypy warnings
+        self.cause_error = cause_error
+        self.url = url
+
+
+# We are doing that just to keep backwards compatibility
+# for our custom UrlError exception
+UrlError = ConnectivityError
+
 ###############################################################################
 #                              ATTACH/ENABLE                                  #
 ###############################################################################
 
 
+class ContractExpiredError(UbuntuProError):
+    _msg = messages.E_CONTRACT_EXPIRED
+
+
 class InvalidServiceOpError(UbuntuProError):
     _formatted_msg = messages.E_INVALID_SERVICE_OP_FAILURE
 
@@ -328,7 +343,6 @@ class EntitlementNotFoundError(UbuntuProError):
 
 
 class EntitlementsNotEnabledError(UbuntuProError):
-
     exit_code = 4
     _msg = messages.E_ENTITLEMENTS_NOT_ENABLED_ERROR
 
@@ -344,6 +358,38 @@ class EntitlementsNotEnabledError(UbuntuProError):
         )
 
 
+class EntitlementNotEnabledError(UbuntuProError):
+    _formatted_msg = messages.E_ENTITLEMENT_NOT_ENABLED_ERROR
+
+    def __init__(self, service: str, reason: messages.NamedMessage):
+        super().__init__(
+            service=service,
+            additional_info={
+                "reason": {
+                    "code": reason.name,
+                    "title": reason.msg,
+                    "additional_info": reason.additional_info,
+                }
+            },
+        )
+
+
+class EntitlementNotDisabledError(UbuntuProError):
+    _formatted_msg = messages.E_ENTITLEMENT_NOT_DISABLED_ERROR
+
+    def __init__(self, service: str, reason: messages.NamedMessage):
+        super().__init__(
+            service=service,
+            additional_info={
+                "reason": {
+                    "code": reason.name,
+                    "title": reason.msg,
+                    "additional_info": reason.additional_info,
+                }
+            },
+        )
+
+
 class AttachFailureDefaultServices(EntitlementsNotEnabledError):
     _msg = messages.E_ATTACH_FAILURE_DEFAULT_SERVICES
 
@@ -352,6 +398,10 @@ class AttachFailureUnknownError(EntitlementsNotEnabledError):
     _msg = messages.E_ATTACH_FAILURE_UNEXPECTED
 
 
+class AttachFailureRestrictedRelease(UbuntuProError):
+    _formatted_msg = messages.ATTACH_FAILURE_RESTRICTED_RELEASE
+
+
 class RepoNoAptKey(UbuntuProError):
     _formatted_msg = messages.E_REPO_NO_APT_KEY
 
@@ -368,6 +418,30 @@ class InvalidContractDeltasServiceType(UbuntuProError):
     _formatted_msg = messages.E_INVALID_CONTRACT_DELTAS_SERVICE_TYPE
 
 
+class EntitlementsAPTDirectivesAreNotUnique(UbuntuProError):
+    _formatted_msg = messages.E_ENTITLEMENTS_APT_DIRECTIVES_ARE_NOT_UNIQUE
+
+
+class RequiredServiceStopsEnable(UbuntuProError):
+    _formatted_msg = messages.E_REQUIRED_SERVICE_STOPS_ENABLE
+
+
+class IncompatibleServiceStopsEnable(UbuntuProError):
+    _formatted_msg = messages.E_INCOMPATIBLE_SERVICE_STOPS_ENABLE
+
+
+class DependentServiceStopsDisable(UbuntuProError):
+    _formatted_msg = messages.E_DEPENDENT_SERVICE_STOPS_DISABLE
+
+
+class LandscapeConfigFailed(UbuntuProError):
+    _msg = messages.E_LANDSCAPE_CONFIG_FAILED
+
+
+class NonInteractiveKernelPurgeDisallowed(UbuntuProError):
+    _msg = messages.E_NON_INTERACTIVE_KERNEL_PURGE_DISALLOWED
+
+
 ###############################################################################
 #                              CLOUD PRO                                      #
 ###############################################################################
@@ -420,6 +494,10 @@ class InvalidFileFormatError(UbuntuProError):
     _formatted_msg = messages.E_INVALID_FILE_FORMAT
 
 
+class InvalidFileEncodingError(UbuntuProError):
+    _formatted_msg = messages.E_INVALID_FILE_ENCODING
+
+
 class ParsingErrorOnOSReleaseFile(UbuntuProError):
     _formatted_msg = messages.E_ERROR_PARSING_VERSION_OS_RELEASE
 
@@ -485,6 +563,10 @@ class InvalidArgChoice(UbuntuProError):
     _formatted_msg = messages.E_CLI_VALID_CHOICES
 
 
+class EmptyConfigValue(UbuntuProError):
+    _formatted_msg = messages.E_CLI_EMPTY_CONFIG_VALUE
+
+
 class GenericInvalidFormat(UbuntuProError):
     _formatted_msg = messages.E_CLI_EXPECTED_FORMAT
 
@@ -513,6 +595,10 @@ class CLIAPIOptionsXORData(UbuntuProError):
     _msg = messages.E_API_ERROR_ARGS_AND_DATA_TOGETHER
 
 
+class PromptDeniedError(UbuntuProError):
+    _msg = messages.E_PROMPT_DENIED
+
+
 ###############################################################################
 #                              MISCELLANEOUS                                  #
 ###############################################################################
