@@ -35,6 +35,11 @@ def is_snapd_installed() -> bool:
     return "snapd" in apt.get_installed_packages_names()
 
 
+def is_snapd_installed_as_a_snap() -> bool:
+    """Returns whether or not snapd is installed as a snap"""
+    return any((snap.name == "snapd" for snap in get_installed_snaps()))
+
+
 def configure_snap_proxy(
     http_proxy: Optional[str] = None,
     https_proxy: Optional[str] = None,
@@ -121,12 +126,12 @@ def get_installed_snaps() -> List[SnapPackage]:
 
 
 def install_snapd():
-    event.info(messages.APT_UPDATING_LISTS)
+    event.info(messages.APT_UPDATING_LIST.format(name="standard Ubuntu"))
     try:
-        apt.run_apt_update_command()
-    except exceptions.UserFacingError as e:
+        apt.update_sources_list(apt.get_system_sources_file())
+    except exceptions.UbuntuProError as e:
         LOG.debug(
-            "Trying to install snapd." " Ignoring apt-get update failure: %s",
+            "Trying to install snapd. Ignoring apt-get update failure: %s",
             str(e),
         )
     try:
@@ -143,7 +148,10 @@ def run_snapd_wait_cmd():
         system.subp([SNAP_CMD, "wait", "system", "seed.loaded"], capture=True)
     except exceptions.ProcessExecutionError as e:
         if re.search(r"unknown command .*wait", str(e).lower()):
-            LOG.warning(messages.SNAPD_DOES_NOT_HAVE_WAIT_CMD)
+            LOG.warning(
+                "Detected version of snapd that does not have wait command"
+            )
+            event.info(messages.SNAPD_DOES_NOT_HAVE_WAIT_CMD)
         else:
             raise
 
@@ -168,6 +176,10 @@ def install_snap(
     )
 
 
+def refresh_snap(snap: str):
+    system.subp([SNAP_CMD, "refresh", snap], capture=True)
+
+
 def get_snap_info(snap: str) -> SnapPackage:
     snap_sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
     snap_sock.connect(SNAPD_SOCKET_PATH)
@@ -183,15 +195,17 @@ def get_snap_info(snap: str) -> SnapPackage:
 
         try:
             data = json.loads(out)
-        except json.JSONDecodeError:
-            with util.disable_log_to_console():
-                LOG.warning(
-                    messages.JSON_PARSER_ERROR.format(
-                        source="SNAPD API {}".format(url),
-                        out=out,
-                    ).msg
-                )
-            raise exceptions.SnapdInvalidJson(url=url, out=out)
+        except json.JSONDecodeError as e:
+            LOG.warning(
+                "JSONDecodeError while parsing result of snap api call to %s, "
+                'returning None. output was: "%s"',
+                url,
+                out,
+                exc_info=e,
+            )
+            raise exceptions.InvalidJson(
+                source="SNAPD API {}".format(url), out=out
+            )
 
         # This means that the snap doesn't exist or is not installed
         if response.status != 200:
@@ -199,10 +213,10 @@ def get_snap_info(snap: str) -> SnapPackage:
                 response.status == 404
                 and data.get("result", {}).get("kind") == "snap-not-found"
             ):
-                raise exceptions.SnapNotInstalledError(snap)
+                raise exceptions.SnapNotInstalledError(snap=snap)
             else:
                 error_msg = data.get("result", {}).get("message")
-                raise exceptions.UnexpectedSnapdAPIError(error_msg)
+                raise exceptions.UnexpectedSnapdAPIError(error=error_msg)
 
     except ConnectionRefusedError:
         raise exceptions.SnapdAPIConnectionRefused()
