@@ -3,10 +3,9 @@ import os
 import sys
 import time
 
-from uaclient import http, system
+from uaclient import http, log, system
 from uaclient.config import UAConfig
 from uaclient.daemon import poll_for_pro_license, retry_auto_attach
-from uaclient.log import setup_journald_logging
 
 LOG = logging.getLogger("ubuntupro.daemon")
 
@@ -20,8 +19,15 @@ def _wait_for_cloud_config():
     LOG.debug("waiting for cloud-config.service to finish")
     for i in range(WAIT_FOR_CLOUD_CONFIG_POLL_TIMES + 1):
         state = system.get_systemd_unit_active_state("cloud-config.service")
+        ci_state = system.get_systemd_unit_active_state("cloud-init.service")
         LOG.debug("cloud-config.service state: %r", state)
-        if state is not None and state == "activating":
+        LOG.debug("cloud-init.service state: %r", ci_state)
+        # if cloud-config.service is not yet activating but cloud-init is
+        # running, wait for cloud-config to start
+        if state == "activating" or (
+            state == "inactive"
+            and (ci_state == "activating" or ci_state == "active")
+        ):
             if i < WAIT_FOR_CLOUD_CONFIG_POLL_TIMES:
                 LOG.debug(
                     "cloud-config.service is activating. "
@@ -40,16 +46,13 @@ def _wait_for_cloud_config():
 
 
 def main() -> int:
-    setup_journald_logging(logging.DEBUG, LOG)
-    # Make sure the ubuntupro.daemon logger does not generate double logging
-    LOG.propagate = False
-    setup_journald_logging(logging.ERROR, logging.getLogger("ubuntupro"))
+    log.setup_journald_logging()
 
     cfg = UAConfig()
 
     http.configure_web_proxy(cfg.http_proxy, cfg.https_proxy)
 
-    LOG.debug("daemon starting")
+    LOG.info("daemon starting")
 
     _wait_for_cloud_config()
 
@@ -70,7 +73,7 @@ def main() -> int:
         LOG.info("mode: retry auto attach")
         retry_auto_attach.retry_auto_attach(cfg)
 
-    LOG.debug("daemon ending")
+    LOG.info("daemon ending")
     return 0
 
 
