@@ -6,8 +6,7 @@ import logging
 from datetime import datetime, timedelta, timezone
 from typing import Callable, Optional
 
-from uaclient import defaults, http
-from uaclient.cli import setup_logging
+from uaclient import http, log
 from uaclient.config import UAConfig
 from uaclient.exceptions import InvalidFileFormatError
 from uaclient.files.state_files import (
@@ -19,7 +18,7 @@ from uaclient.timer.metering import metering_enabled_resources
 from uaclient.timer.update_contract_info import update_contract_info
 from uaclient.timer.update_messaging import update_motd_messages
 
-LOG = logging.getLogger("ubuntupro.lib.timer")
+LOG = logging.getLogger("ubuntupro.timer")
 UPDATE_MESSAGING_INTERVAL = 21600  # 6 hours
 METERING_INTERVAL = 14400  # 4 hours
 UPDATE_CONTRACT_INFO_INTERVAL = 86400  # 24 hours
@@ -49,9 +48,9 @@ class TimedJob:
             return False
 
         try:
-            LOG.debug("Running job: %s", self.name)
+            LOG.info("Running job: %s", self.name)
             if self._job_func(cfg=cfg):
-                LOG.debug("Executed job: %s", self.name)
+                LOG.info("Executed job: %s", self.name)
         except Exception as e:
             LOG.error("Error executing job %s: %s", self.name, str(e))
             return False
@@ -64,11 +63,11 @@ class TimedJob:
         if configured_interval is None:
             return self._default_interval_seconds
         if not isinstance(configured_interval, int) or configured_interval < 0:
-            warning_msg = (
-                "Invalid value for {} interval found in config. "
-                "Default value will be used."
-            ).format(self.name)
-            LOG.warning(warning_msg)
+            LOG.warning(
+                "Invalid value for %s interval found in config. "
+                "Default value will be used.",
+                self.name,
+            )
             return self._default_interval_seconds
         return configured_interval
 
@@ -157,9 +156,10 @@ def run_jobs(cfg: UAConfig, current_time: datetime):
         try:
             timer_jobs_state_file.delete()
         except (OSError, PermissionError) as exception:
-            msg = "Error trying to delete invalid jobs-status.json file: {}"
-            msg = msg.format(str(exception))
-            LOG.warning(msg)
+            LOG.warning(
+                "Error trying to delete invalid jobs-status.json file: %s",
+                str(exception),
+            )
             return
 
     if jobs_status_obj is None:
@@ -179,26 +179,11 @@ def run_jobs(cfg: UAConfig, current_time: datetime):
 
 
 if __name__ == "__main__":
-    setup_logging(
-        logging.CRITICAL,
-        logging.DEBUG,
-        defaults.CONFIG_DEFAULTS["timer_log_file"],
-        logger=LOG,
-    )
+    log.setup_journald_logging()
+
     cfg = UAConfig()
     current_time = datetime.now(timezone.utc)
 
-    # The ua-timer logger should log everything to its file
-    setup_logging(
-        logging.CRITICAL,
-        logging.DEBUG,
-        log_file=cfg.timer_log_file,
-        logger=LOG,
-    )
-    # Make sure the ua-timer logger does not generate double logging
-    LOG.propagate = False
-    # The root logger should log any error to the timer log file
-    setup_logging(logging.CRITICAL, logging.ERROR, log_file=cfg.timer_log_file)
     http.configure_web_proxy(cfg.http_proxy, cfg.https_proxy)
 
     run_jobs(cfg=cfg, current_time=current_time)
