@@ -1,16 +1,14 @@
 package File::Glob;
 
 use strict;
-our($VERSION, @ISA, @EXPORT_OK, @EXPORT_FAIL, %EXPORT_TAGS, $DEFAULT_FLAGS);
+our($DEFAULT_FLAGS);
 
 require XSLoader;
 
-@ISA = qw(Exporter);
-
 # NOTE: The glob() export is only here for compatibility with 5.6.0.
 # csh_glob() should not be used directly, unless you know what you're doing.
 
-%EXPORT_TAGS = (
+our %EXPORT_TAGS = (
     'glob' => [ qw(
         GLOB_ABEND
         GLOB_ALPHASORT
@@ -33,9 +31,9 @@ require XSLoader;
 );
 $EXPORT_TAGS{bsd_glob} = [@{$EXPORT_TAGS{glob}}];
 
-@EXPORT_OK   = (@{$EXPORT_TAGS{'glob'}}, 'csh_glob');
+our @EXPORT_OK   = (@{$EXPORT_TAGS{'glob'}}, 'csh_glob');
 
-$VERSION = '1.33';
+our $VERSION = '1.40';
 
 sub import {
     require Exporter;
@@ -66,7 +64,7 @@ sub import {
 XSLoader::load();
 
 $DEFAULT_FLAGS = GLOB_CSH();
-if ($^O =~ /^(?:MSWin32|VMS|os2|dos|riscos)$/) {
+if ($^O =~ /^(?:MSWin32|VMS|os2|riscos)$/) {
     $DEFAULT_FLAGS |= GLOB_NOCASE();
 }
 
