@@ -16,12 +16,17 @@ import re
 
 import apport.fileutils
 import apport.hookutils
+import apport.ui
+from problem_report import ProblemReport
 
 
-def add_info(report, ui):
-    nm = apport.hookutils.nonfree_kernel_modules()
-    if nm:
-        report["NonfreeKernelModules"] = " ".join(nm)
+def add_info(report: ProblemReport, ui: apport.ui.HookUI) -> None:
+    """Add generic, general information to the problem report."""
+    # TODO: Split into smaller functions/methods
+    # pylint: disable=too-many-branches,too-many-locals
+    nonfree_modules = apport.hookutils.nonfree_kernel_modules()
+    if nonfree_modules:
+        report["NonfreeKernelModules"] = " ".join(nonfree_modules)
 
     # check for low space
     mounts = {"/": "system", "/var": "/var", "/tmp": "/tmp"}
@@ -29,16 +34,16 @@ def add_info(report, ui):
     home = os.getenv("HOME")
     if home:
         mounts[home] = "home"
-    treshold = 50
+    threshold = 50
 
     for mount, mount_name in mounts.items():
         try:
-            st = os.statvfs(mount)
+            stat = os.statvfs(mount)
         except FileNotFoundError:
             continue
-        free_mb = st.f_bavail * st.f_frsize / 1000000
+        free_mb = stat.f_bavail * stat.f_frsize / 1000000
 
-        if free_mb < treshold:
+        if free_mb < threshold:
             report["UnreportableReason"] = (
                 f"Your {mount_name} partition has less than {free_mb} MB"
                 f" of free space available, which leads to problems using"
@@ -62,9 +67,7 @@ def add_info(report, ui):
     # using local libraries?
     if "ProcMaps" in report:
         local_libs = set()
-        for lib in re.finditer(
-            r"\s(/[^ ]+\.so[.0-9]*)$", report["ProcMaps"], re.M
-        ):
+        for lib in re.finditer(r"\s(/[^ ]+\.so[.0-9]*)$", report["ProcMaps"], re.M):
             if not apport.fileutils.likely_packaged(lib.group(1)):
                 local_libs.add(lib.group(1))
         if ui and local_libs:
@@ -82,25 +85,20 @@ Do you want to continue the report process anyway?
             ):
                 raise StopIteration
             report["LocalLibraries"] = " ".join(local_libs)
-            report["Tags"] = (report.get("Tags", "") + " local-libs").strip()
+            report.add_tags(["local-libs"])
 
     # using third-party packages?
     if "[origin:" in report.get("Package", "") or "[origin:" in report.get(
         "Dependencies", ""
     ):
-        report["Tags"] = (
-            report.get("Tags", "") + " third-party-packages"
-        ).strip()
+        report.add_tags(["third-party-packages"])
 
     # using ecryptfs?
     if os.path.exists(os.path.expanduser("~/.ecryptfs/wrapped-passphrase")):
         report["EcryptfsInUse"] = "Yes"
 
     # filter out crashes on missing GLX (LP#327673)
-    in_gl = (
-        "/usr/lib/libGL.so"
-        in (report.get("StacktraceTop") or "\n").splitlines()[0]
-    )
+    in_gl = "/usr/lib/libGL.so" in (report.get("StacktraceTop") or "\n").splitlines()[0]
     if in_gl and "Loading extension GLX" not in apport.hookutils.read_file(
         "/var/log/Xorg.0.log"
     ):
@@ -125,7 +123,7 @@ Do you want to continue the report process anyway?
 
 
 if __name__ == "__main__":
-    r = {}
-    add_info(r, None)
+    r = ProblemReport()
+    add_info(r, apport.ui.NoninteractiveHookUI())
     for key, value in r.items():
         print(f"{key}: {value}")
